; NAME:
;    filtcoverage
;
; PURPOSE:
;     To determine the coverage of a filter profile and spectrum.
;       
; CATEGORY:
;     Data Reduction
;
; CALLING SEQUENCE:
;     result = filtcoverage(wspec,fspec,wfilt,tfilt,CANCEL=cancel)
;
; INPUTS:
;     wspec  - Wavelength array of the spectrum
;     fspec  - Flux array of the spectrum
;     wfilt  - Wavelength array of the filter (units same as wspec)
;     tfilt  - Flux array of the filter transmission
;
; OUTUTS:
;     Returns the result.
;     0 - Full coverage
;     1 - Partial coverage
;     2 - No coverage.
;
; KEYWORD PARAMETERS:    
;     CANCEL   - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     Later
;
;REVISION HISTORY:
;     2002-02-12 - Written by M. Cushing, Institute for Astronomy, UH
;
function filtcoverage,wspec,fspec,wfilt,tfilt,CANCEL=cancel

cancel = 0

if n_params() lt 4 then begin

    print, 'Syntax - filtcoverage,wspec,fspec,wflit,tflit,CANCEL=cancel'
    cancel = 1
    return,-1

endif

zparcheck, 'filtcoverage', wspec, 1, [1,2,3,4,5],1, 'Wspec'
zparcheck, 'filtcoverage', fspec, 2, [1,2,3,4,5],1, 'Fspec'
zparcheck, 'filtcoverage', wfilt, 3, [1,2,3,4,5],1, 'Wfilt'
zparcheck, 'filtcoverage', tfilt, 4, [1,2,3,4,5],1, 'Tfilt'

frange = [min(wfilt,max=max),max]
srange = [min(wspec,max=max),max]

if frange[0] gt srange[0] and frange[1] lt srange[1] then result = 0

if frange[0] lt srange[0] and frange[1] lt srange[1] then result = 1

if frange[0] gt srange[0] and frange[1] gt srange[1] then result = 1

if frange[0] gt srange[1] then result = 2

if frange[1] lt srange[0] then result = 2

return, result

end
