; NAME:
;     xcentroid
;
; PURPOSE:
;     
;       
; CALLING SEQUENCE:
;     xcentroid
;
; INPUTS:
;     x      - A wavelength array
;     y      - A flux array
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xcentroid_initcommon

common xcentroid_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.
;  p - contains info pertaining to the plots.

w = {contmin_fld:[0L,0L],$
     contmax_fld:[0L,0L],$
     centroid:0L,$
     xcentroid_base:0L,$
     keyboard:0L,$
     linename_fld:[0L,0L],$
     momentlog:0L,$
     ofile_fld:[0L,0L],$
     plotwin_base:0L,$
     plotwin1:0,$
     rwave_fld:[0L,0L],$
     track:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin_fld:[0L,0L],$
     ymax_fld:[0L,0L]}

r = {linereg:[!values.f_nan,!values.f_nan],$
     centroid:!values.f_nan,$
     cursormode:'None'}

d = {x:ptr_new(fltarr(2)),$
     y:ptr_new(fltarr(2))}

p = {absxrange1:[0.,0.],$
     absyrange1:[0.,0.],$
     buffer:[0.,0.],$
     pixmap1_wid:0L,$
     plotwin1_wid:0L,$
     xrange1:[0.,0.],$
     yrange1:[0.,0.],$
     plot1size:[700,400],$
     pscale1:!p,$
     xscale1:!x,$
     xtitle:'',$
     yscale1:!y,$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}


;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

end
;
;******************************************************************************
;
pro xcentroid_centroid

common xcentroid_state

z = where(*state.d.x ge state.r.linereg[0] and $
          *state.d.x le state.r.linereg[1])


state.r.centroid = double( total((*state.d.x)[z]*(*state.d.y)[z],/DOUBLE)/$
  total((*state.d.y)[z],/DOUBLE) )

widget_control, state.w.centroid,SET_VALUE='Centroid: '+$
  strtrim(state.r.centroid,2)

end
;
;******************************************************************************
;
pro xcentroid_help

common xcentroid_state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xcentroid_help') then begin

    help_base = widget_base(group_leader = state.w.xcentroid_base, $
                            /column, $
                            title ='X Line Analysis Tool - Help')
    
    h = [['X Line Analysis Tool is a fully resizing widget.'],$
         [' '],$
         ['Keyboard commands:'],$
         [' '],$
         ['c - Clear mouse mode.'],$
         ['    Use to clear a zoom, fix, or remove session.'],$
         [' '],$
         ['i - To zoom IN in whatever zoom mode the cursor is currently'],$
         ['    in.'],$
         [' '],$
         ['h - To lauch the help window.'],$
         [' '],$
         ['o - To zoom OUT in whatever zoom mode the cursor is currently'],$
         ['    in.'],$
         [' '],$
         ['w - To plot the entire spectrum'],$
         [' '],$
         ['x - Enters x zoom mode'],$
         ['    Press left mouse button at lower x value and then at upper'],$
         ['    x value.'],$
         [' '],$
         ['y - Enters y zoom mode'],$
         ['    Press left mouse button at lower y value and then at upper'],$
         ['    y value.'],$
         [' '],$
         ['z - Enters zoom mode'],$
         ['    Press the left mouse button in one corner of the zoom box '],$
         ['    and then move the cursor to the other corner and press the '],$
         ['    the left mouse button.'],$
         [' ']]

    help_text = widget_text(help_base, $
                            /scroll, $
                            value = h, $
                            xsize = 70, $
                            ysize = 24)
    
    quit = widget_button(help_base,$
                         value='Done',$
                         font=buttonfont,$
                         uvalue='Done')
      
    centertlb,help_base

    widget_control, help_base, /Realize
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xzoomplot_help', $
      help_base, $
      /No_Block,$
      event_handler='xcentroid_event'
    
endif

end
;
;******************************************************************************
;
pro xcentroid_plotspec

common xcentroid_state

plot, *state.d.x,*state.d.y,/xsty,/ysty,xrange=state.p.xrange1,$
  yrange=state.p.yrange1,psym=10

;  Plot Line Region if necessary

z = where(finite(state.r.linereg) eq 1,count)
for i = 0, count-1 do begin

    plots, [state.r.linereg[z[i]],state.r.linereg[z[i]]],state.p.yrange1,$
      color=7,linestyle=2

endfor

;  Plot the centroid if necessary

if finite(state.r.centroid) eq 1 then plots, $
  [state.r.centroid,state.r.centroid],!y.crange,color=3,linestyle=1,thick=2



state.p.pscale1 = !p
state.p.xscale1 = !x
state.p.yscale1 = !y

end
;
;******************************************************************************
;
pro xcentroid_plotupdate

common xcentroid_state

wset, state.p.pixmap1_wid
xcentroid_plotspec


wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

end
;
;******************************************************************************
;
pro xcentroid_setminmax

common xcentroid_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.xrange1[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.xrange1[1],2)
widget_control, state.w.ymin_fld[1],set_value=strtrim(state.p.yrange1[0],2)
widget_control, state.w.ymax_fld[1],set_value=strtrim(state.p.yrange1[1],2)

end
;
;******************************************************************************
;
pro xcentroid_zoom,IN=in,OUT=out

common xcentroid_state

delabsx = state.p.absxrange1[1]-state.p.absxrange1[0]
delx    = state.p.xrange1[1]-state.p.xrange1[0]

delabsy = state.p.absyrange1[1]-state.p.absyrange1[0]
dely    = state.p.yrange1[1]-state.p.yrange1[0]

xcen = state.p.xrange1[0]+delx/2.
ycen = state.p.yrange1[0]+dely/2.

case state.r.cursormode of 

    'XZoom': begin

        z = alog10(delabsx/delx)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsx/2.^z/2.
        state.p.xrange1 = [xcen-hwin,xcen+hwin]
        xcentroid_plotupdate

    end

    'YZoom': begin

        z = alog10(delabsy/dely)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsy/2.^z/2.
        state.p.yrange1 = [ycen-hwin,ycen+hwin]
        xcentroid_plotupdate

    end

    'Zoom': begin

        z = alog10(delabsx/delx)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsx/2.^z/2.
        state.p.xrange1 = [xcen-hwin,xcen+hwin]

        z = alog10(delabsy/dely)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsy/2.^z/2.
        state.p.yrange1 = [ycen-hwin,ycen+hwin]

        xcentroid_plotupdate

    end

    else:

endcase

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xcentroid_event, event

common xcentroid_state

widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Clear': begin

        for i = 2, 13 do state.d.lineinfo[state.r.lineidx].(i) = !values.f_nan
        widget_control, state.w.momentlog,SET_VALUE=strarr(13)
        widget_control, state.w.contmin_fld[1],SET_VALUE=''
        widget_control, state.w.contmax_fld[1],SET_VALUE=''
        state.r.linereg[*] = !values.f_nan
        xcentroid_plotupdate

    end

    'Compute Moments': xcentroid_moments

    'Cont Min': begin

        val = cfld(state.w.contmin_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.r.linereg[0] = val
        xcentroid_plotupdate

    end

    'Cont Max': begin

        val = cfld(state.w.contmax_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.r.linereg[1] = val
        xcentroid_plotupdate

    end

    'Done': widget_control, event.top, /DESTROY

    'Keyboard': begin

        case strtrim(event.ch,2) of 
            
            'c': begin

                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xcentroid_plotupdate
                
            end

            'i': xcentroid_zoom,/IN

            'h': xcentroid_help ; Help

            'o': xcentroid_zoom,/OUT

            's': begin

                state.r.cursormode = 'Select'
                state.p.reg[*] = !values.f_nan
                state.r.linereg[*] = !values.f_nan
                state.r.centroid = !values.f_nan
                xcentroid_plotupdate

            end

            'w': begin
                
                state.p.xrange1 = [min((*state.d.x),max=max),max]
                state.p.yrange1 = [0,max(*state.d.y,/NAN)]
                xcentroid_plotupdate

            end

            'x': begin 

                state.r.cursormode = 'XZoom'
                state.p.reg[*] = !values.f_nan
                                
            end

            'y': begin 

                state.r.cursormode = 'YZoom'
                state.p.reg[*] = !values.f_nan

            end

            'z': begin

                state.r.cursormode = 'Zoom'
                state.p.reg[*] = !values.f_nan
               
            end

            '?': xcentroid_help ; Help
            
            else:

        endcase

    end

    'Line Number': begin

        state.r.lineidx = event.index
        if finite(state.d.lineinfo[state.r.lineidx].(1)) eq 0 then begin

            widget_control, state.w.momentlog,SET_VALUE=strarr(13)
            widget_control, state.w.contmin_fld[1],SET_VALUE=''
            widget_control, state.w.contmax_fld[1],SET_VALUE=''
            widget_control, state.w.linename_fld[1],SET_VALUE=''
            widget_control, state.w.rwave_fld[1], SET_VALUE=''
            state.r.linereg[*] = !values.f_nan
            xcentroid_plotupdate
            
        endif else begin

            n = n_tags(state.d.lineinfo[state.r.lineidx])
            str = strarr(n)
            for i = 0, n-1 do str[i] = $
              strtrim(state.d.lineinfo[state.r.lineidx].(i),2)
            
            widget_control, state.w.linename_fld[1],SET_VALUE=str[0]
            widget_control, state.w.rwave_fld[1],SET_VALUE=str[1]
            widget_control, state.w.contmin_fld[1],SET_VALUE=str[2]
            widget_control, state.w.contmax_fld[1],SET_VALUE=str[3]

            widget_control, state.w.momentlog,set_value=[str[0],' ',$
                                                         str[1:3],' ',$
                                                         str[4:7],' ',$
                                                         str[8:11],$
                                                         ' ',str[12:13]]

            state.r.linereg = [float(str[2]),float(str[3])]
            xcentroid_plotupdate
        
        endelse

    end

    'Write File': xcentroid_writefile
    
    else:

endcase

cont: 

end
;
;******************************************************************************
;
pro xcentroid_minmax_event,event

common xcentroid_state

xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmin2 = crange(xmin,state.p.xrange1[1],'X Min',/KLT,$
               WIDGET_ID=state.w.xcentroid_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmin_fld[0],set_value=state.p.xrange1[0]
    return

endif else state.p.xrange1[0] = xmin2

xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmax2 = crange(xmax,state.p.xrange1[0],'X Max',/KGT,$
               WIDGET_ID=state.w.xcentroid_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmax_fld[0],set_value=state.p.xrange1[1]
    return

endif else state.p.xrange1[1] = xmax2

ymin = cfld(state.w.ymin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymin2 = crange(ymin,state.p.yrange1[1],'Y Min',/KLT,$
               WIDGET_ID=state.w.xcentroid_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymin_fld[0],set_value=state.p.yrange1[0]
    return

endif else state.p.yrange1[0] = ymin2

ymax = cfld(state.w.ymax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymax2 = crange(ymax,state.p.yrange1[0],'Y Max',/KGT,$
               WIDGET_ID=state.w.xcentroid_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymax_fld[0],set_value=state.p.yrange1[1]
    return

endif else state.p.yrange1[1] = ymax2

xcentroid_plotupdate


end
;
;******************************************************************************
;
pro xcentroid_plotwinevent1, event

common xcentroid_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    wset, state.p.plotwin1_wid
    erase
    device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1
x  = event.x/float(state.p.plot1size[0])
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

case state.r.cursormode of 

    'Select': begin

        if event.type ne 1 then goto, out        
        z = where(finite(state.r.linereg) eq 1,count)        
        if count eq 0 then begin
            
            state.r.linereg[0] = xy[0]
            xcentroid_plotupdate

        endif
        if count eq 1 then begin

            state.r.linereg[1] = xy[0]
            state.r.linereg = state.r.linereg[sort(state.r.linereg)]
            xcentroid_centroid
            xcentroid_plotupdate

        endif

    end

    'XZoom': begin
        
        if event.type ne 1 then goto, out        
        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then begin
            
            state.p.reg[*,0] = xy[0:1]
            wset, state.p.pixmap1_wid
            plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,$
              /DEVICE,linestyle=1,thick=2
            wset, state.p.plotwin1_wid
            device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                          state.p.pixmap1_wid]
            
        endif else begin
            
            state.p.reg[*,1]   = xy[0:1]
            state.p.xrange1    = [min(state.p.reg[0,*],max=m),m]
            state.p.reg[*]     = !values.f_nan
            state.r.cursormode = 'None'
            xcentroid_plotupdate
            xcentroid_setminmax
            
        endelse
        
        
    end
    
    'YZoom': begin
        
        if event.type ne 1 then goto, out        
        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then begin
            
            state.p.reg[*,0] = xy[0:1]
            wset, state.p.pixmap1_wid
            plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,$
              /DEVICE,linestyle=1,thick=2
            
            wset, state.p.plotwin1_wid
            device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                          state.p.pixmap1_wid]
            
        endif else begin
            
            state.p.reg[*,1]   = xy[0:1]
            state.p.yrange1    = [min(state.p.reg[1,*],max=m),m]
            state.r.cursormode = 'None'
            state.p.reg[*]     = !values.f_nan
            xcentroid_plotupdate
            xcentroid_setminmax
            
        endelse
        
    end
    
    'Zoom': begin

        if event.type ne 1 then goto, out        
        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin
            
            state.p.reg[*,1] = xy[0:1]
            state.p.xrange1 = [min(state.p.reg[0,*],max=max),max]
            state.p.yrange1 = [min(state.p.reg[1,*],max=max),max]
            state.r.cursormode = 'None'
            state.p.reg[*] = !values.f_nan
            xcentroid_plotupdate
            xcentroid_setminmax
            
        endelse
        
    end

    else:

endcase

out:

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]


case state.r.cursormode of 

    'XZoom': plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

    'YZoom': plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE

    'Zoom': begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
        xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
        plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
          linestyle=2,color=2
        plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
          linestyle=2,color=2
        
    end

    else: begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE

    end

endcase

tabinv, *state.d.x,xy[0],idx
idx = round(idx)
label = 'Cursor X: '+strtrim(xy[0],2)+', Y:'+strtrim(xy[1],2)
label = label+'   Spectrum X: '+strtrim( (*state.d.x)[idx],2)+$
  ', Y:'+strtrim( (*state.d.y)[idx],2)
widget_control,state.w.track,set_value=label

cont:
    
end
;
;******************************************************************************
;
pro xcentroid_resize,event

common xcentroid_state

widget_control, state.w.plotwin_base, tlb_get_size = size

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=size[1]-state.p.buffer[1]

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

xcentroid_plotupdate

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xcentroid,x,y,xunits,yunits,YTITLE=ytitle,XTITLE=xtitle,$
            NLINES=nlines,CANCEL=cancel

common xcentroid_state

if not xregistered('xcentroid') then begin
    
    xcentroid_initcommon

;  Build the widget.

    buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
    textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'

    state.w.xcentroid_base = widget_base(TITLE='Xcentroid',$
                                    /COLUMN,$
                                    /TLB_SIZE_EVENTS)

       button = widget_button(state.w.xcentroid_base,$
                              EVENT_PRO='xcentroid_event',$
                              VALUE='Done',$
                              UVALUE='Done',$
                              FONT=buttonfont)
    
       state.w.track = widget_text(state.w.xcentroid_base, $
                                   YSIZE=1)

       state.w.centroid = widget_label(state.w.xcentroid_base,$
                                       /ALIGN_LEFT,$
                                       /DYNAMIC_RESIZE,$
                                       VALUE='Centroid: '+$
                                       strtrim(state.r.centroid,2),$
                                       FONT=buttonfont)
       
       state.w.keyboard = widget_text(state.w.xcentroid_base, $
                                      /ALL_EVENTS, $
                                      SCR_XSIZE=1, $
                                      SCR_YSIZE=1, $
                                      UVALUE='Keyboard', $
                                      EVENT_PRO='xcentroid_event',$
                                      VALUE= '')
       
       state.w.plotwin1 = widget_draw(state.w.xcentroid_base,$
                                      XSIZE=state.p.plot1size[0],$
                                      YSIZE=state.p.plot1size[1],$
                                      /TRACKING_EVENTS,$
                                      /BUTTON_EVENTS,$
                                      /MOTION_EVENTS,$
                                      EVENT_PRO='xcentroid_plotwinevent1',$
                                      UVALUE='Plot Window 1')
       
       row_base = widget_base(state.w.xcentroid_base,$
                              FRAME=1,$
                              EVENT_PRO='xcentroid_event',$
                              /ROW)
       
          xmin = coyote_field2(row_base,$
                               labelfont=buttonfont,$
                               fieldfont=textfont,$
                               title='X Min:',$
                               uval = 'X Min',$
                               xsize=12,$
                               event_pro = 'xcentroid_minmax_event',$
                               /cr_only,$
                               textid=textid)
          state.w.xmin_fld = [xmin,textid]
          
          xmax = coyote_field2(row_base,$
                               labelfont=buttonfont,$
                               fieldfont=textfont,$
                               title='X Max:',$
                               uval = 'X Max',$
                               xsize=12,$
                               event_pro = 'xcentroid_minmax_event',$
                               /cr_only,$
                               textid=textid)
          state.w.xmax_fld = [xmax,textid]
          
          ymin = coyote_field2(row_base,$
                               labelfont=buttonfont,$
                               fieldfont=textfont,$
                               title='Y Min:',$
                               uval = 'Y Min',$
                               xsize=12,$
                               event_pro = 'xcentroid_minmax_event',$
                               /cr_only,$
                               textid=textid)
          state.w.ymin_fld = [ymin,textid]
          
          ymax = coyote_field2(row_base,$
                               labelfont=buttonfont,$
                               fieldfont=textfont,$
                               title='Y Max:',$
                               uval = 'Y Max',$
                               xsize=12,$
                               event_pro = 'xcentroid_minmax_event',$
                               /cr_only,$
                               textid=textid)
          state.w.ymax_fld = [ymax,textid]


; Start the Event Loop. This will be a non-blocking program.

; Get things running.  Center the widget using the Fanning routine.

    mkct      
    centertlb,state.w.xcentroid_base
    widget_control, state.w.xcentroid_base, /REALIZE
    
;  Get plotwin ids
    
    widget_control, state.w.plotwin1, GET_VALUE=xx
    state.p.plotwin1_wid = xx
    
    window, /free, /pixmap,xsize=state.p.plot1size[0],$
      ysize=state.p.plot1size[1]
    state.p.pixmap1_wid = !d.window
    
;  Get sizes for things.
    
    widget_geom = widget_info(state.w.xcentroid_base, /geometry)
    state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
    state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]
    
; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xcentroid', $
      /NO_BLOCK,$
      state.w.xcentroid_base,$
      event_handler='xcentroid_resize'


          
;  Load data

    *state.d.x         = x
    *state.d.y         = y
    state.p.absxrange1 = [min(x,max=max),max]
    state.p.absyrange1 = [0,max(y,/NAN)]
    state.p.xrange1    = state.p.absxrange1
    state.p.yrange1    = state.p.absyrange1

    xcentroid_plotupdate
    xcentroid_setminmax

endif

end








