; NAME:
;     xcleanspec
;
; PURPOSE:
;     To clean SpeX spectra.
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xcleanspec
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     Allows the user to either remove bad pixels or replace bad pixels.
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xcleanspec_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin

    ptr_free, *state.r.orders
    ptr_free, *state.d.hdr
    ptr_free, *state.d.espec
    ptr_free, *state.d.ospec
    ptr_free, *state.d.pspec
    ptr_free, *state.p.reg

endif

state = 0B

end
;
;******************************************************************************
;
pro xcleanspec_help,base

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xcleanspec_help') then begin

    help_base = widget_base(group_leader=base,$
                            /column, $
                            title ='Xcleanspec Help')
    
    h = [['Xcleanspec is a fully resizing widget.'],$
         [' '],$
         ['Keyboard commands:'],$
         [' '],$
         ['c - Clear mouse mode.'],$
         ['    Use to clear a zoom, fix, or remove session.'],$
         [' '],$
         ['f - To fix a region of the spectrum.'],$
         ['    Click the left mouse button on the left and right side of'],$
         ['    the bad region.  The bad region is replaced using the edge'],$
         ['    pixels to define a line.  Type "s" to save the edit.'],$
         [' '],$
         ['h - To lauch the help window.'],$
         [' '],$
         ['o - To plot the original spectrum.  All edits will be lost.'],$
         [' '],$
         ['r - To remove a region of the spectrum.'],$
         ['    Click the left mouse button on the left and right side of'],$
         ['    the bad region.  The bad region is replaced with NaNs'],$
         ['    Type "s" to save the edit'],$
         [' '],$
         ['s - To save the edit'],$
         [' '],$
         ['u - To undo an edit.  The edit will not be stored.'],$
         [' '],$
         ['w - To plot the entire spectrum'],$
         [' '],$
         ['x - Enters x zoom mode'],$
         ['    Press left mouse button at lower x value and then at upper'],$
         ['    x value.'],$
         ['y - Enters y zoom mode'],$
         ['    Press left mouse button at lower y value and then at upper'],$
         ['    y value.'],$
         ['z - Enters zoom mode'],$
         ['    Press the left mouse button in one corner of the zoom box '],$
         ['    and then move the cursor to the other corner and press the '],$
         ['    the left mouse button.'],$
         [' ']]

    help_text = widget_text(help_base, $
                            /scroll, $
                            value = h, $
                            xsize = 70, $
                            ysize = 24)
    
    quit = widget_button(help_base,$
                         value='Done',$
                         font=buttonfont,$
                         uvalue='Done')
      
    centertlb,help_base

    widget_control, help_base, /Realize
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xcleanspec_help', $
      help_base, $
      /No_Block,$
      event_handler='xcleanspechelp_event'
    
endif

end
;
;******************************************************************************
;
pro xcleanspec_loadspec,state

;  Get files.

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

specfile = cfld(state.w.ispectrum_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
specfile = cfile(path+specfile,WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
if cancel then return

;  Read standard spectra.

readspec,specfile,spc,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
  airmass

if norders ne 1 or naps ne 1 then begin

    ok = dialog_message([['Please use xcleanspecxd'],$
                         ['for multi-order spectra.']],/error,$
                        dialog_parent=state.w.xcleanspec_base)
    goto, out

endif

;  Save info

*state.d.ospec  = spc
*state.d.espec  = spc
*state.d.pspec  = spc
*state.d.hdr    = hdr          
*state.r.orders = orders
state.r.norders = norders
state.r.naps    = naps

state.p.order = orders[0]
state.p.ap    = 1
state.p.idx   = 0

state.p.xunits = xunits
state.p.yunits = yunits

;  Get Plot rannge.

state.p.plot1xrange = [min(spc[*,0],max=max),max]
state.p.plot1yrange = [0,max(spc[*,1],/NAN)]
state.p.plot2yrange = [0,max(spc[*,2],/NAN)]

widget_control, state.w.plotwin1, DRAW_BUTTON_EVENTS=1
widget_control, state.w.plotwin2, DRAW_BUTTON_EVENTS=1

xcleanspec_plotupdate,state
xcleanspec_setminmax,state

out:

end
;
;******************************************************************************
;
pro xcleanspec_ploterror,state


xtitle ='!5Wavelength ('+strtrim(state.p.xunits,2)+')'
ytitle ='!5Error ('+strtrim(state.p.yunits,2)+')'

z = where(finite((*state.d.pspec)[*,0]) eq 1)

plot,(*state.d.pspec)[z,0],(*state.d.pspec)[z,2],psym=10,/xsty,/ysty,$
  xrange=state.p.plot1xrange,yrange=state.p.plot2yrange,xtitle=xtitle,$
  ytitle=ytitle

state.p.pscale2 = !p
state.p.xscale2 = !x
state.p.yscale2 = !y


end
;
;******************************************************************************
;
pro xcleanspec_plotflux,state

xtitle ='!5Wavelength ('+strtrim(state.p.xunits,2)+')'
ytitle ='!5Flux ('+strtrim(state.p.yunits,2)+')'

z = where(finite((*state.d.pspec)[*,0]) eq 1)
wave = (*state.d.pspec)[z,0]
flux = (*state.d.pspec)[z,1]

plot,wave,flux,psym=10,/xsty,/ysty,$
  xrange=state.p.plot1xrange,yrange=state.p.plot1yrange,xtitle=xtitle,$
  ytitle=ytitle,position=[0.1,0.1,0.95,0.88]

if state.r.plotlines then begin

;  Label H lines
    
    z = where(state.d.hlines lt state.p.plot1xrange[1] and $
              state.d.hlines gt state.p.plot1xrange[0],count)
    
    
    for i =0, count-1 do begin
        
        mctabinv,wave,(state.d.hlines)[z[i]],idx
        plots, [(state.d.hlines)[z[i]],(state.d.hlines)[z[i]]],$
          [flux[idx],!y.crange[1]],linestyle=1,color=3,thick=2
        
        xy = convert_coord((state.d.hlines)[z[i]],1,/DATA,/TO_NORM)
        name = '!5'+(state.d.hnames)[z[i]]
        xyouts, xy[0],0.88,name,orientation=90,/NORM,$
          charsize=str_size(name,0.2),color=3
        
    endfor

endif

state.p.pscale1 = !p
state.p.xscale1 = !x
state.p.yscale1 = !y

end
;
;******************************************************************************
;
pro xcleanspec_plotupdate,state

wset, state.p.pixmap1_wid
erase
xcleanspec_plotflux,state

wset, state.p.plotwin1_wid
erase
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]


wset, state.p.pixmap2_wid
erase 
xcleanspec_ploterror,state


wset, state.p.plotwin2_wid
erase
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]


end
;
;******************************************************************************
;
pro xcleanspec_setminmax,state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.plot1xrange[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.plot1xrange[1],2)
widget_control, state.w.ymin1_fld[1],$
  set_value=strtrim(state.p.plot1yrange[0],2)
widget_control, state.w.ymax1_fld[1],$
  set_value=strtrim(state.p.plot1yrange[1],2)

widget_control, state.w.ymin2_fld[1],$
  set_value=strtrim(state.p.plot2yrange[0],2)
widget_control, state.w.ymax2_fld[1],$
  set_value=strtrim(state.p.plot2yrange[1],2)

end
;
;******************************************************************************
;
pro xcleanspec_writefile,state

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return
file = cfld(state.w.oname_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

if state.r.fitsoutput then writefits, path+file+'.fits',*state.d.espec,$
  *state.d.hdr

if state.r.textoutput then begin

    npix = fxpar(*state.d.hdr,'NAXIS1')
    openw,lun,path+file+'.txt', /get_lun
    
    for i = 0, n_elements(*state.d.hdr)-1 do printf, lun, (*state.d.hdr)[i]
    
    for i = 0, npix-1 do begin
        
        printf, lun,  strjoin( reform((*state.d.espec)[i,*],3),'  ')
        
    endfor
    close, lun
    free_lun, lun
    
endif    

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xcleanspec_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of

    'Done': begin

        widget_control, event.top, /DESTROY

    end

    'Input Spectrum': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        if path ne '' then path = cpath(path,$
                                        WIDGET_ID=state.w.xcleanspec_base,$
                                        CANCEL=cancel)
        if cancel then return

        obj = dialog_pickfile(DIALOG_PARENT=state.w.xcleanspec_base,$
                              PATH=path,/MUST_EXIST,FILTER='*.fits')
        if obj eq '' then goto, cont
        widget_control,state.w.ispectrum_fld[1], $
          SET_VALUE = strmid(obj[0],strpos(obj,'/',/REVERSE_S)+1)
        setfocus,state.w.ispectrum_fld

    end

    'Hydrogen Lines': begin

        state.r.plotlines = event.select
        xcleanspec_plotupdate,state

    end

    'Keyboard': begin

        case strtrim(event.ch,2) of 
            
            'c': begin
                
                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xcleanspec_plotupdate,state
            
            end

            'f': begin ; Fix

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'Fix'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'h': xcleanspec_help,state.w.xcleanspec_base ; Help

            'o': begin ; Original Spectrum

                *state.d.espec = *state.d.ospec
                *state.d.pspec = *state.d.espec
                xcleanspec_plotupdate,state

            end

            'r': begin ; Remove

                if state.r.cursormode eq 'None' then begin
                    
                    state.r.cursormode = 'Remove'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end
            
            's': *state.d.espec = *state.d.pspec

            'u': begin ; Undo

                *state.d.pspec = *state.d.espec
                state.p.reg[*] = !values.f_nan
                xcleanspec_plotupdate,state

            end

            'w': begin

                
                state.p.plot1xrange = [min((*state.d.espec)[*,0],max=max),max]
                state.p.plot1yrange = [0,max((*state.d.espec)[*,1],/NAN)]
                state.p.plot2yrange = [0,max((*state.d.espec)[*,2],/NAN)] 
                xcleanspec_plotupdate,state

            end

            'x': begin 

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'XZoom'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'y': begin 

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'YZoom'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'z': begin

                state.r.cursormode = 'Zoom'
                state.p.reg[*] = !values.f_nan
                
            end

            '?': xcleanspec_help,state.w.xcleanspec_base ; Help
        
            else:

        endcase

    end

    'Load Spectrum': xcleanspec_loadspec,state

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select

    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xcleanspec_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld
            
        endif
    end


    'Quit': begin

        widget_control, event.top, /DESTROY
        goto, getout

    end
    
    'Write File': xcleanspec_writefile,state

    else:

endcase

cont: 

widget_control, state.w.xcleanspec_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
pro xcleanspec_plotwinevent1, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    wset, state.p.plotwin1_wid
    device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                  state.p.pixmap1_wid]

    wset, state.p.plotwin2_wid
    device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                  state.p.pixmap2_wid]
    
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1
x  = event.x/float(state.p.plot1size[0])
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin

    case state.r.cursormode of 

        'Fix': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1] 

                wset, state.p.plotwin1_wid
                plots,[xy[0],xy[0]],(state.p.yscale1).crange,linesty=1,color=2
                wset, state.p.pixmap1_wid
                plots,[xy[0],xy[0]],(state.p.yscale1).crange,linesty=1,color=2
                
            endif else begin
                
                state.p.reg[*,1] = xy[0:1]
                xrange = reform(state.p.reg[0,sort(state.p.reg[0,*])])
                mctabinv,(*state.d.espec)[*,0,state.p.idx],xrange,idx
                idx = round(idx)
                
                x = [(*state.d.espec)[idx[0],0],(*state.d.espec)[idx[1],0]]
                y = [(*state.d.espec)[idx[0],1],(*state.d.espec)[idx[1],1]]
                e = [(*state.d.espec)[idx[0],2],(*state.d.espec)[idx[1],2]]


                coeff = poly_fit1d(x,y,1,/SILENT)
                (*state.d.pspec)[idx[0]:idx[1],1] = $
                  poly1d((*state.d.espec)[idx[0]:idx[1],0],coeff)

                coeff = poly_fit1d(x,e,1,/SILENT)
                (*state.d.pspec)[idx[0]:idx[1],2] = $
                  poly1d((*state.d.espec)[idx[0]:idx[1],0],coeff)
                
                xcleanspec_plotupdate,state
                state.r.cursormode = 'None'

            endelse

        end

        'Remove': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1] 
                wset, state.p.plotwin1_wid
                plots,[xy[0],xy[0]],(state.p.yscale1).crange,linesty=1,color=2
                wset, state.p.pixmap1_wid
                plots,[xy[0],xy[0]],(state.p.yscale1).crange,linesty=1,color=2
                
            endif else begin
                
                state.p.reg[*,1] = xy[0:1]
                mctabinv,(*state.d.espec)[*,0,state.p.idx],$
                  reform(state.p.reg[0,*]),idx
        
                idx = round(idx)                
                min = idx[0] < idx[1]
                max = idx[0] > idx[1]

                idx = findgen(n_elements((*state.d.pspec)[*,0,0]))
                amin = min(idx,max=amax)
                
                (*state.d.pspec)[min > amin,0] =!values.f_nan
                (*state.d.pspec)[max < amax,0] =!values.f_nan
                
                (*state.d.pspec)[(min-1) > amin,1] = !values.f_nan
                (*state.d.pspec)[(max+1) < amax,1] = !values.f_nan

                (*state.d.pspec)[(min-1) > amin,2] = !values.f_nan
                (*state.d.pspec)[(max+1) < amax,2] = !values.f_nan

                print, n_elements((*state.d.pspec)[*,0])
                z = where(idx lt min or idx gt max)
                (*state.d.pspec) = (*state.d.pspec)[z,*,*]
                print, n_elements((*state.d.pspec)[*,0])

                xcleanspec_plotupdate,state
                state.r.cursormode = 'None'
                
            endelse

        end

        'Zoom': begin
            
            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
                
                state.p.reg[*,1] = xy[0:1]
                state.p.plot1xrange   = [min(state.p.reg[0,*],max=max),max]
                state.p.plot1yrange   = [min(state.p.reg[1,*],max=max),max]
                xcleanspec_plotupdate,state
                xcleanspec_setminmax,state
                state.r.cursormode   = 'None'
                state.p.reg      = [[!values.f_nan,!values.f_nan],$
                                    [!values.f_nan,!values.f_nan]]
                
            endelse

        end

        'XZoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1]
                wset, state.p.pixmap1_wid
                plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,$
                  /DEVICE,linestyle=1,thick=2
                wset, state.p.plotwin1_wid
                device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,$
                              0,state.p.pixmap1_wid]
                wset, state.p.pixmap2_wid
                plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,$
                  /DEVICE,linestyle=1,thick=2
                wset, state.p.plotwin2_wid
                device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,$
                              0,state.p.pixmap2_wid]
               
            endif else begin

                state.p.reg[*,1] = xy[0:1]
                state.p.plot1xrange = [min(state.p.reg[0,*],max=m),m]
                state.r.cursormode = 'None'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xcleanspec_plotupdate,state
                xcleanspec_setminmax,state
                
            endelse


        end

        'YZoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1]
                wset, state.p.pixmap1_wid
                plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,$
                  /DEVICE,linestyle=1,thick=2

                wset, state.p.plotwin1_wid
                device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,$
                              0,state.p.pixmap1_wid]
               
            endif else begin

                state.p.reg[*,1] = xy[0:1]
                state.p.plot1yrange = [min(state.p.reg[1,*],max=m),m]
                state.r.cursormode = 'None'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xcleanspec_plotupdate,state
                xcleanspec_setminmax,state
                
            endelse

        end

        else:

    endcase

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

wset, state.p.plotwin1_wid

case state.r.cursormode of 

    'XZoom': begin

        wset, state.p.plotwin1_wid
        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        wset, state.p.plotwin2_wid
        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

    end
    'YZoom': plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE

    'Zoom': begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
        xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
        plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
          linestyle=2,color=2
        plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
          linestyle=2,color=2
        
    end

    else: begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
        wset, state.p.plotwin2_wid
        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

    end

endcase

cont:

widget_control, state.w.xcleanspec_base, SET_UVALUE=state, /NO_COPY
    
end
;
;******************************************************************************
;
pro xcleanspec_plotwinevent2, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale2
!x = state.p.xscale2
!y = state.p.yscale2
x  = event.x/float(state.p.plot2size[0])
y  = event.y/float(state.p.plot2size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 and state.r.cursormode eq 'YZoom' then begin
            
    z = where(finite(state.p.reg) eq 1,count)
    if count eq 0 then begin
        
        state.p.reg[*,0] = xy[0:1]
        wset, state.p.pixmap2_wid
        plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,$
          /DEVICE,linestyle=1,thick=2
        
        wset, state.p.plotwin2_wid
        device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                      state.p.pixmap2_wid]
        
    endif else begin
        
        state.p.reg[*,1] = xy[0:1]
        state.p.plot2yrange = [min(state.p.reg[1,*],max=m),m]
        state.r.cursormode = 'None'
        state.p.reg = [[!values.f_nan,!values.f_nan],$
                       [!values.f_nan,!values.f_nan]]
        xcleanspec_plotupdate,state
        xcleanspec_setminmax,state
        
    endelse
    
endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

if state.r.cursormode eq 'YZoom' then begin

    plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE

endif else begin

    plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE
    plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

    wset, state.p.plotwin1_wid
    plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

endelse

cont:

widget_control, state.w.xcleanspec_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xcleanspec_minmax,event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of 

    'X Min': begin

        xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmin2 = crange(xmin,state.p.plot1xrange[1],'X Min',/KLT,$
                       WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmin_fld[0],set_value=state.p.plotxrange[0]
            return
            
        endif else state.p.plot1xrange[0] = xmin2

    end
    'X Max': begin

        xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmax2 = crange(xmax,state.p.plot1xrange[0],'X Max',/KGT,$
                       WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmax_fld[0],set_value=state.p.plotxrange[1]
            return
            
        endif else state.p.plot1xrange[1] = xmax2

    end
    'Y1 Min': begin

        ymin = cfld(state.w.ymin1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot1yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin1_fld[0],set_val=state.p.plot1yrange[0]
            return
            
        endif else state.p.plot1yrange[0] = ymin2
        
    end
    'Y1 Max': begin

        ymax = cfld(state.w.ymax1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot1yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax1_fld[0],set_val=state.p.plot1yrange[1]
            return
            
        endif else state.p.plot1yrange[1] = ymax2
        
    end
    'Y2 Min': begin

        ymin = cfld(state.w.ymin2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot2yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin2_fld[0],set_val=state.p.plot2yrange[0]
            return
            
        endif else state.p.plot2yrange[0] = ymin2
        
    end
    'Y2 Max': begin

        ymax = cfld(state.w.ymax2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot2yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xcleanspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax2_fld[0],set_val=state.p.plot2yrange[1]
            return
            
        endif else state.p.plot2yrange[1] = ymax2
        
    end
    
endcase

xcleanspec_plotupdate,state

widget_control, state.w.xcleanspec_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xcleanspec_resize, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

widget_control, state.w.xcleanspec_base, tlb_get_size = size

;  Window 1

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=(size[1]-state.p.buffer[1])*1.4/2.4

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

;  Window 2

state.p.plot2size[0]=size[0]-state.p.buffer[0]
state.p.plot2size[1]=(size[1]-state.p.buffer[1])/2.4

widget_control, state.w.plotwin2, draw_xsize=state.p.plot2size[0]
widget_control, state.w.plotwin2, draw_ysize=state.p.plot2size[1]

wdelete,state.p.pixmap2_wid
window, /free, /pixmap,xsize=state.p.plot2size[0],ysize=state.p.plot2size[1]
state.p.pixmap2_wid = !d.window

xcleanspec_plotupdate,state

widget_control, state.w.xcleanspec_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xcleanspechelp_event,event

widget_control, event.top, /DESTROY

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xcleanspec

if xregistered('xcleanspec') then goto, cont

last   = strpos(!path,'Spantool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
packagepath = strmid(!path,first+1,last-first+7)

readfmt,packagepath+'/data/HI.dat','F8.6,1x,A12',hlines,hnames

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {keyboard:0L,$
     ispectrum_fld:[0L,0L],$
     oname_fld:[0L,0L],$
     order_dl:0L,$
     path_fld:[0L,0L],$
     plotwin1:0,$
     plotwin2:0,$
     xcleanspec_base:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin1_fld:[0L,0L],$
     ymax1_fld:[0L,0L],$
     ymin2_fld:[0L,0L],$
     ymax2_fld:[0L,0L]}

r = {cursormode:'None',$
     fitsoutput:1,$
     naps:0,$
     norders:0,$
     orders:ptr_new(fltarr(2)),$
     plotlines:1,$
     textoutput:0}

d = {hdr:ptr_new(strarr(2)),$
     hlines:hlines,$
     hnames:hnames,$
     espec:ptr_new(fltarr(2)),$
     ospec:ptr_new(fltarr(2)),$
     pspec:ptr_new(fltarr(2))}

p = {activespec:1,$
     ap:0,$
     buffer:[0.,0.],$
     idx:0,$
     order:0,$
     pixmap1_wid:0L,$
     pixmap2_wid:0L,$
     plotwin1_wid:0L,$
     plotwin2_wid:0L,$
     plot1xrange:[0.,0.],$
     plot1yrange:[0.,0.],$
     plot1size:[670,330],$
     plot2yrange:[0.,0.],$
     plot2size:[670,330/1.4],$
     pscale1:!p,$
     xscale1:!x,$
     pscale2:!p,$
     xscale2:!x,$
     xunits:'',$
     yscale1:!y,$
     yscale2:!y,$
     yunits:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}


;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

;  Build the widget.

buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
    
state.w.xcleanspec_base = widget_base(title='Xcleanspec', $
                                      /COLUMN,$
                                      /TLB_SIZE_EVENTS)

   quit_button = widget_button(state.w.xcleanspec_base,$
                               FONT=buttonfont,$
                               EVENT_PRO='xcleanspec_event',$
                               VALUE='Quit',$
                               UVALUE='Quit')
   
   state.w.keyboard = widget_text(state.w.xcleanspec_base, $
                                  /ALL_EVENTS, $
                                  SCR_XSIZE=1, $
                                  SCR_YSIZE=1, $
                                  UVALUE='Keyboard', $
                                  EVENT_PRO='xcleanspec_event',$
                                  VALUE= '')
   
   row_base = widget_base(state.w.xcleanspec_base,$
                          /ROW)

      col1_base = widget_base(row_base,$
                              EVENT_PRO='xcleanspec_event',$
                              /COLUMN)
      
         box1_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
         
            label = widget_label(box1_base,$
                                 VALUE='1.  Load Spectra',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)

            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               button = widget_button(row,$
                                      FONT=buttonfont,$
                                      VALUE='Path',$
                                      UVALUE='Path Button',$
                                      EVENT_PRO='xcleanspec_event')
               
               field = coyote_field2(row,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE=':',$
                                     UVALUE='Path Field',$
                                     XSIZE=25,$
                                     EVENT_PRO='xcleanspec_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
               state.w.path_fld = [field,textid]    
               
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               input = widget_button(row,$
                                     FONT=buttonfont,$
                                     VALUE='Input Spectrum',$
                                     UVALUE='Input Spectrum',$
                                     EVENT_PRO='xcleanspec_event')
               
               input_fld = coyote_field2(row,$
                                         LABELFONT=buttonfont,$
                                         FIELDFONT=textfont,$
                                         TITLE=':',$
                                         UVALUE='Input Spectrum Field',$
                                         XSIZE=15,$
                                         EVENT_PRO='xcleanspec_event',$
                                         /CR_ONLY,$
                                         TEXTID=textid)
               state.w.ispectrum_fld = [input_fld,textid]
               
            load = widget_button(box1_base,$
                                 VALUE='Load Spectrum',$
                                 UVALUE='Load Spectrum',$
                                 FONT=buttonfont)
            
         box4_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
         
            label = widget_label(box4_base,$
                                 VALUE='5.  Write Spectra to File',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)
            
            outformat_bg = cw_bgroup(box4_base,$
                                     font=buttonfont,$
                                     ['FITS','Text'],$
                                     /row,$
                                     /return_name,$
                                     /no_release,$
                                     /nonexclusive,$
                                     label_left='Output Format:',$
                                     UVALUE='Output Format',$
                                     set_VALUE=[1,0])
            
            oname = coyote_field2(box4_base,$
                                  labelfont=buttonfont,$
                                  fieldfont=textfont,$
                                  title='File Name:',$
                                  event_pro = 'xtellcor_event',$
                                  uvalue='Object File Oname',$
                                  /cr_only,$
                                  xsize=18,$
                                  textID=textid)
            state.w.oname_fld = [oname,textid]
            
            write = widget_button(box4_base,$
                                  value='Write File',$
                                  uvalue='Write File',$
                                  font=buttonfont)



          col2_base = widget_base(row_base,$
                                  EVENT_PRO='xcleanspec_event',$
                                  /COLUMN)
          
             hlines_bg = cw_bgroup(col2_base,$
                                   ['Hydrogen Lines'],$
                                   FONT=buttonfont,$
                                   UVALUE='Hydrogen Lines',$
                                   SET_VALUE=[1],$
                                   /NONEXCLUSIVE)

             state.w.plotwin1 = widget_draw(col2_base,$
                                            XSIZE=state.p.plot1size[0],$
                                            YSIZE=state.p.plot1size[1],$
                                            /TRACKING_EVENTS,$
                                            /MOTION_EVENTS,$
                                         EVENT_PRO='xcleanspec_plotwinevent1',$
                                            UVALUE='Plot Window 1')
       
             row_base = widget_base(col2_base,$
                                    /ROW)
   
                xmin = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='X Min:',$
                                     UVALUE='X Min',$
                                     XSIZE=12,$
                                     EVENT_PRO='xcleanspec_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.xmin_fld = [xmin,textid]
                
                xmax = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='X Max:',$
                                     UVALUE='X Max',$
                                     XSIZE=12,$
                                     EVENT_PRO='xcleanspec_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.xmax_fld = [xmax,textid]
                
                ymin = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Min:',$
                                     UVALUE='Y1 Min',$
                                     XSIZE=12,$
                                     EVENT_PRO='xcleanspec_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymin1_fld = [ymin,textid]
                
                ymax = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Max:',$
                                     UVALUE='Y1 Max',$
                                     XSIZE=12,$
                                     EVENT_PRO='xcleanspec_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymax1_fld = [ymax,textid]
                
             state.w.plotwin2 = widget_draw(col2_base,$
                                            XSIZE=state.p.plot2size[0],$
                                            YSIZE=state.p.plot2size[1],$
                                            /TRACKING_EVENTS,$
                                            /MOTION_EVENTS,$
                                         EVENT_PRO='xcleanspec_plotwinevent2',$
                                            UVALUE='Plot Window 2')
          
             row_base = widget_base(col2_base,$
                                    /ROW,$
                                    EVENT_PRO='xcleanspec_event')
          
                ymin = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Min:',$
                                     UVALUE='Y2 Min',$
                                     XSIZE=12,$
                                     EVENT_PRO='xcleanspec_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymin2_fld = [ymin,textid]
                
                ymax = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Max:',$
                                     UVALUE='Y2 Max',$
                                     XSIZE=12,$
                                     EVENT_PRO='xcleanspec_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymax2_fld = [ymax,textid]

; Get things running.  Center the widget using the Fanning routine.
      
    centertlb,state.w.xcleanspec_base
      
    widget_control, state.w.xcleanspec_base, /REALIZE
    
    mkct
    
;  Get plotwin ids
    
    widget_control, state.w.plotwin1, GET_VALUE=x
    state.p.plotwin1_wid = x
    widget_control, state.w.plotwin2, GET_VALUE=x
    state.p.plotwin2_wid = x
    
    window, /free, /pixmap,xsize=state.p.plot1size[0],$
      ysize=state.p.plot1size[1]
    state.p.pixmap1_wid = !d.window

    window, /free, /pixmap,xsize=state.p.plot2size[0],$
      ysize=state.p.plot2size[1]
    state.p.pixmap2_wid = !d.window
    
;  Get sizes for things.
    
    widget_geom = widget_info(state.w.xcleanspec_base, /geometry)
    state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
    state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]-$
      state.p.plot2size[1]

; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xcleanspec', $
      state.w.xcleanspec_base, $
      cleanup = 'xcleanspec_cleanup',$
      event_handler='xcleanspec_resize',$
      /NO_BLOCK
    
; Put state variable into the user value of the top level base.

widget_control, state.w.xcleanspec_base, set_uvalue=state, /no_copy

cont:

end
