; NAME:
;     xcompspec (Xcompspec)    
;
; PURPOSE:
;     Allows the user to compare spectra
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xcompspec,spectra,YTITLE=ytitle,XTITLE=xtitle,TITLE=title,CANCEL=cancel
;
; INPUTS:
;     A structure of spectra where each field is a 3-D array of
;     wavelength, flux, and errors.
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     XTITLE       - A string giving the X axis title
;     YTITLE       - A string giving the Y axis title
;     TITLE        - A string giving the title of the plot
;     CANCEL       - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     The widget is resizable.  Type 'h' for cursor commands.
;
; MODIFICATION HISTORY:
;     2001-12-06 - Written by M. Cushing, Institute for Astronomy, UH
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xcompspec_startup




common xcompspec_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     fix_bg:0L,$
     keyboard:0L,$
     message:0L,$
     norm_base:0L,$
     norm_fld:[0L,0L],$
     offset_base:0L,$
     offset_dl:0L,$
     label_fld:[0L,0L],$
     label_bg:0L,$
     linestyle_dl:0L,$
     offset_fld:[0L,0L],$
     plotwin:0,$
     spec_bg:0L,$
     spec_base:0L,$
     speccolor_dl:0L,$
     textfont:'-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1',$
     title_fld:[0L,0L],$
     xcompspec_base:0L,$
     xlabel_fld:[0L,0L],$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ylabel_fld:[0L,0L],$
     ylog_bg:0L,$
     ymin_fld:[0L,0L],$
     ymax_fld:[0L,0L]}

r = {colorcode:1,$
     cursormode:'None',$
     fix:0,$
     label:1,$
     modspec:0,$
     norm:1.,$
     normspec:0,$
     nspec:0,$
     offsetspec:0,$
     offsets:ptr_new(fltarr(2)),$
     plotspec:ptr_new(fltarr(2)),$
     remove:0,$
     ylog:0}


d = {labels:ptr_new(fltarr(2)),$
     spectra:ptr_new(fltarr(2)),$
     mspectra:ptr_new(fltarr(2))}

p = {buffer:[0.,0.],$
     colors:ptr_new(fltarr(2)),$
     cursor:0,$
     pixmap_wid:0L,$
     plotwin_wid:0L,$
     linestyles:ptr_new(fltarr(2)),$
     absxrange:[0.,0.],$
     absyrange:[0.,0.],$
     xrange:[0.,0.],$
     yrange:[0.,0.],$
     plotsize:[720,450],$
     pscale:!p,$
     xscale:!x,$
     xlabel:'',$
     title:'',$
     yscale:!y,$
     ylabel:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

;  Build the widget.
    
state.w.xcompspec_base = widget_base(title='Xcompspec', $
                                  /column,$
                                  /tlb_size_events)

   quit_button = widget_button(state.w.xcompspec_base,$
                               font=state.w.buttonfont,$
                               event_pro='xcompspec_event',$
                               value='Done',$
                               uvalue='Done')
   
   state.w.keyboard = widget_text(state.w.xcompspec_base, $
                                  /all_events, $
                                  scr_xsize = 1, $
                                  scr_ysize = 1, $
                                  uvalue = 'Keyboard', $
                                  event_pro='xcompspec_event',$
                                  value = '')
   

   state.w.message = widget_text(state.w.xcompspec_base, $
                                 YSIZE=1)
      
      
   col_base = widget_base(state.w.xcompspec_base,$
                          frame=1,$
                          /column)
   
      state.w.plotwin = widget_draw(col_base,$
                                    xsize=state.p.plotsize[0],$
                                    ysize=state.p.plotsize[1],$
                                    /TRACKING_EVENTS,$
                                    /button_events,$
                                    /motion_events,$
                                    event_pro='xcompspec_plotwin_event',$
                                    uvalue='Plot Window 1')
      
      widget_control, state.w.plotwin,draw_button_events=0
      
   row_base = widget_base(col_base,$
                          frame=1,$
                          /row)
   
      xmin = coyote_field2(row_base,$
                           labelfont=state.w.buttonfont,$
                           fieldfont=textfont,$
                           title='X Min:',$
                           uval = 'X Min',$
                           xsize=12,$
                           event_pro = 'xcompspec_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.xmin_fld = [xmin,textid]
      
      xmax = coyote_field2(row_base,$
                           labelfont=state.w.buttonfont,$
                           fieldfont=textfont,$
                           title='X Max:',$
                           uval = 'X Max',$
                           xsize=12,$
                           event_pro = 'xcompspec_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.xmax_fld = [xmax,textid]
      
      ymin = coyote_field2(row_base,$
                           labelfont=state.w.buttonfont,$
                           fieldfont=textfont,$
                           title='Y Min:',$
                           uval = 'Y Min',$
                           xsize=12,$
                           event_pro = 'xcompspec_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.ymin_fld = [ymin,textid]
      
      ymax = coyote_field2(row_base,$
                           labelfont=state.w.buttonfont,$
                           fieldfont=textfont,$
                           title='Y Max:',$
                           uval = 'Y Max',$
                           xsize=12,$
                           event_pro = 'xcompspec_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.ymax_fld = [ymax,textid]

; Get things running.  Center the widget using the Fanning routine.

mkct
          
centertlb,state.w.xcompspec_base
widget_control, state.w.xcompspec_base, /realize

;  Get plotwin ids

widget_control, state.w.plotwin, get_value = x
state.p.plotwin_wid = x

window, /free, /pixmap,xsize=state.p.plotsize[0],$
  ysize=state.p.plotsize[1]
state.p.pixmap_wid = !d.window

;  Get sizes for things.

widget_geom = widget_info(state.w.xcompspec_base, /geometry)

state.p.buffer[0]=widget_geom.xsize-state.p.plotsize[0]
state.p.buffer[1]=widget_geom.ysize-state.p.plotsize[1]

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xcompspec', $
  state.w.xcompspec_base, $
  event_handler='xcompspec_resize_event',$
  /NO_BLOCK

end
;
;******************************************************************************
;
pro xcompspec_cleanup,xcompspec_base

common xcompspec_state

widget_control, xcompspec_base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin
        
endif
state = 0B

end
;
;******************************************************************************
;
pro xcompspec_cp

common xcompspec_state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
state.w.buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

set = *state.r.plotspec
z = where(*state.r.plotspec gt 0,count)
if count ne 0 then set[z] = 1

if not xregistered('xcompspec_cp') then begin

    cp_base = widget_base(group_leader = state.w.xcompspec_base, $
                          /COLUMN, $
                          title ='Control Panel')

       row_base = widget_base(cp_base,$
                              /ROW)
       
          state.w.spec_base = widget_base(row_base,$
                                          /COLUMN,$
                                          /BASE_ALIGN_LEFT,$
                                          /FRAME)
          
             label = widget_label(state.w.spec_base,$
                                  FONT=state.w.buttonfont,$
                                  VALUE='Plot Spectra',$
                                  /ALIGN_CENTER)
            
             names = strtrim(indgen(state.r.nspec)+1,2)+' '+*state.d.labels
             state.w.spec_bg = cw_bgroup(state.w.spec_base,$
                                         FONT=state.w.buttonfont,$
                                         names,$
                                         /NONEXCLUSIVE,$
                                         /RETURN_INDEX,$
                                         /COLUMN,$
                                         SET_VALUE=set,$
                                         UVALUE='Plot Spectra')
         col2_base = widget_base(row_base,$
                                 /COLUMN,$
                                 /BASE_ALIGN_LEFT,$
                                 /FRAME)

             label = widget_label(col2_base,$
                                  FONT=state.w.buttonfont,$
                                  VALUE='Modify Plot',$
                                  /ALIGN_CENTER)

             state.w.label_bg = cw_bgroup(col2_base,$
                                          ['Label'],$
                                          FONT=state.w.buttonfont,$
                                          UVALUE='Plot Label',$
                                          SET_VALUE=state.r.label,$
                                          /NONEXCLUSIVE)

             row = widget_base(col2_base,$
                               /ROW)
             
                state.w.ylog_bg = cw_bgroup(row,$
                                            ['log (Y)'],$
                                            FONT=state.w.buttonfont,$
                                            UVALUE='Y Log',$
                                            SET_VALUE=state.r.ylog,$
                                           /NONEXCLUSIVE)

                state.w.fix_bg = cw_bgroup(row,$
                                           ['Fix Y Range'],$
                                           FONT=state.w.buttonfont,$
                                           UVALUE='Fix Y Range',$
                                           SET_VALUE=state.r.fix,$
                                           /NONEXCLUSIVE)



              values = strtrim(indgen(state.r.nspec)+1,2)
              mspectra_dl = widget_droplist(col2_base,$ 
                                            FONT=state.w.buttonfont,$
                                            TITLE='Spectrum: ',$
                                            VALUE=values,$
                                            UVALUE='Modify Spectrum')
              
              label = coyote_field2(col2_base,$
                                    LABELFONT=state.w.buttonfont,$
                                    FIELDFONT=state.w.textfont,$
                                    TITLE='Label:',$
                                    UVALUE='Label',$
                                    XSIZE=20,$
                                    VALUE=(*state.d.labels)[0],$
                                    EVENT_PRO='xcompspec_event',$
                                    /CR_ONLY,$
                                    TEXTID=textid)
              state.w.label_fld = [label,textid]      


              values = ['White','Red','Green','Blue','Yellow','Magenta',$
                        'Cyan']
              state.w.speccolor_dl = widget_droplist(col2_base,$ 
                                                     FONT=state.w.buttonfont,$
                                                     TITLE='Color: ',$
                                                     VALUE=values,$
                                                     UVALUE='Spectrum Color')
              values = ['Solid','Dotted','Dashed','Dash Dot','Dash Dot Dot',$
                        'Long Dash']
              state.w.linestyle_dl = widget_droplist(col2_base,$              
                                                     FONT=state.w.buttonfont,$
                                                     TITLE='Line Style: ',$
                                                     VALUE=values,$
                                                     UVALUE='Line Style')
              
              xlabel = coyote_field2(col2_base,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE='X title:',$
                                     UVALUE='X Title',$
                                     XSIZE=20,$
                                     VALUE=state.p.xlabel,$
                                     EVENT_PRO='xcompspec_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
              state.w.xlabel_fld = [xlabel,textid]      

              ylabel = coyote_field2(col2_base,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE='Y title:',$
                                     UVALUE='Y Title',$
                                     XSIZE=20,$
                                     VALUE=state.p.ylabel,$
                                     EVENT_PRO='xcompspec_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
              state.w.ylabel_fld = [ylabel,textid]      

              title = coyote_field2(col2_base,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE='Title:',$
                                     UVALUE='Title',$
                                     XSIZE=20,$
                                     VALUE=state.p.title,$
                                     EVENT_PRO='xcompspec_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
              state.w.title_fld = [title,textid]      

         col3_base = widget_base(row_base,$
                                 /COLUMN,$
                                 /BASE_ALIGN_LEFT,$
                                 /FRAME)

             label = widget_label(col3_base,$
                                  FONT=state.w.buttonfont,$
                                  VALUE='Modify Spectra',$
                                  /ALIGN_CENTER)

             normon_bg = cw_bgroup(col3_base,$
                                   ['On'],$
                                   LABEL_LEFT='Normalize: ',$
                                   FONT=state.w.buttonfont,$
                                   SET_VALUE=[state.r.normspec],$
                                   UVALUE='Normalization On',$
                                   /NONEXCLUSIVE)             
             
             state.w.norm_base = widget_base(col3_base,$
                                             /COLUMN)
             widget_control, state.w.norm_base, SENSITIVE=state.r.normspec
       
                norm = coyote_field2(state.w.norm_base,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE='Norm Wavelength:',$
                                     UVALUE='Normalization Wavelength',$
                                     XSIZE=10,$
                                     VALUE=state.r.norm,$
                                     EVENT_PRO='xcompspec_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.norm_fld = [norm,textid]                      

                values = strtrim(indgen(state.r.nspec)+1,2)
                nspectra_dl = widget_droplist(state.w.norm_base,$ 
                                              FONT=state.w.buttonfont,$
                                              TITLE='Spectrum: ',$
                                              VALUE=values,$
                                              UVALUE='Offset Spectrum')

                offset = coyote_field2(state.w.norm_base,$
                                       LABELFONT=state.w.buttonfont,$
                                       FIELDFONT=state.w.textfont,$
                                       TITLE='Offset:',$
                                       UVALUE='Spectrum Offset',$
                                       XSIZE=10,$
                                       VALUE=0,$
                                       EVENT_PRO='xcompspec_event',$
                                       /CR_ONLY,$
                                       TEXTID=textid)
                state.w.offset_fld = [offset,textid]      
                  
         quit = widget_button(cp_base,$
                              VALUE='Done',$
                              FONT=state.w.buttonfont,$
                              UVALUE='Done')
         
   centertlb,cp_base
   
   widget_control, cp_base, /Realize
   
; Start the Event Loop. This will be a non-blocking program.
   
   XManager, 'xcompspec_cp', $
     cp_base, $
     /No_Block,$
     event_handler='xcompspec_event'
   
endif

end
                             ;
;******************************************************************************
;
pro xcompspec_getxrange

common xcompspec_state

spectra = *state.d.mspectra

z = where(*state.r.plotspec gt 0,count)
if count gt 0 then begin

    mins = fltarr(count)
    maxs = fltarr(count)
    for i = 0, count-1 do begin

        mins[i] = min( (spectra.(z[i]))[*,0],MAX=max)
        maxs[i] = max

    endfor
    state.p.xrange = [min(mins),max(maxs)]
    
endif

end
;
;******************************************************************************
;
pro xcompspec_getyrange

common xcompspec_state

spectra = *state.d.mspectra

z = where(*state.r.plotspec gt 0,count)

if count gt 0 then begin

    mins = fltarr(count)
    maxs = fltarr(count)
    for i = 0, count-1 do begin

        good = where((spectra.(z[i]))[*,0] gt state.p.xrange[0] and $
                     (spectra.(z[i]))[*,0] lt state.p.xrange[1])
        mins[i] = min( (spectra.(z[i]))[good,1],/NAN,MAX=max)
        maxs[i] = max

    endfor
    state.p.yrange = [min(mins),max(maxs)]
    
endif

end
;
;******************************************************************************
;
pro xcompspec_help

common xcompspec_state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xcompspec_help') then begin

    help_base = widget_base(group_leader = state.w.xcompspec_base, $
                            /column, $
                            title ='Xcompspec Help')
    
    h = [['Xcompspec is a fully resizing widget.'],$
         [' '],$
         ['Keyboard commands:'],$
         [' '],$
         ['c - Clear mouse mode.'],$
         ['    Use to clear a zoom, fix, or remove session.'],$
         [' '],$
         ['i - To zoom IN in whatever zoom mode the cursor is currently'],$
         ['    in.'],$
         [' '],$
         ['h - To lauch the help window.'],$
         [' '],$
         ['o - To zoom OUT in whatever zoom mode the cursor is currently'],$
         ['    in.'],$
         [' '],$
         ['m - To launch the control panel.'],$
         [' '],$
         ['p - To plot the spectrum to a postscript file.'],$
         [' '],$
         ['w - To plot the entire spectra'],$
         [' '],$
         ['x - Enters x zoom mode'],$
         ['    Press left mouse button at lower x value and then at upper'],$
         ['    x value.'],$
         ['y - Enters y zoom mode'],$
         ['    Press left mouse button at lower y value and then at upper'],$
         ['    y value.'],$
         ['z - Enters zoom mode'],$
         ['    Press the left mouse button in one corner of the zoom box '],$
         ['    and then move the cursor to the other corner and press the '],$
         ['    the left mouse button.'],$
         [' ']]

    help_text = widget_text(help_base, $
                            /scroll, $
                            value = h, $
                            xsize = 70, $
                            ysize = 24)
    
    quit = widget_button(help_base,$
                         value='Done',$
                         font=buttonfont,$
                         uvalue='Done')
      
    centertlb,help_base

    widget_control, help_base, /Realize
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xcompspec_help', $
      help_base, $
      /No_Block,$
      event_handler='xcompspec_event'
    
endif

end
;
;******************************************************************************
;
pro xcompspec_normspec

common xcompspec_state

spectra = *state.d.spectra

state.r.norm = cfld(state.w.norm_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return

z = where(*state.r.plotspec gt 0,count)

for i = 0, count-1 do begin

    tabinv, (spectra.(z[i]))[*,0],state.r.norm,idx
    nspec  = (spectra.(z[i]))[*,1] / (spectra.(z[i]))[idx,1]
    spectra.(z[i]) = [[(spectra.(z[i]))[*,0]],[nspec]]


endfor
*state.d.mspectra = spectra

end
;
;******************************************************************************
;
pro xcompspec_plotspec

common xcompspec_state

xrange  = state.p.xrange
yrange  = state.p.yrange
spectra = *state.d.mspectra

z = where(*state.r.plotspec gt 0,count)
offsets = (state.r.normspec eq 1) ? (*state.r.offsets):fltarr(state.r.nspec)

if count gt 0 then begin

    plot,(spectra.(z[0]))[*,0],(spectra.(z[0]))[*,1]+offsets[z[0]],/xsty,$
      /ysty,yrange=yrange,xrange=xrange,psym=10,xtitle=state.p.xlabel,$
      ytitle=state.p.ylabel,title=state.p.title,/NODATA,$
      color=pscolor(),ylog=state.r.ylog

    for i = 0,count-1 do begin
        
        oplot,(spectra.(z[i]))[*,0],(spectra.(z[i]))[*,1]+offsets[z[i]],$
          psym=10,color=pscolor((*state.p.colors)[z[i]]),$
          linestyle=(*state.p.linestyles)[z[i]]
        names = (i eq 0) ? (*state.d.labels)[z[i]]:$
          [names,(*state.d.labels)[z[i]]]
        
    endfor
    
endif

;  Labels

if count gt 0 and state.r.label eq 1 then legend, (*state.d.labels)[z],$
  /RIGHT,/TOP,BOX=0,TEXTCOLOR=pscolor((*state.p.colors)[z])

cont:

end
;
;******************************************************************************
;
pro xcompspec_plotupdate,PS=ps

common xcompspec_state

if keyword_set(PS) then begin

    xcompspec_plotspec

endif else begin

    
    wset, state.p.pixmap_wid
    erase
    xcompspec_plotspec

    wset, state.p.plotwin_wid
    device, copy=[0,0,state.p.plotsize[0],state.p.plotsize[1],0,0,$
                  state.p.pixmap_wid]

    state.p.xscale = !x
    state.p.yscale = !y
    state.p.pscale = !p
    state.p.cursor = 1
    
    widget_control, state.w.plotwin, /DRAW_MOTION_EVENTS
    widget_control, state.w.plotwin, /DRAW_BUTTON_EVENTS
    
endelse

end
;
;******************************************************************************
;
pro xcompspec_setminmax

common xcompspec_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.xrange[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.xrange[1],2)
widget_control, state.w.ymin_fld[1],set_value=strtrim(state.p.yrange[0],2)
widget_control, state.w.ymax_fld[1],set_value=strtrim(state.p.yrange[1],2)

end
;
;******************************************************************************
;
pro xcompspec_zoom,IN=in,OUT=out

common xcompspec_state

delabsx = state.p.absxrange[1]-state.p.absxrange[0]
delx    = state.p.xrange[1]-state.p.xrange[0]

delabsy = state.p.absyrange[1]-state.p.absyrange[0]
dely    = state.p.yrange[1]-state.p.yrange[0]

xcen = state.p.xrange[0]+delx/2.
ycen = state.p.yrange[0]+dely/2.

case state.r.cursormode of 

    'XZoom': begin

        z = alog10(delabsx/delx)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsx/2.^z/2.
        state.p.xrange = [xcen-hwin,xcen+hwin]
        xcompspec_plotupdate

    end

    'YZoom': begin

        z = alog10(delabsy/dely)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsy/2.^z/2.
        state.p.yrange = [ycen-hwin,ycen+hwin]
        xcompspec_plotupdate

    end

    'Zoom': begin

        z = alog10(delabsx/delx)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsx/2.^z/2.
        state.p.xrange = [xcen-hwin,xcen+hwin]

        z = alog10(delabsy/dely)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsy/2.^z/2.
        state.p.yrange = [ycen-hwin,ycen+hwin]

        xcompspec_plotupdate

    end

    else:

endcase

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xcompspec_event, event

common xcompspec_state

widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of

    'Done': widget_control, event.top, /DESTROY
    
    'Fix Y Range': state.r.fix = event.select

    'Keyboard': begin

        case strtrim(event.ch,2) of 

            '?': xcompspec_help 
            
            'c': begin ; Clear

                state.r.cursormode = 'None'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xcompspec_plotupdate
                
            end

            'i': xcompspec_zoom,/IN

            'h': xcompspec_help ; Help

            'o': xcompspec_zoom,/OUT


            'm': xcompspec_cp  ;  Control Panel

            'p': begin ; Plot

                forminfo = CMPS_FORM(/INITIALIZE,$
                                     SELECT='Full Landscape (color)')

                formInfo = CMPS_FORM(Cancel=canceled, Create=create, $
                                     defaults=forminfo,$
                                     button_names = ['Create PS File'],$
                                     Parent=state.w.xcompspec_base)
                
                IF NOT canceled THEN BEGIN

                        thisDevice = !D.Name
                        Set_Plot, "PS"
                        Device, _Extra=formInfo
                        xcompspec_plotupdate,/PS
                        Device, /Close
                        Set_Plot, thisDevice

                ENDIF
                
            end

            'w': begin

                xcompspec_getxrange
                if not state.r.fix then xcompspec_getyrange
                xcompspec_plotupdate
                xcompspec_setminmax
                
            end

            'x': begin 

                state.r.cursormode = 'XZoom'
                state.p.reg[*] = !values.f_nan                

            end

            'y': begin 

                state.r.cursormode = 'YZoom'
                state.p.reg[*] = !values.f_nan

            end

            'z': begin          ; Zoom

                state.r.cursormode = 'Zoom'
                state.p.reg[*] = !values.f_nan

            end
        
            else:

        endcase

    end

    'Label': begin

        label = cfld(state.w.label_fld,7,/EMPTY,CANCEL=cancel)
        if cancel then goto, cont

        (*state.d.labels)[state.r.modspec] = strtrim(label,2)

        widget_control, state.w.spec_base, UPDATE=0
        widget_control, state.w.spec_bg, /DESTROY

        names = strtrim(indgen(state.r.nspec)+1,2)+' '+*state.d.labels
        state.w.spec_bg = cw_bgroup(state.w.spec_base,$
                                    FONT=state.w.buttonfont,$
                                    names,$
                                    /NONEXCLUSIVE,$
                                    /RETURN_INDEX,$
                                    /COLUMN,$
                                    SET_VALUE=set,$
                                    UVALUE='Plot Spectra')
        
        widget_control, state.w.spec_base, UPDATE=1
        xcompspec_plotupdate

    end

    'Line Style': begin

        (*state.p.linestyles)[state.r.modspec] = event.index
        xcompspec_plotupdate

    end

    'Modify Spectrum': begin

        state.r.modspec = event.index
        widget_control, state.w.speccolor_dl,$
          SET_DROPLIST_SELECT=(*state.p.colors)[event.index]-1
        widget_control, state.w.linestyle_dl,$
          SET_DROPLIST_SELECT=(*state.p.linestyles)[event.index]
        widget_control, state.w.label_fld[1],$
          SET_VALUE=(*state.d.labels)[event.index]
        
    end

    'Normalization On': begin

        state.r.normspec = event.select
        if state.r.normspec then begin

            widget_control, state.w.norm_base, SENSITIVE=1
            xcompspec_normspec            

        endif else begin

            widget_control, state.w.norm_base, SENSITIVE=0
            *state.d.mspectra = *state.d.spectra

        endelse
        if not state.r.fix then xcompspec_getyrange
        xcompspec_setminmax
        xcompspec_plotupdate
 
    end

    'Normalization Wavelength': begin

        xcompspec_normspec
        if not state.r.fix then xcompspec_getyrange
        xcompspec_plotupdate
        xcompspec_setminmax

    end

    'Offset Spectrum': begin
        
        state.r.offsetspec = event.index
        widget_control, state.w.offset_fld[1],$
          SET_VALUE=string((*state.r.offsets)[event.index],format='(f5.2)')
        setfocus, state.w.offset_fld

    end

    'Plot Label': begin

        state.r.label = event.select
        xcompspec_plotupdate
 
    end
    'Plot Spectra': begin

        (*state.r.plotspec)[event.value] = (event.select eq 0) ? 0:1
        if state.r.normspec then xcompspec_normspec
        if not state.r.fix then xcompspec_getyrange
        xcompspec_plotupdate
        xcompspec_setminmax

    end

    'Spectrum Color': begin

        (*state.p.colors)[state.r.modspec] = event.index+1
        xcompspec_plotupdate

    end

    'Spectrum Offset': begin

        val = cfld(state.w.offset_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return

        (*state.r.offsets)[state.r.offsetspec] = val
        if not state.r.fix then xcompspec_getyrange
        xcompspec_plotupdate
        xcompspec_setminmax

    end

    'Title': begin

        label = cfld(state.w.title_fld,7,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.p.title = label
        xcompspec_plotupdate

    end

    'X Title': begin

        label = cfld(state.w.xlabel_fld,7,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.p.xlabel = label
        xcompspec_plotupdate

    end

    'Y Log': begin
        
        state.r.ylog = event.select
        xcompspec_plotupdate

    end
    'Y Title': begin

        label = cfld(state.w.ylabel_fld,7,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.p.ylabel = label
        xcompspec_plotupdate

    end

endcase

cont: 

end
;
;******************************************************************************
;
pro xcompspec_plotwin_event, event

common xcompspec_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale
!x = state.p.xscale
!y = state.p.yscale
x  = event.x/float(state.p.plotsize[0])
y  = event.y/float(state.p.plotsize[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin

    
    if state.r.cursormode eq 'None' then goto, cont
    z = where(finite(state.p.reg) eq 1,count)
    if count eq 0 then begin

        wset, state.p.pixmap_wid
        state.p.reg[*,0] = xy[0:1]
        case state.r.cursormode of

            'XZoom': plots, [event.x,event.x],$
              [0,state.p.plotsize[1]],color=2,/DEVICE,linestyle=1,thick=2

            'YZoom': plots, [0,state.p.plotsize[0]],$
              [event.y,event.y],color=2,/DEVICE,linestyle=1,thick=2
            
            else:

        endcase
        wset, state.p.plotwin_wid
        device, copy=[0,0,state.p.plotsize[0],state.p.plotsize[1],0,0,$
                      state.p.pixmap_wid]
        

    endif else begin 
        
        state.p.reg[*,1] = xy[0:1]
        case state.r.cursormode of 

            'XZoom': state.p.xrange   = [min(state.p.reg[0,*],max=max),max]
            
            'YZoom': state.p.yrange   = [min(state.p.reg[1,*],max=max),max]

            'Zoom': begin

                state.p.xrange   = [min(state.p.reg[0,*],max=max),max]
                state.p.yrange   = [min(state.p.reg[1,*],max=max),max]

            end

        endcase
        xcompspec_plotupdate
        state.r.cursormode   = 'None'
        xcompspec_setminmax
        
    endelse

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin_wid
device, copy=[0,0,state.p.plotsize[0],state.p.plotsize[1],0,0,$
              state.p.pixmap_wid]

case state.r.cursormode of 

    'XZoom': plots, [event.x,event.x],[0,state.p.plotsize[1]],color=2,/DEVICE

    'YZoom': plots, [0,state.p.plotsize[0]],[event.y,event.y],color=2,/DEVICE

    'Zoom': begin

        plots, [event.x,event.x],[0,state.p.plotsize[1]],color=2,/DEVICE
        plots, [0,state.p.plotsize[0]],[event.y,event.y],color=2,/DEVICE
        xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
        plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
          linestyle=2,color=2
        plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
          linestyle=2,color=2
        
    end

    else: begin

        plots, [event.x,event.x],[0,state.p.plotsize[1]],color=2,/DEVICE
        plots, [0,state.p.plotsize[0]],[event.y,event.y],color=2,/DEVICE

    end

endcase

;  Update cursor position.

if state.p.cursor then begin
    
    label = 'Cursor X: '+strtrim(xy[0],2)+', Y:'+strtrim(xy[1],2)
    widget_control,state.w.message,set_value=label
   
endif
    
cont:
    
end
;
;******************************************************************************
;
pro xcompspec_minmax_event,event

common xcompspec_state

xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmin2 = crange(xmin,state.p.xrange[1],'X Min',/KLT,$
               WIDGET_ID=state.w.xcompspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmin_fld[0],set_value=state.p.xrange[0]
    return

endif else state.p.xrange[0] = xmin2

xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmax2 = crange(xmax,state.p.xrange[0],'X Max',/KGT,$
               WIDGET_ID=state.w.xcompspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmax_fld[0],set_value=state.p.xrange[1]
    return

endif else state.p.xrange[1] = xmax2

ymin = cfld(state.w.ymin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymin2 = crange(ymin,state.p.yrange[1],'Y Min',/KLT,$
               WIDGET_ID=state.w.xcompspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymin_fld[0],set_value=state.p.yrange[0]
    return

endif else state.p.yrange[0] = ymin2

ymax = cfld(state.w.ymax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymax2 = crange(ymax,state.p.yrange[0],'Y Max',/KGT,$
               WIDGET_ID=state.w.xcompspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymax_fld[0],set_value=state.p.yrange[1]
    return

endif else state.p.yrange[1] = ymax2

xcompspec_plotupdate

end
;
;******************************************************************************
;
pro xcompspec_resize_event, event

common xcompspec_state

widget_control, state.w.xcompspec_base, tlb_get_size = size

state.p.plotsize[0]=size[0]-state.p.buffer[0]
state.p.plotsize[1]=size[1]-state.p.buffer[1]

widget_control, state.w.plotwin, draw_xsize=state.p.plotsize[0]
widget_control, state.w.plotwin, draw_ysize=state.p.plotsize[1]

wdelete,state.p.pixmap_wid
window, /free, /pixmap,xsize=state.p.plotsize[0],ysize=state.p.plotsize[1]
state.p.pixmap_wid = !d.window

wset, state.p.plotwin_wid
device, copy=[0,0,state.p.plotsize[0],state.p.plotsize[1],0,0,$
              state.p.pixmap_wid]

xcompspec_plotupdate

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;

;
;******************************************************************************
;
pro xcompspec,spectra,labels,YTITLE=ytitle,XTITLE=xtitle,TITLE=title,$
              CANCEL=cancel

cancel = 0

common xcompspec_state

if n_params() eq 2 then begin
    
    zparcheck, 'xcompspec', spectra, 1, 8, 1, 'Spectra' 
    
    if not xregistered('xcompspec') then xcompspec_startup

    state.p.ylabel     = (n_elements(YTITLE) ne 0) ? ytitle:''
    state.p.xlabel     = (n_elements(XTITLE) ne 0) ? xtitle:''
    state.p.title      = (n_elements(TITLE) ne 0) ? title:''
    *state.d.spectra   = spectra
    *state.d.mspectra  = spectra
    *state.d.labels    = labels
    state.r.nspec      = n_tags(spectra)
    *state.r.plotspec  = intarr(state.r.nspec)
    *state.r.offsets   = fltarr(state.r.nspec)
    *state.p.colors    = intarr(state.r.nspec)+1
    *state.p.linestyles = intarr(state.r.nspec)

    (*state.r.plotspec)[0] = 1
    state.p.absxrange = [min((spectra.(0))[*,0],max=max),max]
    state.p.absyrange = [min((spectra.(0))[*,1],max=max,/NAN),max]

    state.p.xrange = state.p.absxrange
    state.p.yrange = state.p.absyrange
    xcompspec_plotupdate
    xcompspec_setminmax     
    xcompspec_cp

    
endif else begin

    cancel = 1
    print, 'Syntax - xcompspec,spectra,labels,YTITLE=ytitle,XTITLE=xtitle,$'
    print, '                   TITLE=title,CANCEL=cancel'
    return
    
endelse

end





