; NAME:
;     xfluxcal
;
; PURPOSE:
;     To flux calibate a SpeX spectrum
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xfluxcal
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     Later
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xfluxcal_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Determine Scales': xfluxcal_getscales,state

    'Load Spectrum': xfluxcal_loadspec,state

    'Magnitude Filter': begin

        state.r.currentfilter = byte(event.value)
        widget_control, state.w.magfilt_fld[1],$
          SET_VALUE=strtrim((*state.r.objmags)[state.r.currentfilter],2)
        setfocus,state.w.magfilt_fld                

    end

    'Magnitude Filter Field': begin

        val = cfld(state.w.magfilt_fld,4,/EMPTY,CANCEL=cancel)        
        if cancel then begin

            widget_control, state.w.magfilt_fld[1],$
              SET_VALUE=strtrim((*state.r.objmags)[state.r.currentfilter],2)
            goto,cont

        endif else begin

            (*state.r.objmags)[state.r.currentfilter] = val
            state.r.currentfilter = (state.r.currentfilter+1) < $
              (state.r.nfilters-1)
            widget_control, state.w.magfilt_bg,SET_VALUE=state.r.currentfilter
            widget_control, state.w.magfilt_fld[1],$
              SET_VALUE=strtrim((*state.r.objmags)[state.r.currentfilter],2)
            setfocus,state.w.magfilt_fld

        endelse
    end

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select
    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xfluxcal_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xfluxcal_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld
            setfocus,state.w.spectrum_fld

        endif

    end

    'Path Field': setfocus,state.w.spectrum_fld

    'Quit': begin

        widget_control, event.top, /DESTROY
        goto, getout
        
    end

    'Scale Filter': (*state.r.usescale)[event.value] = total(event.select)

    'Spectrum Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        spec = dialog_pickfile(DIALOG_PARENT=state.w.xfluxcal_base,$
                              PATH=path,/MUST_EXIST,FILTER='*.fits')
        if spec eq '' then goto, cont
        widget_control,state.w.spectrum_fld[1], $
          SET_VALUE = strmid(spec[0],strpos(spec,'/',/REVERSE_S)+1)

    end

    'System': begin

        state.r.currentsystem = total(event.index)
        xfluxcal_changesystem,state

    end

    'Write File': xfluxcal_writefile,state

    else:

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xfluxcal_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xfluxcal_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin


    ptr_free, state.r.objmags
    ptr_free, state.r.scales
    ptr_free, state.r.usescale

    ptr_free, state.d.spec
    ptr_free, state.d.hdr
    ptr_free, state.d.wvega
    ptr_free, state.d.fvega
    ptr_free, state.d.filtertrans
    ptr_free, state.d.files


endif
state = 0B

end
;
;******************************************************************************
;
pro xfluxcal_changesystem,state

widget_control, state.w.magfiltrow_base,UPDATE=0
widget_control, state.w.magfilt_bg,DESTROY=1

state.w.magfilt_bg = cw_bgroup(state.w.magfiltrow_base,$
                               FONT=state.w.buttonfont,$
                               (state.d.filters).(state.r.currentsystem),$
                               /ROW,$
                               /RETURN_INDEX,$
                               /NO_RELEASE,$
                               SET_VALUE=0,$
                               /EXCLUSIVE,$
                               UVALUE='Magnitude Filter')

widget_control, state.w.magfiltrow_base,UPDATE=1

widget_control, state.w.scalefiltrow_base,UPDATE=0
widget_control, state.w.scalefilt_bg,DESTROY=1

state.w.scalefilt_bg = cw_bgroup(state.w.scalefiltrow_base,$
                                 FONT=state.w.buttonfont,$
                                 (state.d.filters).(state.r.currentsystem),$
                                 /ROW,$
                                 /RETURN_INDEX,$
                                 /NONEXCLUSIVE,$
                                 UVALUE='Scale Filter')

widget_control, state.w.magfiltrow_base,UPDATE=1

state.r.currentfilter = 0 
state.r.nfilters   = n_elements((state.d.filters).(state.r.currentsystem))
*state.r.objmags   = fltarr(state.r.nfilters)+!values.f_nan
*state.r.scales    = fltarr(state.r.nfilters)+1.0
*state.r.usescale  = intarr(state.r.nfilters)

widget_control, state.w.magfilt_fld[1],$
  SET_VALUE=strtrim((*state.r.objmags)[0],2)

;  Load the filter profiles

for i = 0, state.r.nfilters-1 do begin

    rdfloat,state.r.packagepath+'data/'+$
      ((*state.d.files).(state.r.currentsystem))[i],w,t
    key = 'filter'+string(i,format='(i2.2)')

    struc = (i eq 0) ? create_struct(key,[[w],[t]]):$
      create_struct(struc,key,[[w],[t]])

endfor

*state.d.filtertrans = struc
xfluxcal_updatescales,state




end
;
;******************************************************************************
;
pro xfluxcal_getscales,state

trans = *state.d.filtertrans


for i = 0, state.r.nfilters-1 do begin

    scale = photscale((*state.d.spec)[*,0],(*state.d.spec)[*,1],$
                      (*state.r.objmags)[i],*state.d.wvega,*state.d.fvega,$
                      (state.r.vegazero)[state.r.currentsystem],$
                      (trans.(i))[*,0],(trans.(i))[*,1],PLOT=0,CANCEL=cancel)
    if cancel then scale = !values.f_nan
    
    (*state.r.scales)[i] = scale

endfor
xfluxcal_updatescales,state

end
;
;******************************************************************************
;
pro xfluxcal_loadspec,state

;  Get files.

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

spec = cfld(state.w.spectrum_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
spec = cfile(path+spec,WIDGET_ID=state.w.xfluxcal_base,CANCEL=cancel)
if cancel then return

;  Read spectra and load data.

readspec,spec,spec,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,stdairmass

*state.d.spec = spec
*state.d.hdr  = hdr

state.p.xunits = xunits
state.p.yunits = yunits

;  Load Vega

restore, state.r.packagepath+'data/vega5.sav'

*state.d.wvega = wvin/10000.
c = 2.99792458e+8

case state.p.yunits of 

    'ergss-1cm-2A-1': *state.d.fvega = fvin

    'ergss-1cm-2Hz-1': *state.d.fvega = fvin*(*state.d.wvega)^2*(1.0e-2/c)

    'Wm-2um-1': *state.d.fvega = fvin*10.

    'Wm-2Hz-1': *state.d.fvega = fvin*(*state.d.wvega)^2*(1.0e-5/c)
    
    'Jy': *state.d.fvega = fvin*(*state.d.wvega)^2*(1.0e21/c) 

endcase

end
;
;******************************************************************************
;
pro xfluxcal_updatescales,state


for i = 0, state.r.nfilters-1 do begin

    fname = ((state.d.filters).(state.r.currentsystem))[i]+'...............'
    if i eq 0 then begin

        txt = string(fname,format='(a10)')+string((*state.r.scales)[i],$
                                                  format='(1x,f6.3)')
        
    endif else begin

        txt=[[txt],[string(fname,format='(a10)')+string((*state.r.scales)[i],$
                                                        format='(1x,f6.3)')]]


    endelse

endfor
widget_control,state.w.scales_txt,SET_VALUE=txt

end
;
;******************************************************************************
;
pro xfluxcal_writefile,state

print, *state.r.usescale

hdr  = *state.d.hdr
spec = *state.d.spec

z = where(*state.r.usescale eq 1,count)
if count eq 0 then begin
    
    ok = dialog_message('Please select at least 1 filter.',$
                        /error,dialog_parent=state.w.xfluxcal_base)
    return

endif 

scale = total((*state.r.scales)[z])/float(count)
spec[*,1] = spec[*,1]*scale
spec[*,2] = spec[*,2]*sqrt(scale)

file = cfld(state.w.filename_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return



fxaddpar,hdr,'HISTORY','This spectrum has been scaled by '+strtrim(scale,2)+'.'


if state.r.fitsoutput then begin

    writefits,file+'.fits',spec,hdr

endif
if state.r.textoutput then begin

    npix = fxpar(hdr,'NAXIS1')
    openw,lun,file+'.txt', /get_lun
    
    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]
    
    for i = 0, npix-1 do begin
        
        printf, lun,  strjoin( reform(spec[i,*],3),'  ' )
            
    endfor
    close, lun
    free_lun, lun
    
endif

end
;
;******************************************************************************
;
; ------------------------------Main Program---------------------------------- 
;
;******************************************************************************
;
;
pro xfluxcal

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.
;  p - contains info pertaining to plotting.


last   = strpos(!path,'Spantool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

packagepath = cpath(result,CANCEL=cancel)
if cancel then return

rdphotfile,packagepath+'data/photsystems.dat',nsystems,systems,filters,$
  files,vegazero
if cancel then return


w = {buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     filename_fld:[0L,0L],$
     magfilt_bg:0L,$
     magfilt_fld:[0L,0L],$
     magfiltrow_base:0L,$
     path_fld:[0L,0L],$
     scalefiltrow_base:0L,$
     scalefilt_bg:0L,$
     scales_txt:0L,$
     spectrum_fld:[0L,0L],$
     textfont:'-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1',$
     xfluxcal_base:0L}

r = {currentsystem:0,$
     currentfilter:0,$
     fitsoutput:1.,$
     nfilters:n_elements(filters.(0)),$
     objmags:ptr_new(fltarr(n_elements(filters.(0)))+!values.f_nan),$
     packagepath:packagepath,$
     scales:ptr_new(fltarr(n_elements(filters.(0))+1.0)),$
     textoutput:0.,$
     usescale:ptr_new(intarr(n_elements(filters.(0)))),$
     vegazero:vegazero}

d = {files:ptr_new(fltarr(2)),$
     filters:filters,$
     filtertrans:ptr_new(fltarr(2)),$
     fvega:ptr_new(fltarr(2)),$
     hdr:ptr_new(fltarr(2)),$
     spec:ptr_new(fltarr(2)),$
     systems:systems,$
     wvega:ptr_new(fltarr(2))}

*d.files = files

p = {xunits:'',$
     yunits:''}

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

state.w.xfluxcal_base = widget_base(title='Xfluxcal', $
                                    EVENT_PRO='xfluxcal_event',$
                                    /COLUMN)
    
   quit_button = widget_button(state.w.xfluxcal_base,$
                               FONT=state.w.buttonfont,$
                               VALUE='Done',$
                               UVALUE='Quit')
       
   row_base = widget_base(state.w.xfluxcal_base,$
                          /ROW)

      col1_base = widget_base(row_base,$
                              /COLUMN)
                  
         box1_base = widget_base(col1_base,$
                                 FRAME=1,$
                                 /COLUMN)

            label = widget_label(box1_base,$
                                 VALUE='1.  Load Spectrum',$
                                 FONT=state.w.buttonfont,$
                                 /ALIGN_LEFT)
            
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
         
               button = widget_button(row,$
                                      FONT=state.w.buttonfont,$
                                      VALUE='Path',$
                                      UVALUE='Path Button')
               
               field = coyote_field2(row,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE=':',$
                                     UVALUE='Path Field',$
                                     XSIZE=25,$
                                     TEXTID=textid)
               state.w.path_fld = [field,textid]    
               
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
         
               button = widget_button(row,$
                                      FONT=state.w.buttonfont,$
                                      VALUE='Spectrum',$
                                      UVALUE='Spectrum Button',$
                                      EVENT_PRO='xfluxcal_event')
               
               field = coyote_field2(row,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE=':',$
                                     UVALUE='Spectrum Field',$
                                     XSIZE=18,$
                                     EVENT_PRO='xfluxcal_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
               state.w.spectrum_fld = [field,textid]
            
            load = widget_button(box1_base,$
                                 VALUE='Load Spectrum',$
                                 FONT=state.w.buttonfont,$
                                 UVALUE='Load Spectrum')

         box2_base = widget_base(col1_base,$
                                 FRAME=1,$
                                 /COLUMN)
                              
            label = widget_label(box2_base,$
                                 VALUE='2.  Load Magnitudes',$
                                 FONT=state.w.buttonfont,$
                                 /ALIGN_LEFT)

            dl = widget_droplist(box2_base,$
                                 FONT=state.w.buttonfont,$
                                 TITLE='System:',$
                                 VALUE=state.d.systems,$
                                 UVALUE='System')   

            state.w.magfiltrow_base = widget_base(box2_base,$
                                                  /ROW,$
                                                  /BASE_ALIGN_CENTER)
            
               state.w.magfilt_bg = cw_bgroup(state.w.magfiltrow_base,$
                                              FONT=state.w.buttonfont,$
                                              (state.d.filters).(0),$
                                              /ROW,$
                                              /RETURN_INDEX,$
                                              /NO_RELEASE,$
                                              SET_VALUE=0,$
                                              /EXCLUSIVE,$
                                              UVALUE='Magnitude Filter')
               
            fld = coyote_field2(box2_base,$
                                LABELFONT=state.w.buttonfont,$
                                FIELDFONT=state.w.textfont,$
                                TITLE='Magnitude:',$
                                UVALUE='Magnitude Filter Field',$
                                XSIZE=10,$
                                VALUE=(*state.r.objmags)[0],$
                                EVENT_PRO='xfluxcal_event',$
                                /CR_ONLY,$
                                TEXTID=textid)
            state.w.magfilt_fld = [fld,textid]

      col2_base = widget_base(row_base,$
                              /COLUMN)

         box3_base = widget_base(col2_base,$
                                 FRAME=1,$
                                 /COLUMN)
                  
            label = widget_label(box3_base,$
                                 VALUE='3.  Scale Factors',$
                                 FONT=state.w.buttonfont,$
                                 /ALIGN_LEFT)

            scale = widget_button(box3_base,$
                                  VALUE='Determine Scales',$
                                  FONT=state.w.buttonfont,$
                                  UVALUE='Determine Scales')
            
            state.w.scales_txt = widget_text(box3_base,$
                                             /SCROLL,$
                                             XSIZE=35,$
                                             YSIZE=5)

         box4_base = widget_base(col2_base,$
                                 FRAME=1,$
                                 /COLUMN)
      
            label = widget_label(box4_base,$
                                 VALUE='4.  Write File',$
                                 FONT=state.w.buttonfont,$
                                 /ALIGN_LEFT)

            state.w.scalefiltrow_base = widget_base(box4_base,$
                                                    /ROW,$
                                                    /BASE_ALIGN_CENTER)

            state.w.scalefilt_bg = cw_bgroup(state.w.scalefiltrow_base,$
                                             (state.d.filters).(0),$
                                             LABEL_LEFT='Scale with:',$
                                             FONT=state.w.buttonfont,$
                                             /ROW,$
                                             /RETURN_INDEX,$
                                             /NONEXCLUSIVE,$
                                             UVALUE='Scale Filter')
            
         outformat_bg = cw_bgroup(box4_base,$
                                  font=state.w.buttonfont,$
                                  ['FITS','Text'],$
                                  /row,$
                                  /return_name,$
                                  /nonexclusive,$
                                  label_left='Output Format:',$
                                  UVALUE='Output Format',$
                                  set_VALUE=[1,0])

         field = coyote_field2(box4_base,$
                               labelfont=state.w.buttonfont,$
                               fieldfont=state.w.textfont,$
                               title='Object File:',$
                               uvalue='Object File Oname',$
                               xsize=18,$
                               textID=textid)
         state.w.filename_fld = [field,textid]
            
         write = widget_button(box4_base,$
                               value='Write File',$
                               uvalue='Write File',$
                               font=state.w.buttonfont)
         

; Get things running.  Center the widget using the Fanning routine.
            
centertlb,state.w.xfluxcal_base

widget_control, state.w.xfluxcal_base, /REALIZE
xfluxcal_changesystem,state

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xfluxcal', $
  state.w.xfluxcal_base, $
  cleanup='xfluxcal_cleanup',$
  /NO_BLOCK

; Put state variable into the user value of the top level base.

widget_control, state.w.xfluxcal_base, set_uvalue=state, /no_copy

end
