;
;******************************************************************************
;
pro xgetscales_initcommon,std,stdorders,stdmag,stdbmv,wvega,fvega,fcvega,$
                          fc2vega,kernel,obj,objorders,objnaps,atmos_wave,$
                          atmos_trans,initscale,scales,YITLE=ytitle,$
                          XTITLE=xtitle,Vshift=vshift ,Vrot=vrot,CANCEL=cancel

common xgetscales_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

;  Load data

last        = strpos(!path,'Spantool')
first       = strpos(!path,':',last,/REVERSE_SEARCH)
packagepath = strmid(!path,first+1,last-first+7)

readfmt,packagepath+'/data/HI.dat','F8.6,1x,A12',hlines,hnames

w = {keyboard:0L,$
     fix_bg:0L,$
     message:0L,$
     order_dl:0L,$
     plotwin1:0,$
     plotwin2:0,$
     scalezero_fld:[0L,0L],$
     slider:0L,$
     subregscale_fld:[0L,0L],$
     tension_fld:[0L,0L],$
     vshift_fld:[0L,0L],$
     vrot_fld:[0L,0L],$
     xgetscales_base:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin_fld:[0L,0L],$
     ymax_fld:[0L,0L]}

r = {stdmag:stdmag,$
     packagepath:packagepath,$
     tension:10.,$
     stdbmv:stdbmv,$
     cancel:0,$
     plotatmos:1,$
     cursormode:'None',$
     fix:0,$
     scalespec:ptr_new(fltarr(2)),$
     spectype:'Telluric',$
     stdidx:0}

c = {a:ptr_new(fltarr(2)),$
     b:ptr_new(fltarr(2)),$
     c:ptr_new(fltarr(2)),$
     d:ptr_new(fltarr(2)),$
     e:ptr_new(fltarr(2)),$
     f:ptr_new(fltarr(2))}


d = {atrans:[[atmos_wave],[atmos_trans]],$
     hlines:hlines,$
     initscale:initscale,$
     deltascales:ptr_new(fltarr(2)),$
     kernel:kernel,$
     hnames:hnames,$
     norders:n_elements(stdorders),$
     std:std,$
     wvega:wvega,$
     fvega:fvega,$
     fcvega:fcvega,$
     fnvega:fvega/fcvega,$
     owvega:wvega,$
     ofvega:fvega,$
     ofcvega:fcvega,$
     ofc2vega:fc2vega,$
     stdorders:stdorders,$
     obj:obj,$
     objorders:objorders,$
     objnaps:objnaps,$
     tellspec:ptr_new(fltarr(2)),$
     vshift:(n_elements(vshift) eq 0) ? 0.0:vshift,$
     vrot:(n_elements(vrot) eq 0) ? 0.0:vrot}


p = {buffer:[0.,0.],$
     fix:0,$
     cursor:0,$
     modcpt:-1,$
     pixmap1_wid:0L,$
     pixmap2_wid:0L,$
     plot1size:[770,170],$
     plot2size:[770,280],$
     plot1scale:0.,$
     plot2scale:0.,$
     plotwin1_wid:0L,$
     plotwin2_wid:0L,$
     xrange2:[0.,0.],$
     yrange1:[0.,0.],$
     yrange2:[0.,0.],$
     pscale1:!p,$
     pscale2:!p,$
     spec:ptr_new(fltarr(2)),$
     xscale1:!x,$
     xscale2:!x,$
     xtitle:'',$
     yscale1:!y,$
     yscale2:!y,$
     ytitle:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}

p.plot1scale = float(p.plot1size[1])/(p.plot1size[1]+p.plot2size[1])
p.plot2scale = float(p.plot2size[1])/(p.plot1size[1]+p.plot2size[1])

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p,c:c}

xgetscales_modvega


;  Load up the initial control points


for i = 0,state.d.norders-1 do begin
    
    min = min(state.d.std[*,0,i],max=max)
    z = where(state.d.hlines gt min and state.d.hlines lt max,count)
    hlines = state.d.hlines[z]
    *state.c.(i) = [min,initscale]
    for j = 0,count-1 do begin
        
        *state.c.(i) = [[*state.c.(i)],[hlines[j],initscale]]
        
    endfor
    *state.c.(i) = [[*state.c.(i)],[max,initscale]]
    s = sort((*state.c.(i))[0,*])
    *state.c.(i) = (*state.c.(i))[*,s]
    
endfor

end
;  
;******************************************************************************
;
;-----------------------------Event Handlers----------------------------------
;
;******************************************************************************
;
pro xgetscales_event,event

common xgetscales_state
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Accept': widget_control, event.top, /DESTROY

    'Atmosphere': begin

        state.r.plotatmos = event.select
        xgetscales_plotupdate2

    end

    'Cancel': begin

        state.r.cancel = 1
        widget_control, event.top, /DESTROY

    end

    'Fix': state.p.fix = event.select

    'Keyboard': begin

        case strtrim(event.ch,2) of 

            'c': begin

                if state.r.cursormode eq 'Select Region' then begin

                    z = where(finite(state.r.scalereg) eq 1,count)
                    if count eq 0 then begin

                        xgetscales_modoffsets,$
                          (*state.r.scalespec)[state.r.scalereg[0,0]-1]
                        state.r.scalereg[*] = !values.f_nan
                        widget_control, state.w.regionscale_base, MAP=0
                        xgetscales_makescale
                        xgetscales_maketelluric
                        xgetscales_makespec
                        
                    endif else state.r.scalereg[*] = !values.f_nan

                endif

                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xgetscales_plotupdate1
                xgetscales_plotupdate2


            end

            'f': state.r.cursormode = 'Find Region'

            'm': state.r.cursormode = 'Move Cpt'

            'w': begin

                xgetscales_getminmax
                xgetscales_setminmax
                xgetscales_plotupdate1
                xgetscales_plotupdate2

            end

            's': begin

                state.r.cursormode = 'Select Region'
                state.r.scalereg[*] = !values.f_nan

            end

            'x': begin 

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'XZoom'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'y': begin 

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'YZoom'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'z': begin

                state.r.cursormode = 'Zoom'
                state.p.reg[*] = !values.f_nan
                
            end
            
            else:

        endcase

    end

    'Order': begin

        state.r.stdidx = event.index
        xgetscales_makescale
        xgetscales_maketelluric
        xgetscales_makespec
        xgetscales_getminmax
        xgetscales_setminmax
        xgetscales_plotupdate1
        xgetscales_plotupdate2        
        
        state.p.fix = 0
        widget_control, state.w.fix_bg, SET_VALUE=0

    end

    'Reset Control Points': begin

        (*state.c.(state.r.stdidx))[1,*] = state.d.initscale
        xgetscales_makescale
        xgetscales_maketelluric
        xgetscales_makespec
        xgetscales_plotupdate1
        xgetscales_plotupdate2                

    end

    'Spline Tension': begin
        
        tension = cfld(state.w.tension_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        tension = crange(tension,[0.0,10.0],'Tension',/KGT,/KLT,$
                         WIDGET_ID=state.w.xgetscales_base,CANCEL=cancel)
        if cancel then return

        state.r.tension = tension 

        xgetscales_makescale
        xgetscales_makespec
        xgetscales_plotupdate1
        xgetscales_maketelluric
        xgetscales_plotupdate2      
        
    end

    'Spectrum Type': begin

        state.r.spectype = event.value
        xgetscales_makespec
        if not state.p.fix then begin

            xgetscales_getminmax
            xgetscales_setminmax

        endif else begin

            z = where(*state.p.spec gt state.p.xrange2[0] and $
                      *state.p.spec lt state.p.xrange2[1])
            state.p.yrange2 = [min((*state.p.spec)[z,1],max=max,/NAN),max]
            xgetscales_setminmax

        endelse
        xgetscales_plotupdate1
        xgetscales_plotupdate2

    end

    'Vrot': begin

        val = cfld(state.w.vrot_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then goto, cont
        state.d.vrot = val
        xgetscales_modvega
        xgetscales_maketelluric
        xgetscales_makespec
        xgetscales_plotupdate2       

    end

    'Vshift': begin
        
        val = cfld(state.w.vshift_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then goto, cont
        state.d.vshift = val
        xgetscales_modvega
        xgetscales_maketelluric
        xgetscales_makespec
        xgetscales_plotupdate2       


    end

    else:

endcase

cont: 

end
;
;******************************************************************************
;
pro xgetscales_plotwin1event,event

common xgetscales_state
widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then begin

        widget_control, state.w.keyboard, SENSITIVE=0
        wset, state.p.plotwin1_wid
        device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                      state.p.pixmap1_wid]
        wset, state.p.plotwin2_wid
        device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                      state.p.pixmap2_wid]
        
    endif
    goto, cont
    
endif else widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

wset, state.p.plotwin1_wid

;  Load up plot scales.

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1

;  Determine the wavelength and flux values for the event.

x  = event.x/float(state.p.plot1size[0])  
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

case event.type of 
    
    0: begin
        
        state.p.reg[*,0] = xy[0:1]
        xgetscales_whichpoint,idx
        state.p.modcpt = idx
        
    end
    
    1: begin

        if state.p.modcpt ne -1 then begin

            widget_control, /HOURGLASS
            tmp = state.p.modcpt
            state.p.modcpt = -1
            xgetscales_maketelluric
            xgetscales_makespec
            xgetscales_plotupdate1
            xgetscales_plotupdate2
        
        endif
    end
    
    2: begin
        
        if state.p.modcpt ne -1 then begin
            
            (*state.c.(state.r.stdidx))[1,state.p.modcpt] = xy[1]
            xgetscales_makescale
            xgetscales_plotupdate1
            
        endif
        
    end
    
endcase

;  Copy the pixmap and draw the cross hair.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

wset, state.p.plotwin1_wid
plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
;wset,state.p.plotwin2_wid
;plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE


tabinv, (*state.p.spec)[*,0],xy[0],idx
idx = round(idx)
label = 'Cursor X: '+strtrim(xy[0],2)+', Y: '+strtrim(xy[1],2)
label = label+'   Scale X: '+strtrim((*state.p.spec)[idx,0],2)+$
  ', Scale Y: '+strtrim((*state.r.scalespec)[idx],2)
widget_control,state.w.message,set_value=label

cont:

end
;
;******************************************************************************
;
pro xgetscales_plotwin2event,event

common xgetscales_state
widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then begin

        widget_control, state.w.keyboard, SENSITIVE=0
        wset, state.p.plotwin1_wid
        device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                      state.p.pixmap1_wid]
        wset, state.p.plotwin2_wid
        device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                      state.p.pixmap2_wid]
        
    endif
    goto, cont
    
endif else widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

wset, state.p.plotwin2_wid

;  Load up plot scales.

!p = state.p.pscale2
!x = state.p.xscale2
!y = state.p.yscale2

;  Determine the wavelength and flux values for the event.

x  = event.x/float(state.p.plot2size[0])  
y  = event.y/float(state.p.plot2size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin

    case state.r.cursormode of 

       'Zoom': begin
       
           z = where(finite(state.p.reg) eq 1,count)
           if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin

               state.p.reg[*,1] = xy[0:1]
               state.p.xrange2 = [min(state.p.reg[0,*],max=max),max]
               state.p.yrange2 = [min(state.p.reg[1,*],max=max),max]
               state.r.cursormode = 'None'
               state.p.reg[*] = !values.f_nan
               xgetscales_plotupdate1
               xgetscales_plotupdate2
               xgetscales_setminmax
               
           endelse

        end
           
        'XZoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1]
                wset, state.p.pixmap2_wid
                plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,$
                  /DEVICE,linestyle=1,thick=2
               wset, state.p.plotwin2_wid
               device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,$
                             0,state.p.pixmap2_wid]
               
            endif else begin

                state.p.reg[*,1] = xy[0:1]
                state.p.xrange2 = [min(state.p.reg[0,*],max=m),m]
                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xgetscales_plotupdate1
                xgetscales_plotupdate2
                xgetscales_setminmax
                
            endelse


        end

        'YZoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1]
                wset, state.p.pixmap2_wid
                plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,$
                  /DEVICE,linestyle=1,thick=2

                wset, state.p.plotwin2_wid
                device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],$
                              0,0,state.p.pixmap2_wid]
               
            endif else begin

                state.p.reg[*,1] = xy[0:1]
                state.p.yrange2 = [min(state.p.reg[1,*],max=m),m]
                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xgetscales_plotupdate1
                xgetscales_plotupdate2
                xgetscales_setminmax
                
            endelse

        end
        else:

    endcase

endif

;  Copy the pixmap and draw the cross hair.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

case state.r.cursormode of 

    'XZoom': begin

        wset, state.p.plotwin2_wid
        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE
        wset, state.p.plotwin1_wid
        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        
    end

    'YZoom': plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE

    'Zoom': begin

        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE
        plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE
        xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
        plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
          linestyle=2,color=2
        plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
          linestyle=2,color=2
        
    end

    else: begin
        
        wset, state.p.plotwin2_wid
        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE
        plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE
        wset, state.p.plotwin1_wid
        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

    end

endcase

;  Update cursor position

tabinv, (*state.p.spec)[*,0],xy[0],idx
idx = round(idx)
label = 'Cursor X: '+strtrim(xy[0],2)+', Y: '+strtrim(xy[1],2)
label = label+'   Spectrum X: '+strtrim( (*state.p.spec)[idx,0],2)+$
  ', Y: '+strtrim( (*state.p.spec)[idx,1],2)+', Scale: '+$
  strtrim((*state.r.scalespec)[idx],2)
widget_control,state.w.message,set_value=label

cont:
end
;
;******************************************************************************
;
pro xgetscales_resizeevent, event

common xgetscales_state

widget_control, state.w.xgetscales_base, tlb_get_size = size

;  Window 1

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=(size[1]-state.p.buffer[1])*state.p.plot1scale

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

;  Window 2

state.p.plot2size[0]=size[0]-state.p.buffer[0]
state.p.plot2size[1]=(size[1]-state.p.buffer[1])*state.p.plot2scale

widget_control, state.w.plotwin2, draw_xsize=state.p.plot2size[0]
widget_control, state.w.plotwin2, draw_ysize=state.p.plot2size[1]

wdelete,state.p.pixmap2_wid
window, /free, /pixmap,xsize=state.p.plot2size[0],ysize=state.p.plot2size[1]
state.p.pixmap2_wid = !d.window


xgetscales_plotupdate1
xgetscales_plotupdate2

end
;
;******************************************************************************
;
;---------------------------- Support Procedures ----------------------------
;
;******************************************************************************
;
pro xgetscales_getminmax

common xgetscales_state

state.p.xrange2 = [min((*state.p.spec)[*,0],max=max),max]
state.p.yrange2 = [min((*state.p.spec)[*,1],max=max,/NAN),max]

end
;
;******************************************************************************
;
pro xgetscales_makescale

common xgetscales_state

scalespec = spline((*state.c.(state.r.stdidx))[0,*],$
                            (*state.c.(state.r.stdidx))[1,*],$
                            state.d.std[*,0,state.r.stdidx],$
                            state.r.tension)

z = where(finite(scalespec) eq 1,count)
;if count ne 0 then begin

*state.r.scalespec = scalespec

;endif else begin

;    state.r.tension = state.r.tension-1
;    widget_control, state.w.tension_fld[1],$
;      SET_VALUE=string(state.r.tension)


;endelse

end
;
;******************************************************************************
;
pro xgetscales_makespec

common xgetscales_state

case state.r.spectype of

    'Telluric': begin

        wave = state.d.std[*,0,state.r.stdidx]
        spec = *state.d.tellspec
        *state.p.spec = [[wave],[1./spec]]

    end

    'Object': begin

        interpspec,state.d.std[*,0,state.r.stdidx],*state.d.tellspec,$
          state.d.obj[*,0,state.d.objnaps*state.r.stdidx],tellspec

        wave = state.d.obj[*,0,state.d.objnaps*state.r.stdidx]
        spec = state.d.obj[*,1,state.d.objnaps*state.r.stdidx]*tellspec
        *state.p.spec = [[wave],[spec]]

    end

endcase

end
;
;******************************************************************************
;
pro xgetscales_maketelluric

common xgetscales_state

telluric, state.d.std[*,0,state.r.stdidx],$
  state.d.std[*,1,state.r.stdidx],state.d.std[*,2,state.r.stdidx],$
  state.r.stdmag,state.r.stdbmv,state.d.kernel,$
  *state.r.scalespec,state.d.wvega,state.d.fvega,state.d.fcvega,$
  state.d.ofc2vega,tellcor,tellcor_error

*state.d.tellspec = tellcor

end
;
;******************************************************************************
;
pro xgetscales_plotscale

common xgetscales_state

if state.p.modcpt eq -1 then begin
    
    min = min(*state.r.scalespec,max=max)
    del = (max-min)*0.3
    if del lt 1e-2 then del = 0.5
    state.p.yrange1 = [min-del,max+del]

endif

plot,(*state.p.spec)[*,0],*state.r.scalespec,/xsty,/ysty,$
  yrange=state.p.yrange1,xrange=state.p.xrange2,xtitle=state.p.xtitle,$
  ytitle='!5Scale Factor',psym=10,position=[0.1,0.1,0.95,0.95]

ncpts = n_elements((*state.c.(state.r.stdidx))[0,*])

for i = 0, ncpts-1 do begin
    
    plots,[(*state.c.(state.r.stdidx))[0,i],(*state.c.(state.r.stdidx))[1,i]],$
      psym=2,color=3
      
endfor

state.p.pscale1 = !p
state.p.xscale1 = !x
state.p.yscale1 = !y

end
;
;******************************************************************************
;
pro xgetscales_plotspec

common xgetscales_state

if state.r.plotatmos then begin

    z = where( state.d.atrans[*,0] lt state.p.xrange2[1] and $
               state.d.atrans[*,0] gt state.p.xrange2[0],count)
    
    if count ne 0 then begin
        
        plot, state.d.atrans[z,0],state.d.atrans[z,1],color=5,$
          yrange=[0,1],ystyle=5,xstyle=5,xrange=state.p.xrange2,psym=10,$
          position=[0.1,0.1,0.95,0.88]
        ticks = string(findgen(11)*.1,format='(f3.1)')
        axis,yaxis=1,yticks=10,ytickname=ticks,yminor=1,color=5
        
    endif
    ystyle = 9

endif else ystyle=1
plot,(*state.p.spec)[*,0],(*state.p.spec)[*,1],$
  /xsty,ystyle=ystyle,yrange=state.p.yrange2,xrange=state.p.xrange2,$
  xtitle=state.p.xtitle,ytitle=state.p.ytitle,/NOERASE,psym=10,$
  position=[0.1,0.1,0.95,0.88]

;  Label H lines

z = where(state.d.hlines lt state.p.xrange2[1] and $
          state.d.hlines gt state.p.xrange2[0],count)


for i =0, count-1 do begin

    tabinv,(*state.p.spec)[*,0],state.d.hlines[z[i]],idx
    plots, [state.d.hlines[z[i]],state.d.hlines[z[i]]],$
      [(*state.p.spec)[idx,1],!y.crange[1]],linestyle=1,color=3,thick=2

    xy = convert_coord(state.d.hlines[z[i]],1,/DATA,/TO_NORM)
    name = '!5'+state.d.hnames[z[i]]
    xyouts, xy[0],0.88,name,orientation=90,/NORM,$
      charsize=str_size(name,0.2),color=3

endfor

state.p.pscale2 = !p
state.p.xscale2 = !x
state.p.yscale2 = !y

end
;
;******************************************************************************
;
pro xgetscales_plotupdate1

common xgetscales_state

wset, state.p.pixmap1_wid
erase
xgetscales_plotscale

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

end
;
;******************************************************************************
;
pro xgetscales_plotupdate2

common xgetscales_state

wset, state.p.pixmap2_wid
erase
xgetscales_plotspec 

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

end
;
;******************************************************************************
;
pro xgetscales_setminmax

common xgetscales_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.xrange2[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.xrange2[1],2)
widget_control, state.w.ymin_fld[1],set_value=strtrim(state.p.yrange2[0],2)
widget_control, state.w.ymax_fld[1],set_value=strtrim(state.p.yrange2[1],2)

end
;
;******************************************************************************
;
pro xgetscales_modvega

common xgetscales_state

rshift        = 1.0 + (state.d.vshift/2.99792458E5)
state.d.wvega = state.d.owvega*rshift

if state.d.vrot ne 0.0 then begin
    
    vdisp = 1e-5

    c = 2.99792458E5
    vstep = c*vdisp

    npix = state.d.vrot/vstep
    nkern = 2*round(npix)
    if not nkern mod 2 then nkern = nkern + 1
    
    kernel_x = (indgen(nkern)-nkern/2 ) * vstep
    kernel = fltarr(nkern)
    z = where(abs(kernel_x) lt state.d.vrot)
    kernel[z] = ( (1. - (kernel_x[z]/state.d.vrot)^2)^0.5 ) / state.d.vrot
    kernel = kernel/total(kernel)
    
    cfcvega = convol( (state.d.ofcvega/state.d.ofc2vega-1.0), kernel ) + 1.0
    cfcvega = temporary(cfcvega)*state.d.ofc2vega

    cfnvega = convol( (state.d.ofvega/state.d.ofcvega-1.0), kernel ) + 1.0

    state.d.fvega  = cfnvega*cfcvega
    state.d.fcvega = cfcvega
    state.d.fnvega = cfnvega

endif



end
;
;******************************************************************************
;
pro xgetscales_whichpoint,idx

common xgetscales_state

idx = -1
del = state.p.xrange2[1]-state.p.xrange2[0]
ncpts = n_elements((*state.c.(state.r.stdidx))[0,*])

for i = 0,ncpts-1 do begin

    if state.p.reg[0,0] gt (*state.c.(state.r.stdidx))[0,i]-(del*0.005) and $
      state.p.reg[0,0] lt (*state.c.(state.r.stdidx))[0,i]+(del*0.005) then $
      idx = i

endfor

end
;
;******************************************************************************
;
;-----------------------------Main Program------------------------------------
;
;******************************************************************************
;
pro xgetscales,std,stdorders,stdmag,stdbmv,wvega,fvega,fcvega,fc2vega,$
               kernel,obj,objorders,objnaps,atmos_wave,atmos_trans,$
               initscale,scales,YITLE=ytitle,XTITLE=xtitle,vshift=vshift,$
               vrot=vrot,CANCEL=cancel

common xgetscales_state

if not xregistered('xgetscales') then begin
        
    xgetscales_initcommon,std,stdorders,stdmag,stdbmv,wvega,fvega,fcvega,$
      fc2vega,kernel,obj,objorders,objnaps,atmos_wave,atmos_trans,$
      initscale,scales,YITLE=ytitle,XTITLE=xtitle,vshift=vshift,vrot=vrot,$
      CANCEL=cancel
    
;  Build the widget.

    buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
    textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
    
    state.w.xgetscales_base = widget_base(title='Xgetscales', $
                                          /COLUMN,$
                                          /TLB_SIZE_EVENTS)

       quit_button = widget_button(state.w.xgetscales_base,$
                                   FONT=buttonfont,$
                                   EVENT_PRO='xgetscales_event',$
                                   VALUE='Cancel',$
                                   UVALUE='Cancel')


       state.w.message = widget_text(state.w.xgetscales_base, $
                                     VALUE='',$
                                     YSIZE=1)

       state.w.keyboard = widget_text(state.w.xgetscales_base, $
                                      /ALL_EVENTS, $
                                      SCR_XSIZE=1, $
                                      SCR_YSIZE=1, $
                                      UVALUE='Keyboard', $
                                      EVENT_PRO='xgetscales_event',$
                                      VALUE= '')

    row = widget_base(state.w.xgetscales_base,$
                      EVENT_PRO='xgetscales_event',$
                      /ROW,$
                      FRAME=1,$
                      /BASE_ALIGN_CENTER)
    
       bg = cw_bgroup(row,$
                      FONT=buttonfont,$
                      ['Telluric','Object'],$
                      /ROW,$
                      /RETURN_NAME,$
                      /NO_RELEASE,$
                      /EXCLUSIVE,$
                      LABEL_LEFT='Spectrum:',$
                      UVALUE='Spectrum Type',$
                      SET_VALUE=0)
       
       state.w.order_dl = widget_droplist(row,$
                                          FONT=buttonfont,$
                                          TITLE='Order:',$
                                          VALUE='01',$
                                          UVALUE='Order')
       
       button = widget_button(row,$
                              VALUE='Reset Control Points',$
                              FONT=buttonfont,$
                              UVALUE='Reset Control Points')    



    state.w.plotwin1 = widget_draw(state.w.xgetscales_base,$
                                   XSIZE=state.p.plot1size[0],$
                                   YSIZE=state.p.plot1size[1],$
                                   /TRACKING_EVENTS,$
                                   /BUTTON_EVENTS,$
                                   /MOTION_EVENTS,$
                                   EVENT_PRO='xgetscales_plotwin1event',$
                                   UVALUE='Plot Window 1')


    row = widget_base(state.w.xgetscales_base,$
                      EVENT_PRO='xgetscales_event',$
                      FRAME=1,$
                      /ROW,$
                      /BASE_ALIGN_CENTER)
    
       fld = coyote_field2(row,$
                           LABELFONT=buttonfont,$
                           FIELDFONT=textfont,$
                           TITLE='Spline Tension:',$
                           UVALUE='Spline Tension',$
                           VALUE=10.0,$
                           XSIZE=8,$
                           EVENT_PRO='xgetscales_event',$
                           /CR_ONLY,$
                           TEXTID=textid)
       state.w.tension_fld = [fld,textid]

       if n_elements(vshift) eq 0 then begin
       
           v = coyote_field2(row,$
                             LABELFONT=buttonfont,$
                             FIELDFONT=textfont,$
                             TITLE='Vshift :',$
                             UVALUE='Vshift',$
                             XSIZE=8,$
                             EVENT_PRO='xgetscales_event',$
                             /CR_ONLY,$
                             TEXTID=textid)
           state.w.vshift_fld = [v,textid]

           v = coyote_field2(row,$
                             LABELFONT=buttonfont,$
                             FIELDFONT=textfont,$
                             TITLE='Vrot:',$
                             UVALUE='Vrot',$
                             XSIZE=8,$
                             EVENT_PRO='xgetscales_event',$
                             /CR_ONLY,$
                             TEXTID=textid)
           state.w.vrot_fld = [v,textid]


    
       endif

       atmos_bg = cw_bgroup(row,$
                            ['Atmosphere'],$
                            FONT=buttonfont,$
                            UVALUE='Atmosphere',$
                            SET_VALUE=[1],$
                            /NONEXCLUSIVE)

    
    state.w.plotwin2 = widget_draw(state.w.xgetscales_base,$
                                   XSIZE=state.p.plot2size[0],$
                                   YSIZE=state.p.plot2size[1],$
                                   /TRACKING_EVENTS,$
                                   /BUTTON_EVENTS,$
                                   /MOTION_EVENTS,$
                                   EVENT_PRO='xgetscales_plotwin2event',$
                                   UVALUE='Plot Window 2')
    
    row_base = widget_base(state.w.xgetscales_base,$
                           EVENT_PRO='xgetscales_event',$
                           /ROW)
         
       xmin = coyote_field2(row_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='X Min:',$
                            UVALUE='X Min',$
                            XSIZE=12,$
                            EVENT_PRO='xgetscales_minmaxevent',$
                            /CR_ONLY,$
                            TEXTID=textid)
       state.w.xmin_fld = [xmin,textid]
       
       xmax = coyote_field2(row_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='X Max:',$
                            UVALUE='X Max',$
                            XSIZE=12,$
                            EVENT_PRO='xgetscales_minmaxevent',$
                            /CR_ONLY,$
                            TEXTID=textid)
       state.w.xmax_fld = [xmax,textid]
       
       ymin = coyote_field2(row_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='Y Min:',$
                            UVALUE='Y Min',$
                            XSIZE=12,$
                            EVENT_PRO='xgetscales_minmaxevent',$
                            /CR_ONLY,$
                            TEXTID=textid)
       state.w.ymin_fld = [ymin,textid]
       
       ymax = coyote_field2(row_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='Y Max:',$
                            UVALUE='Y Max',$
                            XSIZE=12,$
                            EVENT_PRO='xgetscales_minmaxevent',$
                            /CR_ONLY,$
                            TEXTID=textid)
       state.w.ymax_fld = [ymax,textid]
       
       state.w.fix_bg = cw_bgroup(row_base,$
                                  ['Fix'],$
                                  FONT=buttonfont,$
                                  UVALUE='Fix',$
                                  /NONEXCLUSIVE)
       
    accept_button = widget_button(state.w.xgetscales_base,$
                                  FONT=buttonfont,$
                                  EVENT_PRO='xgetscales_event',$
                                  VALUE='Accept',$
                                  UVALUE='Accept')
    

    state.p.ytitle = (n_elements(YTITLE) ne 0) ? ytitle:''
    state.p.xtitle = (n_elements(XTITLE) ne 0) ? xtitle:''

; Get things running.  Center the widget using the Fanning routine.
      
    mkct
    centertlb,state.w.xgetscales_base
    widget_control, state.w.xgetscales_base, /REALIZE

    
    widget_control, state.w.order_dl,set_val=string(state.d.stdorders,$
                                                    format='(i2.2)')
    
    if n_elements(VSHIFT) eq 0 then widget_control, state.w.vshift_fld[1],$
      set_va=string(state.d.vshift,format='(f6.2)')

    if n_elements(VROT) eq 0 then widget_control, state.w.vrot_fld[1],$
      set_va=string(state.d.vrot,format='(f6.2)')


;  Get plotwin ids
    
    widget_control, state.w.plotwin1, GET_VALUE=x
    state.p.plotwin1_wid = x

    widget_control, state.w.plotwin2, GET_VALUE=x
    state.p.plotwin2_wid = x
    
    window, /free, /pixmap,xsize=state.p.plot1size[0],$
      ysize=state.p.plot1size[1]
    state.p.pixmap1_wid = !d.window

    window, /free, /pixmap,xsize=state.p.plot2size[0],$
      ysize=state.p.plot2size[1]
    state.p.pixmap2_wid = !d.window

;  Get sizes for things.
    
    widget_geom = widget_info(state.w.xgetscales_base, /geometry)
    state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
    state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]-$
      state.p.plot2size[1]

;  Get things running

    xgetscales_makescale
    xgetscales_maketelluric
    xgetscales_makespec
    xgetscales_getminmax
    xgetscales_setminmax
    xgetscales_plotupdate1
    xgetscales_plotupdate2

; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xgetscales', $
      state.w.xgetscales_base, $
      event_handler='xgetscales_resizeevent'

;  Now return required info

    scales = fltarr(n_elements(state.d.std[*,0,0]),state.d.norders)
    for i = 0, state.d.norders-1 do begin

        scales[*,i] = spline((*state.c.(i))[0,*],(*state.c.(i))[1,*],$
                             state.d.std[*,0,i],state.r.tension)
        
    endfor
    cancel = state.r.cancel
    vshift = state.d.vshift 
    vrot   = state.d.vrot

    ptr_free, state.r.scalespec

    ptr_free, state.c.a
    ptr_free, state.c.b
    ptr_free, state.c.c
    ptr_free, state.c.d
    ptr_free, state.c.e
    ptr_free, state.c.f


    ptr_free, state.d.deltascales
    ptr_free, state.d.tellspec

    state = 0B

endif

end

