; NAME:
;     xlat
;
; PURPOSE:
;     
;       
; CALLING SEQUENCE:
;     xlat
;
; INPUTS:
;     x      - A wavelength array
;     y      - A flux array
;     e      - An error array
;     xunits - An integer giving the units of the wavelength array
;              0 - microns
;              1 - Angstroms
;              2 - nanometers
;     yunits - An integer giving the units of the flux array
;              0 - W m-2 um-1
;              1 - ergs s-1 cm-2 A-1
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xlat_initcommon,nlines,xunits,yunits

case xunits of 

    0: x = 'um'

    1: x = 'A'

    2: x = 'nm'

endcase

case yunits of 

    0: y = 'W m-2 um-1'
    
    1: y = 'ergs s-1 cm-2 A-1'

endcase

common xlat_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.
;  p - contains info pertaining to the plots.

w = {contmin_fld:[0L,0L],$
     contmax_fld:[0L,0L],$
     errorlog:0L,$
     xlat_base:0L,$
     keyboard:0L,$
     linename_fld:[0L,0L],$
     momentlog:0L,$
     ofile_fld:[0L,0L],$
     plotwin_base:0L,$
     plotwin1:0,$
     rwave_fld:[0L,0L],$
     track:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin_fld:[0L,0L],$
     ymax_fld:[0L,0L]}

r = {anamode:'EW',$
     errors:0,$
     lineidx:0,$
     linereg:[!values.f_nan,!values.f_nan],$
     cursormode:'None',$
     nlines:nlines,$
     xunits:x,$
     yunits:y}

d = {linedat:replicate({ID:' ',$
                        rwave:!values.f_nan,$
                        wmin:!values.f_nan,$
                        wmax:!values.f_nan,$
                        m0_flux:!values.f_nan,em0_flux:!values.f_nan,$
                        m1_flux:!values.f_nan,em1_flux:!values.f_nan,$
                        rv_flux:!values.f_nan,erv_flux:!values.f_nan,$
                        m2_flux:!values.f_nan,em2_flux:!values.f_nan,$
                        m0_net:!values.f_nan,em0_net:!values.f_nan,$
                        m1_net:!values.f_nan,em1_net:!values.f_nan,$
                        rv_net:!values.f_nan,erv_net:!values.f_nan,$
                        m2_net:!values.f_nan,em2_net:!values.f_nan,$
                        EW:!values.f_nan,eEW:!values.f_nan,$
                        m0_cont:!values.f_nan,em0_cont:!values.f_nan},nlines),$
     wwave:ptr_new(fltarr(2)),$
     e:ptr_new(fltarr(2)),$
     x:ptr_new(fltarr(2)),$
     y:ptr_new(fltarr(2))}

p = {absxrange1:[0.,0.],$
     absyrange1:[0.,0.],$
     buffer:[0.,0.],$
     pixmap1_wid:0L,$
     plotwin1_wid:0L,$
     xrange1:[0.,0.],$
     yrange1:[0.,0.],$
     plot1size:[700,400],$
     pscale1:!p,$
     xscale1:!x,$
     xtitle:'',$
     yscale1:!y,$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}


;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

end
;
;******************************************************************************
;
pro xlat_displayresults

common xlat_state

strlog = strarr(18)

strlog[0]  = strtrim(state.d.linedat[state.r.lineidx].ID,2)
strlog[1]  = ' '
strlog[2]  = strtrim(state.d.linedat[state.r.lineidx].rwave,2)
strlog[3]  = strtrim(state.d.linedat[state.r.lineidx].wmin,2)
strlog[4]  = strtrim(state.d.linedat[state.r.lineidx].wmax,2)
strlog[5]  = ' '
strlog[6]  = strtrim(state.d.linedat[state.r.lineidx].m0_flux,2)
strlog[7]  = strtrim(state.d.linedat[state.r.lineidx].m1_flux,2)
strlog[8]  = strtrim(state.d.linedat[state.r.lineidx].rv_flux,2)
strlog[9]  = strtrim(state.d.linedat[state.r.lineidx].m2_flux,2)
strlog[10] = ' '
strlog[11] = strtrim(state.d.linedat[state.r.lineidx].m0_net,2)
strlog[12] = strtrim(state.d.linedat[state.r.lineidx].m1_net,2)
strlog[13] = strtrim(state.d.linedat[state.r.lineidx].rv_net,2)
strlog[14] = strtrim(state.d.linedat[state.r.lineidx].m2_net,2)
strlog[15] = ' '
strlog[16] = strtrim(state.d.linedat[state.r.lineidx].EW,2)
strlog[17] = strtrim(state.d.linedat[state.r.lineidx].m0_cont,2)

widget_control, state.w.momentlog,set_value=strlog


if state.r.errors then begin

    strlog = strarr(18)
    strlog[0]  = ''
    strlog[1]  = ''
    strlog[2]  = ''
    strlog[3]  = ''
    strlog[4]  = ''
    strlog[5]  = ''
    strlog[6]  = strtrim(state.d.linedat[state.r.lineidx].em0_flux,2)
    strlog[7]  = strtrim(state.d.linedat[state.r.lineidx].em1_flux,2)
    strlog[8]  = strtrim(state.d.linedat[state.r.lineidx].erv_flux,2)
    strlog[9]  = strtrim(state.d.linedat[state.r.lineidx].em2_flux,2)
    strlog[10] = ' '
    strlog[11] = strtrim(state.d.linedat[state.r.lineidx].em0_net,2)
    strlog[12] = strtrim(state.d.linedat[state.r.lineidx].em1_net,2)
    strlog[13] = strtrim(state.d.linedat[state.r.lineidx].erv_net,2)
    strlog[14] = strtrim(state.d.linedat[state.r.lineidx].em2_net,2)
    strlog[15] = ' '
    strlog[16] = strtrim(state.d.linedat[state.r.lineidx].eEW,2)
    strlog[17] = strtrim(state.d.linedat[state.r.lineidx].em0_cont,2)
    
    widget_control, state.w.errorlog,set_value=strlog

endif

end
;
;******************************************************************************
;
pro xlat_help

common xlat_state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xlat_help') then begin

    help_base = widget_base(group_leader = state.w.xlat_base, $
                            /column, $
                            title ='X Line Analysis Tool - Help')
    
    h = [['X Line Analysis Tool is a fully resizing widget.'],$
         [' '],$
         ['Keyboard commands:'],$
         [' '],$
         ['c - Clear mouse mode.'],$
         ['    Use to clear a zoom, fix, or remove session.'],$
         [' '],$
         ['i - To zoom IN in whatever zoom mode the cursor is currently'],$
         ['    in.'],$
         [' '],$
         ['h - To lauch the help window.'],$
         [' '],$
         ['o - To zoom OUT in whatever zoom mode the cursor is currently'],$
         ['    in.'],$
         [' '],$
         ['w - To plot the entire spectrum'],$
         [' '],$
         ['x - Enters x zoom mode'],$
         ['    Press left mouse button at lower x value and then at upper'],$
         ['    x value.'],$
         [' '],$
         ['y - Enters y zoom mode'],$
         ['    Press left mouse button at lower y value and then at upper'],$
         ['    y value.'],$
         [' '],$
         ['z - Enters zoom mode'],$
         ['    Press the left mouse button in one corner of the zoom box '],$
         ['    and then move the cursor to the other corner and press the '],$
         ['    the left mouse button.'],$
         [' ']]

    help_text = widget_text(help_base, $
                            /scroll, $
                            value = h, $
                            xsize = 70, $
                            ysize = 24)
    
    quit = widget_button(help_base,$
                         value='Done',$
                         font=buttonfont,$
                         uvalue='Done')
      
    centertlb,help_base

    widget_control, help_base, /Realize
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xzoomplot_help', $
      help_base, $
      /No_Block,$
      event_handler='xlat_event'
    
endif

end
;
;******************************************************************************
;
pro xlat_moments

common xlat_state

id = cfld(state.w.linename_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
rwave = cfld(state.w.rwave_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return

wmin = cfld(state.w.contmin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return

wmax = cfld(state.w.contmax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return

e = *state.d.e
if not state.r.errors then delvarx,e
results = linemoments(*state.d.x,*state.d.y,wmin,wmax,rwave,FERR=e,$
                      MERR=merr,CANCEL=cancel)

;  Convert the EW to Angstroms

case state.r.xunits of 

    'um': begin

        results[8] = results[8]*1E4
        if n_elements(merr) ne 0 then merr[8] = merr[8]*1e4

    end

    'A':  

    'nm': begin

        results[8] = results[8]*10.
        if n_elements(merr) ne 0 then merr[8] = merr[8]*10.

    end

endcase

;  Load results and display results

state.d.linedat[state.r.lineidx].ID    = id
state.d.linedat[state.r.lineidx].rwave = rwave
state.d.linedat[state.r.lineidx].wmin  = wmin
state.d.linedat[state.r.lineidx].wmax  = wmax

for i = 0, 9 do state.d.linedat[state.r.lineidx].((i*2+4)) = results[i]

if state.r.errors then for i = 0, 9 do $
  state.d.linedat[state.r.lineidx].((i*2+5)) = merr[i]

xlat_displayresults

end
;
;******************************************************************************
;
pro xlat_plotspec

common xlat_state

plot, *state.d.x,*state.d.y,/xsty,/ysty,xrange=state.p.xrange1,$
  yrange=state.p.yrange1,psym=10

;  Plot Line Region if necessary

z = where(finite(state.r.linereg) eq 1,count)
for i = 0, count-1 do begin

    plots, [state.r.linereg[z[i]],state.r.linereg[z[i]]],state.p.yrange1,$
      color=7,linestyle=2

endfor

;  Plot the continuum 

if count eq 2 then begin

    tabinv,*state.d.x,state.r.linereg,idx
    ridx = round(idx)

    m = ( (*state.d.y)[ridx[1]]-(*state.d.y)[ridx[0]] ) / $
      ( (*state.d.x)[ridx[1]]-(*state.d.x)[ridx[0]] )
    b = (*state.d.y)[ridx[0]] - ( m * (*state.d.x)[ridx[0]] )
    oplot, (*state.d.x)[ridx[0]:ridx[1]],$
      poly((*state.d.x)[ridx[0]:ridx[1]],[b,m]),color=3


endif



state.p.pscale1 = !p
state.p.xscale1 = !x
state.p.yscale1 = !y

end
;
;******************************************************************************
;
pro xlat_plotupdate

common xlat_state

wset, state.p.pixmap1_wid
xlat_plotspec


wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

end
;
;******************************************************************************
;
pro xlat_plotwin

common xlat_state

buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'

state.w.plotwin_base = widget_base(GROUP_LEADER=state.w.xlat_base, $
                                   TITLE='Plot Window',$
                                   /COLUMN,$
                                   /TLB_SIZE_EVENTS)

   state.w.track = widget_text(state.w.plotwin_base, $
                               YSIZE=1)
   
   state.w.keyboard = widget_text(state.w.plotwin_base, $
                                  /ALL_EVENTS, $
                                  SCR_XSIZE=1, $
                                  SCR_YSIZE=1, $
                                  UVALUE='Keyboard', $
                                  EVENT_PRO='xlat_event',$
                                  VALUE= '')

   state.w.plotwin1 = widget_draw(state.w.plotwin_base,$
                                  XSIZE=state.p.plot1size[0],$
                                  YSIZE=state.p.plot1size[1],$
                                  /TRACKING_EVENTS,$
                                  /BUTTON_EVENTS,$
                                  /MOTION_EVENTS,$
                                  EVENT_PRO='xlat_plotwinevent1',$
                                  UVALUE='Plot Window 1')
   
   row_base = widget_base(state.w.plotwin_base,$
                          FRAME=1,$
                          /ROW)
   
      xmin = coyote_field2(row_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='X Min:',$
                           uval = 'X Min',$
                           xsize=12,$
                           event_pro = 'xlat_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.xmin_fld = [xmin,textid]
      
      xmax = coyote_field2(row_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='X Max:',$
                           uval = 'X Max',$
                           xsize=12,$
                           event_pro = 'xlat_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.xmax_fld = [xmax,textid]
      
      ymin = coyote_field2(row_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='Y Min:',$
                           uval = 'Y Min',$
                           xsize=12,$
                           event_pro = 'xlat_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.ymin_fld = [ymin,textid]
      
      ymax = coyote_field2(row_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='Y Max:',$
                           uval = 'Y Max',$
                           xsize=12,$
                           event_pro = 'xlat_minmax_event',$
                           /cr_only,$
                           textid=textid)
      state.w.ymax_fld = [ymax,textid]
         
; Get things running.  Center the widget using the Fanning routine.

mkct      
centertlb,state.w.plotwin_base
widget_control, state.w.plotwin_base, /REALIZE

;  Get plotwin ids
    
widget_control, state.w.plotwin1, GET_VALUE=xx
state.p.plotwin1_wid = xx

window, /free, /pixmap,xsize=state.p.plot1size[0],$
  ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

;  Get sizes for things.
    
widget_geom = widget_info(state.w.plotwin_base, /geometry)
state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]

; Start the Event Loop. This will be a non-blocking program.
    
XManager, 'plotwin', $
  /NO_BLOCK,$
  state.w.plotwin_base,$
  event_handler='xlat_resize'

end
;
;******************************************************************************
;
pro xlat_setminmax

common xlat_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.xrange1[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.xrange1[1],2)
widget_control, state.w.ymin_fld[1],set_value=strtrim(state.p.yrange1[0],2)
widget_control, state.w.ymax_fld[1],set_value=strtrim(state.p.yrange1[1],2)

end
;
;******************************************************************************
;
pro xlat_zoom,IN=in,OUT=out

common xlat_state

delabsx = state.p.absxrange1[1]-state.p.absxrange1[0]
delx    = state.p.xrange1[1]-state.p.xrange1[0]

delabsy = state.p.absyrange1[1]-state.p.absyrange1[0]
dely    = state.p.yrange1[1]-state.p.yrange1[0]

xcen = state.p.xrange1[0]+delx/2.
ycen = state.p.yrange1[0]+dely/2.

case state.r.cursormode of 

    'XZoom': begin

        z = alog10(delabsx/delx)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsx/2.^z/2.
        state.p.xrange1 = [xcen-hwin,xcen+hwin]
        xlat_plotupdate

    end

    'YZoom': begin

        z = alog10(delabsy/dely)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsy/2.^z/2.
        state.p.yrange1 = [ycen-hwin,ycen+hwin]
        xlat_plotupdate

    end

    'Zoom': begin

        z = alog10(delabsx/delx)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsx/2.^z/2.
        state.p.xrange1 = [xcen-hwin,xcen+hwin]

        z = alog10(delabsy/dely)/alog10(2)
        if keyword_set(IN) then z = z+1 else z=z-1
        hwin = delabsy/2.^z/2.
        state.p.yrange1 = [ycen-hwin,ycen+hwin]

        xlat_plotupdate

    end

    else:

endcase

end
;
;******************************************************************************
;
pro xlat_writefile

common xlat_state

ofile = cfld(state.w.ofile_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

z = where( finite((state.d.linedat).wmin) eq 1,count)
if count eq 0 then begin

    print, 'No data...'
    goto, cont

endif  
n = n_tags(state.d.linedat[0])

openw,lun,ofile,/GET_LUN

for i = 0,count-1 do begin

    str = strarr(n)
    for j = 0, n-1 do str[j] = strtrim(state.d.linedat[z[i]].(j),2)

;  Screwy format thing to get the ID left justified AND not have to 
;  format the rest of the data

    data = strjoin(str[1:*],'  ')
    dlength = strlen(data)

    idlength = strlen(str[0])
    space = 2+(10-idlength)
    format ='(a'+strtrim(idlength,2)+','+strtrim(space,2)+'x,a'+$
      strtrim(dlength,2)+')'

    printf, lun,[str[0],data],format=format

endfor

free_lun, lun

print, 'Wrote '+ofile+' to disk.'

cont:

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xlat_event, event

common xlat_state

widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Clear': begin

        for i = 2, 13 do state.d.linedat[state.r.lineidx].(i) = !values.f_nan
        widget_control, state.w.momentlog,SET_VALUE=strarr(13)
        widget_control, state.w.errorlog,SET_VALUE=strarr(13)
        widget_control, state.w.contmin_fld[1],SET_VALUE=''
        widget_control, state.w.contmax_fld[1],SET_VALUE=''
        state.r.linereg[*] = !values.f_nan
        xlat_plotupdate

    end

    'Compute Moments': xlat_moments

    'Cont Min': begin

        val = cfld(state.w.contmin_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.r.linereg[0] = val
        xlat_plotupdate

    end

    'Cont Max': begin

        val = cfld(state.w.contmax_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.r.linereg[1] = val
        xlat_plotupdate

    end

    'Done': widget_control, event.top, /DESTROY

    'Keyboard': begin

        case strtrim(event.ch,2) of 
            
            'c': begin

                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xlat_plotupdate
                
            end

            'i': xlat_zoom,/IN

            'h': xlat_help ; Help

            'o': xlat_zoom,/OUT

            's': begin

                state.r.cursormode = 'Select'
                state.p.reg[*] = !values.f_nan

            end

            'w': begin
                
                state.p.xrange1 = [min((*state.d.x),max=max),max]
                state.p.yrange1 = [0,max(*state.d.y,/NAN)]
                xlat_plotupdate

            end

            'x': begin 

                state.r.cursormode = 'XZoom'
                state.p.reg[*] = !values.f_nan
                                
            end

            'y': begin 

                state.r.cursormode = 'YZoom'
                state.p.reg[*] = !values.f_nan

            end

            'z': begin

                state.r.cursormode = 'Zoom'
                state.p.reg[*] = !values.f_nan
               
            end

            '?': xlat_help ; Help
            
            else:

        endcase

    end

    'Line Number': begin

        state.r.lineidx = event.index
        if finite(state.d.linedat[state.r.lineidx].(1)) eq 0 then begin

            widget_control, state.w.momentlog,SET_VALUE=strarr(13)
            widget_control, state.w.errorlog,SET_VALUE=strarr(13)
            widget_control, state.w.contmin_fld[1],SET_VALUE=''
            widget_control, state.w.contmax_fld[1],SET_VALUE=''
            widget_control, state.w.linename_fld[1],SET_VALUE=''
            widget_control, state.w.rwave_fld[1], SET_VALUE=''
            state.r.linereg[*] = !values.f_nan
            xlat_plotupdate
            
        endif else begin

            xlat_displayresults
            state.r.linereg = [state.d.linedat[state.r.lineidx].wmin,$
                               state.d.linedat[state.r.lineidx].wmax]
            xlat_plotupdate
        
        endelse

    end

    'Write File': xlat_writefile
    
    else:

endcase

cont: 

end
;
;******************************************************************************
;
pro xlat_minmax_event,event

common xlat_state

xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmin2 = crange(xmin,state.p.xrange1[1],'X Min',/KLT,$
               WIDGET_ID=state.w.xlat_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmin_fld[0],set_value=state.p.xrange1[0]
    return

endif else state.p.xrange1[0] = xmin2

xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmax2 = crange(xmax,state.p.xrange1[0],'X Max',/KGT,$
               WIDGET_ID=state.w.xlat_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmax_fld[0],set_value=state.p.xrange1[1]
    return

endif else state.p.xrange1[1] = xmax2

ymin = cfld(state.w.ymin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymin2 = crange(ymin,state.p.yrange1[1],'Y Min',/KLT,$
               WIDGET_ID=state.w.xlat_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymin_fld[0],set_value=state.p.yrange1[0]
    return

endif else state.p.yrange1[0] = ymin2

ymax = cfld(state.w.ymax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymax2 = crange(ymax,state.p.yrange1[0],'Y Max',/KGT,$
               WIDGET_ID=state.w.xlat_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymax_fld[0],set_value=state.p.yrange1[1]
    return

endif else state.p.yrange1[1] = ymax2

xlat_plotupdate


end
;
;******************************************************************************
;
pro xlat_plotwinevent1, event

common xlat_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    wset, state.p.plotwin1_wid
    erase
    device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1
x  = event.x/float(state.p.plot1size[0])
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

case state.r.cursormode of 

    'Select': begin

        if event.type ne 1 then goto, out        
        z = where(finite(state.r.linereg) eq 1,count)        
        if count eq 0 then begin
            
            state.r.linereg[0] = xy[0]
            xlat_plotupdate

        endif
        if count eq 1 then begin

            state.r.linereg[1] = xy[0]
            state.r.linereg = state.r.linereg[sort(state.r.linereg)]
            widget_control, state.w.contmin_fld[1],$
              SET_VALUE=strtrim(state.r.linereg[0],2)
            widget_control, state.w.contmax_fld[1],$
              SET_VALUE=strtrim(state.r.linereg[1],2)
            xlat_plotupdate
            xlat_moments

        endif

    end

    'XZoom': begin
        
        if event.type ne 1 then goto, out        
        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then begin
            
            state.p.reg[*,0] = xy[0:1]
            wset, state.p.pixmap1_wid
            plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,$
              /DEVICE,linestyle=1,thick=2
            wset, state.p.plotwin1_wid
            device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                          state.p.pixmap1_wid]
            
        endif else begin
            
            state.p.reg[*,1]   = xy[0:1]
            state.p.xrange1    = [min(state.p.reg[0,*],max=m),m]
            state.p.reg[*]     = !values.f_nan
            state.r.cursormode = 'None'
            xlat_plotupdate
            xlat_setminmax
            
        endelse
        
        
    end
    
    'YZoom': begin
        
        if event.type ne 1 then goto, out        
        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then begin
            
            state.p.reg[*,0] = xy[0:1]
            wset, state.p.pixmap1_wid
            plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,$
              /DEVICE,linestyle=1,thick=2
            
            wset, state.p.plotwin1_wid
            device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                          state.p.pixmap1_wid]
            
        endif else begin
            
            state.p.reg[*,1]   = xy[0:1]
            state.p.yrange1    = [min(state.p.reg[1,*],max=m),m]
            state.r.cursormode = 'None'
            state.p.reg[*]     = !values.f_nan
            xlat_plotupdate
            xlat_setminmax
            
        endelse
        
    end
    
    'Zoom': begin

        if event.type ne 1 then goto, out        
        z = where(finite(state.p.reg) eq 1,count)
        if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin
            
            state.p.reg[*,1] = xy[0:1]
            state.p.xrange1 = [min(state.p.reg[0,*],max=max),max]
            state.p.yrange1 = [min(state.p.reg[1,*],max=max),max]
            state.r.cursormode = 'None'
            state.p.reg[*] = !values.f_nan
            xlat_plotupdate
            xlat_setminmax
            
        endelse
        
    end

    else:

endcase

out:

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]


case state.r.cursormode of 

    'XZoom': plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

    'YZoom': plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE

    'Zoom': begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
        xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
        plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
          linestyle=2,color=2
        plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
          linestyle=2,color=2
        
    end

    else: begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE

    end

endcase

tabinv, *state.d.x,xy[0],idx
idx = round(idx)
label = 'Cursor X: '+strtrim(xy[0],2)+', Y:'+strtrim(xy[1],2)
label = label+'   Spectrum X: '+strtrim( (*state.d.x)[idx],2)+$
  ', Y:'+strtrim( (*state.d.y)[idx],2)
widget_control,state.w.track,set_value=label

cont:
    
end
;
;******************************************************************************
;
pro xlat_resize,event

common xlat_state

widget_control, state.w.plotwin_base, tlb_get_size = size

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=size[1]-state.p.buffer[1]

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

xlat_plotupdate

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xlat,x,y,e,xunits,yunits,YTITLE=ytitle,XTITLE=xtitle,$
            NLINES=nlines,CANCEL=cancel

common xlat_state

if not xregistered('xlat') then begin
        
    if n_elements(NLINES) eq 0 then nlines = 20
    if n_elements(XUNITS) eq 0 then xunits = 0
    if n_elements(YUNITS) eq 0 then yunits = 0
    
    xlat_initcommon, nlines,xunits,yunits

;  Build the widget.

    buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
    textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'


    state.w.xlat_base = widget_base(title='X Line Analysis Tool', $
                                       /COLUMN,$
                                       EVENT_PRO='xlat_event')

       quit_button = widget_button(state.w.xlat_base,$
                                   FONT=buttonfont,$
                                   VALUE='Done',$
                                   UVALUE='Done')

       row_base = widget_base(state.w.xlat_base,$
                              /ROW)

          col1_base = widget_base(row_base,$
                                  FRAME=1,$
                                  /COLUMN)

             value = string(findgen(nlines)+1,format='(i2.2)')
             line_dl = widget_droplist(col1_base,$
                                       font=buttonfont,$
                                       title='Line Number:',$
                                       value=value,$
                                       uvalue='Line Number')
             
             line_name = coyote_field2(col1_base,$
                                     labelfont=buttonfont,$
                                     fieldfont=textfont,$
                                     title='Line Name:',$
                                     uvalue='Line Name',$
                                     xsize=15,$
                                     textID=textid)
             state.w.linename_fld = [line_name,textid]
             
             wave = coyote_field2(col1_base,$
                                  labelfont=buttonfont,$
                                  fieldfont=textfont,$
                                  title='Rest Wave:',$
                                  uvalue='Rest Wavelength',$
                                  xsize=15,$
                                  textID=textid)
             state.w.rwave_fld = [wave,textid]

             min = coyote_field2(col1_base,$
                                  labelfont=buttonfont,$
                                  fieldfont=textfont,$
                                  title='Wave Min:',$
                                  event_pro = 'xlat_event',$
                                  uvalue='Cont Min',$
                                  xsize=15,$
                                  /cr_only,$
                                  textID=textid)
             state.w.contmin_fld = [min,textid]
             
             max = coyote_field2(col1_base,$
                                  labelfont=buttonfont,$
                                  fieldfont=textfont,$
                                  title='Wave Max:',$
                                  event_pro = 'xlat_event',$
                                  uvalue='Cont Max',$
                                  xsize=15,$
                                  /cr_only,$
                                  textID=textid)
             state.w.contmax_fld = [max,textid]

                
             compute = widget_button(col1_base,$
                                     VALUE='Compute Moments',$
                                     FONT=buttonfont,$
                                     UVALUE='Compute Moments')             

             ofile = coyote_field2(col1_base,$
                                   labelfont=buttonfont,$
                                   fieldfont=textfont,$
                                   title='Output File:',$
                                   event_pro = 'xlat_event',$
                                   uvalue='Output File',$
                                   xsize=15,$
                                   /cr_only,$
                                   textID=textid)
             state.w.ofile_fld = [ofile,textid]
             
                
             button = widget_button(col1_base,$
                                    VALUE='Write File',$
                                    FONT=buttonfont,$
                                    UVALUE='Write File')

             
          col2_base = widget_base(row_base,$
                                  FRAME=1,$
                                  /COLUMN)

             row = widget_base(col2_base,$
                               /ROW)

;  Set up labels

                w  = state.r.xunits
                f  = strsplit(state.r.yunits,' ',/EXTRACT)
                n  = n_elements(f)
                f  = strjoin(f[0:n-2],' ')
                
                value = ['Line',' ','Rest Wave ('+w+')',$
                         'Min Wave ('+w+')','Max Wave ('+w+')',' ',$
                         'M0_flux ('+f+')','M1_flux ('+f+')',$
                         'RV_flux (km s-1)','M2_flux ('+w+')',' ',$
                         'M0_net ('+f+')','M1_net ('+f+')',$
                         'RV_net (km s-1)','M2_net ('+w+')',' ',$
                         'EW (A)','M0_cont ('+f+')']
                
                
                label = widget_text(row,$
                                    xsize=20,$
                                    ysize=n_elements(value), $
                                    value=value)
                
                state.w.momentlog = widget_text(row,$
                                                xsize=15,$
                                                ysize=n_elements(value))
                
                value = ['',' ','','','','','+-','+-','+-','+-','',$
                         '+-','+-','+-','+-','','+-','+-']

                label = widget_text(row,$
                                    xsize=2,$
                                    ysize=n_elements(value), $
                                    value=value)

                state.w.errorlog = widget_text(row,$
                                               xsize=15,$
                                               ysize=n_elements(value))

             clear = widget_button(col2_base,$
                                   VALUE='Clear',$
                                   FONT=buttonfont,$
                                   UVALUE='Clear')


;  Load data


    *state.d.x         = x
    *state.d.y         = y
    if n_elements(E) ne 0 then begin

        *state.d.e = e
        state.r.errors = 1

    endif
    state.p.absxrange1 = [min(x,max=max),max]
    state.p.absyrange1 = [0,max(y,/NAN)]
    state.p.xrange1    = state.p.absxrange1
    state.p.yrange1    = state.p.absyrange1

    centertlb,state.w.xlat_base
    widget_control, state.w.xlat_base, /REALIZE

; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xlat', $
      /NO_BLOCK,$
      state.w.xlat_base

; Start Plot Window

    xlat_plotwin
    xlat_plotupdate
    xlat_setminmax

endif

end








