; NAME:
;     xmag2flux
;
; PURPOSE:
;     To flux calibate a SpeX spectrum
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xmag2flux
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     Later
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xmag2flux_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Magnitude Field': xmag2flux_flux,state

    'Magnitude Filter': begin

        state.r.currentfilter = byte(event.value)
        xmag2flux_flux,state

    end
    'Path Field': setfocus,state.w.spectrum_fld

    'Quit': begin

        widget_control, event.top, /DESTROY
        goto, getout
        
    end

    'System': begin

        state.r.currentsystem = total(event.index)
        xmag2flux_changesystem,state
        xmag2flux_flux,state

    end

    'Units': begin

        case event.index of 

            0: state.r.units = 'ergs s-1 cm-2 A-1'
            1: state.r.units = 'ergs s-1 cm-2 Hz-1'
            2: state.r.units = 'W m-2 um-1'
            3: state.r.units = 'W m-2 Hz-1'
            4: state.r.units = 'Jy'
        
        endcase 
        xmag2flux_flux,state

    end

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xmag2flux_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xmag2flux_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin

endif
state = 0B

end
;
;******************************************************************************
;
pro xmag2flux_changesystem,state

widget_control, state.w.magfiltrow_base,UPDATE=0
widget_control, state.w.magfilt_bg,DESTROY=1

state.w.magfilt_bg = cw_bgroup(state.w.magfiltrow_base,$
                               FONT=state.w.buttonfont,$
                               (state.d.filters).(state.r.currentsystem),$
                               /ROW,$
                               /RETURN_INDEX,$
                               /NO_RELEASE,$
                               SET_VALUE=0,$
                               /EXCLUSIVE,$
                               UVALUE='Magnitude Filter')

widget_control, state.w.magfiltrow_base,UPDATE=1


state.r.currentfilter = 0 
state.r.nfilters   = n_elements((state.d.filters).(state.r.currentsystem))

;  Load the filter profiles

for i = 0, state.r.nfilters-1 do begin

    rdfloat,state.r.packagepath+'/data/'+$
      ((*state.d.files).(state.r.currentsystem))[i],w,t
    key = 'filter'+string(i,format='(i2.2)')

    struc = (i eq 0) ? create_struct(key,[[w],[t]]):$
      create_struct(struc,key,[[w],[t]])

endfor

*state.d.filtertrans = struc

end
;
;******************************************************************************
;
pro xmag2flux_flux,state

mag = cfld(state.w.mag_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return

trans   = *state.d.filtertrans
w_trans = (trans.(state.r.currentfilter))[*,0]*1e4 
t_trans = (trans.(state.r.currentfilter))[*,1] 

w_vega  = state.d.wvega
t_vega  = state.d.fvega

case state.r.units of 



endcase







return

interpspec,state.d.wvega/1e4,state.d.fvega,w,fvega

z = where(finite(fvega) eq 1)
min = min(w,max=max)

l_eff = traper(w,w*fvega[z]*t,min,max)/traper(w,fvega[z]*t,min,max)

wavelength = 'Effective Wavelength: '+strtrim(l_eff,2)+' um'
widget_control, state.w.wavelength, SET_VALUE=wavelength

fvega = traper(w,fvega[z],min,max)

flux = fvega* 10^(-0.4*(mag-(state.r.vegazero)[state.r.currentsystem]))

flux = 'Flux Density: '+strtrim(flux,2)

widget_control, state.w.flux,SET_VALUE=flux    





end
;
;******************************************************************************
;
; ------------------------------Main Program---------------------------------- 
;
;******************************************************************************
;
;
pro xmag2flux

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.
;  p - contains info pertaining to plotting.


last   = strpos(!path,'Spantool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

packagepath = cpath(result,CANCEL=cancel)
if cancel then return

rdphotfile,packagepath+'data/photsystems.dat',nsystems,systems,filters,$
  files,vegazero
if cancel then return

restore, packagepath+'/data/vega5.sav'



w = {buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     flux:0L,$
     magfilt_bg:0L,$
     mag_fld:[0L,0L],$
     magfiltrow_base:0L,$
     textfont:'-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1',$
     wavelength:0L,$
     xmag2flux_base:0L}

r = {currentsystem:0,$
     currentfilter:0,$
     nfilters:n_elements(filters.(0)),$
     packagepath:packagepath,$
     units:'ergs s-1 cm-2 A-1',$
     vegazero:vegazero}

d = {files:ptr_new(fltarr(2)),$
     filters:filters,$
     filtertrans:ptr_new(fltarr(2)),$
     fvega:fvin,$
     systems:systems,$
     wvega:wvin}

*d.files = files

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d}

state.w.xmag2flux_base = widget_base(title='Xmag2flux', $
                                     /COLUMN,$
                                     EVENT_PRO='xmag2flux_event')
    
   quit_button = widget_button(state.w.xmag2flux_base,$
                               FONT=state.w.buttonfont,$
                               VALUE='Done',$
                               UVALUE='Quit')
       
   systems_dl = widget_droplist(state.w.xmag2flux_base,$
                                FONT=state.w.buttonfont,$
                                TITLE='System:',$
                                VALUE=state.d.systems,$
                                UVALUE='System')   

      state.w.magfiltrow_base = widget_base(state.w.xmag2flux_base,$
                                            /ROW,$
                                            /BASE_ALIGN_CENTER)
      
         state.w.magfilt_bg = cw_bgroup(state.w.magfiltrow_base,$
                                        FONT=state.w.buttonfont,$
                                        (state.d.filters).(0),$
                                        /ROW,$
                                        /RETURN_INDEX,$
                                        /NO_RELEASE,$
                                        SET_VALUE=0,$
                                        /EXCLUSIVE,$
                                        UVALUE='Magnitude Filter')
         
      fld = coyote_field2(state.w.xmag2flux_base,$
                          LABELFONT=state.w.buttonfont,$
                          FIELDFONT=state.w.textfont,$
                          TITLE='Magnitude:',$
                          VALUE='0.0',$
                          UVALUE='Magnitude Field',$
                          XSIZE=10,$
                          EVENT_PRO='xmag2flux_event',$
                          /CR_ONLY,$
                          TEXTID=textid)
      state.w.mag_fld = [fld,textid]
      
      value =['ergs s-1 cm-2 A-1','ergs s-1 cm-2 Hz-1',$
              'W m-2 um-1','W m-2 Hz-1','Jy']
      units_dl = widget_droplist(state.w.xmag2flux_base,$
                                 font=state.w.buttonfont,$
                                 title='Units:',$
                                 value=value,$
                                 uvalue='Units')

      state.w.wavelength = widget_label(state.w.xmag2flux_base,$
                                        /ALIGN_LEFT,$
                                        VALUE='Effective Wavelength:',$
                                        FONT=state.w.buttonfont,$
                                        /DYNAMIC_RESIZE)

      state.w.flux = widget_label(state.w.xmag2flux_base,$
                                  /ALIGN_LEFT,$
                                  VALUE='Flux Density:',$
                                  FONT=state.w.buttonfont,$
                                  /DYNAMIC_RESIZE)
      
; Get things running.  Center the widget using the Fanning routine.
            
centertlb,state.w.xmag2flux_base

widget_control, state.w.xmag2flux_base, /REALIZE
xmag2flux_changesystem,state

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xmag2flux', $
  state.w.xmag2flux_base, $
  cleanup='xmag2flux_cleanup',$
  /NO_BLOCK

xmag2flux_flux,state

; Put state variable into the user value of the top level base.

widget_control, state.w.xmag2flux_base, set_uvalue=state, /no_copy



end
