;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xmergeorders_addorder,state

z = where(*state.r.orders eq state.r.addorder)

nonan               = where(finite((*state.d.origspec)[*,1,z]) eq 1)
*state.d.greenspec = reform((*state.d.origspec)[nonan,*,z])

;  Get position of greenspec with respect to bluespec

if min((*state.d.greenspec)[*,0]) lt min((*state.d.mergedspec)[*,0],/NAN) $
  then state.p.greenpos = 'Left' else $
  if max((*state.d.greenspec)[*,0]) gt max((*state.d.mergedspec)[*,0],/NAN) $
  then state.p.greenpos = 'Right' else state.p.greenpos = 'Inside'

;  Make title

if state.p.greenpos eq 'Left' then $
  orders = [state.r.addorder,state.r.addorder-1]
if state.p.greenpos eq 'Right' then $
  orders = [state.r.addorder+1,state.r.addorder]
if state.p.greenpos eq 'Inside' then $
  orders = [8,7]

state.p.title='!5Overlap - Orders '+string(orders[0],format='(i2.2)')+$
  ' and '+string(orders[1],format='(i2.2)')

state.p.reg = [[!values.f_nan,!values.f_nan],$
               [!values.f_nan,!values.f_nan]]

xmergeorders_getminmax1,state
xmergeorders_getminmax2,state
xmergeorders_plotupdate,state
xmergeorders_setminmax,state


end
;
;******************************************************************************
;
pro xmergeorders_autoscale,state

state.r.scale = 1
xmergeorders_cutscalespec,state

z = where(finite(state.p.reg) eq 0,count)

if count ne 0 then begin

    result=dialog_message("Please select a region using 's'.",$
                          /INFORMATION,DIALOG_PARE=state.w.xmergeorders_base)
    return

endif
xrange = state.p.reg[0,sort(state.p.reg[0,*])]

zb = where((*state.d.bluespec)[*,0] gt xrange[0] and $
           (*state.d.bluespec)[*,0] lt xrange[1])

zy = where((*state.d.greenspec)[*,0] gt xrange[0] and $
           (*state.d.greenspec)[*,0] lt xrange[1])

bluespec_w   = reform((*state.d.bluespec)[zb,0])
bluespec_f   = reform((*state.d.bluespec)[zb,1])
bluespec_e   = reform((*state.d.bluespec)[zb,2])

greenspec_w = reform((*state.d.greenspec)[zy,0])
greenspec_f = reform((*state.d.greenspec)[zy,1])
greenspec_e = reform((*state.d.greenspec)[zy,2])

interpspec,greenspec_w,greenspec_f,bluespec_w,ngreenspec_f,ngreenspec_e,$
  YAERROR=greenspec_e

z = where(finite(bluespec_f) eq 1 and finite(ngreenspec_f) eq 1 and $
          finite(bluespec_e) eq 1 and finite(ngreenspec_e) eq 1)

denom = bluespec_e^2 + ngreenspec_e^2

scale = total(ngreenspec_f[z]*bluespec_f[z]/denom[z])/$
  total(ngreenspec_f[z]^2/denom[z])

state.r.scale = scale
widget_control, state.w.scale_fld[1],SET_VALUE=strtrim(state.r.scale,2)

state.r.cursormode = 'None'

xmergeorders_cutscalespec,state
xmergeorders_plotupdate,state

end
;
;******************************************************************************
;
pro xmergeorders_changeaddorder,state


if n_elements(*state.r.mergedorders) ne state.r.norders then begin

    possible = [min(*state.r.mergedorders)-1,max(*state.r.mergedorders)+1]
    z = where(possible ge min(*state.r.orders) and $
              possible le max(*state.r.orders))
    val = string(possible[z],format='(i2.2)')
    
    widget_control, state.w.addorder_base,UPDATE=0
    widget_control, state.w.addorder_bg,/DESTROY

    state.w.addorder_bg = cw_bgroup(state.w.addorder_base,$
                                    val,$
                                    /row,$
                                    /NO_RELEASE,$
                                    /return_name,$
                                    /exclusive,$
                                    label_left='Add Order:',$
                                    UVALUE='Add Order',$
                                    font=state.w.buttonfont)

    widget_control, state.w.addorder_base,UPDATE=1
    message = '!5Please select an Order to Add.'

endif else message = 'Merging complete.'

wset, state.p.plotwin1_wid
erase
xyouts,0.5,0.5,message,/NORM,charsiz=3,ALIGN=0.5
wset, state.p.pixmap1_wid
erase
xyouts,0.5,0.5,message,/NORM,charsiz=3,ALIGN=0.5

wset, state.p.plotwin2_wid
erase
wset, state.p.pixmap2_wid
erase


end
;
;******************************************************************************
;
pro xmergeorders_changeanchor,state,index

*state.r.mergedorders = (*state.r.orders)[index]

nonan = where(finite((*state.d.origspec)[*,1,index]) eq 1)
*state.d.bluespec   = reform((*state.d.origspec)[nonan,*,index])
*state.d.mergedspec = reform((*state.d.origspec)[nonan,*,index])

;  Clear history later.

xmergeorders_changeaddorder,state

end
;
;******************************************************************************
;
pro xmergeorders_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin

    ptr_free, *state.r.mergeorders
    ptr_free, *state.r.orders

endif

state = 0B

end
;
;******************************************************************************
;
pro xmergeorders_combinespec,state

yspec_w = (*state.d.greenspec)[*,0]
yspec_f = (*state.d.greenspec)[*,1]
yspec_e = (*state.d.greenspec)[*,2]

bspec_w = (*state.d.bluespec)[*,0]
bspec_f = (*state.d.bluespec)[*,1]
bspec_e = (*state.d.bluespec)[*,2]

if state.p.greenpos eq 'Right' then begin

    nleft = n_elements(bspec_w)
    zbm   = where(bspec_w gt yspec_w[0],count)

    if count eq 0 then begin   

        tempmergeorder_w = [bspec_w,!values.f_nan,!values.f_nan,yspec_w]
        tempmergeorder_f = [bspec_f[0:nleft-2],!values.f_nan,!values.f_nan,$
                            !values.f_nan,!values.f_nan,yspec_f[1:*]]
        tempmergeorder_e = [bspec_e[0:nleft-2],!values.f_nan,!values.f_nan,$
                            !values.f_nan,!values.f_nan,yspec_e[1:*]]

    endif else begin

        mbspec_w = bspec_w[[zbm[0]-1,zbm]]
        mbspec_f = bspec_f[[zbm[0]-1,zbm]]
        mbspec_e = bspec_e[[zbm[0]-1,zbm]]

        interpspec,mbspec_w,mbspec_f,yspec_w,nmbspec_f,nmbspec_e,$
          YAERROR=mbspec_e

        meancomb,[[yspec_f],[nmbspec_f]],olap_flux,olap_var,$
          DATAVAR=[[yspec_e^2],[nmbspec_e^2]],CANCEL=cancel
        
        nonan = where(finite(olap_flux) eq 1)
        nan   = where(finite(olap_flux) eq 0)


        tempmergeorder_w = [bspec_w[0:zbm[0]-1],yspec_w]
        tempmergeorder_f = [bspec_f[0:zbm[0]-1],olap_flux[nonan],yspec_f[nan]]
        tempmergeorder_e = [bspec_e[0:zbm[0]-1],sqrt(olap_var[nonan]),$
                            yspec_e[nan]]

    endelse

endif
if state.p.greenpos eq 'Left' then begin

    nleft = n_elements(yspec_w)
    zym   = where(yspec_w gt bspec_w[0],count)

    if count eq 0 then begin

        tempmergeorder_w = [yspec_w,!values.f_nan,!values.f_nan,bspec_w]
        tempmergeorder_f = [yspec_f[0:nleft-2],!values.f_nan,!values.f_nan,$
                            !values.f_nan,!values.f_nan,bspec_f[1:*]]
        tempmergeorder_e = [yspec_e[0:nleft-2],!values.f_nan,!values.f_nan,$
                            !values.f_nan,!values.f_nan,bspec_e[1:*]]

    endif else begin

        myspec_w = yspec_w[[zym[0]-1,zym]]
        myspec_f = yspec_f[[zym[0]-1,zym]]
        myspec_e = yspec_e[[zym[0]-1,zym]]

        interpspec,myspec_w,myspec_f,bspec_w,nmyspec_f,nmyspec_e,$
          YAERROR=myspec_e

        meancomb,[[bspec_f],[nmyspec_f]],olap_flux,olap_var,$
          DATAVAR=[[bspec_e^2],[nmyspec_e^2]],CANCEL=cancel
        
        nonan = where(finite(olap_flux) eq 1)
        nan   = where(finite(olap_flux) eq 0)


        tempmergeorder_w = [yspec_w[0:zym[0]-1],bspec_w]
        tempmergeorder_f = [yspec_f[0:zym[0]-1],olap_flux[nonan],bspec_f[nan]]
        tempmergeorder_e = [yspec_e[0:zym[0]-1],sqrt(olap_var[nonan]),$
                            bspec_e[nan]]


    endelse

endif
if state.p.greenpos eq 'Inside' then begin

;  Since this is for Order 08 ShortXD, we don't have to worry about 
;  which spectrum is sampled at a higher frequency.  

    interpspec,yspec_w,yspec_f,bspec_w,nyspec_f,nyspec_e,$
      YAERROR=yspec_e
    
    meancomb,[[bspec_f],[nyspec_f]],olap_flux,olap_var,$
      DATAVAR=[[bspec_e^2],[nyspec_e^2]],CANCEL=cancel
    
    nonan = where(finite(olap_flux) eq 1)
    
    tempmergeorder_w = bspec_w
    tempmergeorder_f = bspec_f & tempmergeorder_f[nonan] = olap_flux[nonan]
    tempmergeorder_e = bspec_e & tempmergeorder_e[nonan] =sqrt(olap_var[nonan])

endif



*state.d.tempmergedspec = [[tempmergeorder_w],[tempmergeorder_f],$
                           [tempmergeorder_e]]

end
;
;******************************************************************************
;
pro xmergeorders_cutscalespec,state

if finite(state.r.trimblue) eq 1 then begin

    if state.p.greenpos eq 'Right' then begin

        z = where((*state.d.mergedspec)[*,0] lt state.r.trimblue or $
                  finite((*state.d.mergedspec)[*,0]) eq 0)
        *state.d.bluespec = (*state.d.mergedspec)[z,*]

    endif else begin

        z = where((*state.d.mergedspec)[*,0] gt state.r.trimblue or $
                  finite((*state.d.mergedspec)[*,0]) eq 0)
        *state.d.bluespec = (*state.d.mergedspec)[z,*]

    endelse
    
endif else (*state.d.bluespec) = (*state.d.mergedspec)

order = where(*state.r.orders eq state.r.addorder)
if finite(state.r.trimgreen) eq 1 then begin

    if state.p.greenpos eq 'Right' then begin

        z = where((*state.d.origspec)[*,0,order] gt state.r.trimgreen)
        *state.d.greenspec = (*state.d.origspec)[z,*,order]
        (*state.d.greenspec)[*,1] = (*state.d.greenspec)[*,1]*state.r.scale
        (*state.d.greenspec)[*,2] = (*state.d.greenspec)[*,2]*$
          sqrt(abs(state.r.scale))

    endif else begin

        z = where((*state.d.origspec)[*,0,order] lt state.r.trimgreen)
        *state.d.greenspec = (*state.d.origspec)[z,*,order]
        (*state.d.greenspec)[*,1] = (*state.d.greenspec)[*,1]*state.r.scale
        (*state.d.greenspec)[*,2] = (*state.d.greenspec)[*,2]*$
          sqrt(abs(state.r.scale))

    endelse

endif else begin

    *state.d.greenspec = reform((*state.d.origspec)[*,*,order])   
    (*state.d.greenspec)[*,1] = (*state.d.greenspec)[*,1]*state.r.scale
    (*state.d.greenspec)[*,2] = (*state.d.greenspec)[*,2]*$
      sqrt(abs(state.r.scale)) 

endelse

z = where(finite((*state.d.greenspec)[*,1]) eq 1)
*state.d.greenspec = (*state.d.greenspec)[z,*]

end
;
;******************************************************************************
;
pro xmergeorders_getminmax1,state

 ;  Get X range

if state.p.greenpos eq 'Right' then begin

    yidx = where(*state.r.orders eq state.r.addorder)
    state.p.plot1xrange[1] = max((*state.d.origspec)[*,0,yidx])
    
    bidx = where(*state.r.orders eq state.r.addorder+1)
    state.p.plot1xrange[0] = min((*state.d.origspec)[*,0,bidx])
    
endif else begin
    
    yidx = where(*state.r.orders eq state.r.addorder)
    state.p.plot1xrange[0] = min((*state.d.origspec)[*,0,yidx])
    
    bidx = where(*state.r.orders eq state.r.addorder-1)
    state.p.plot1xrange[1] = max((*state.d.origspec)[*,0,bidx])
    
endelse

;  Get Y range

z     = where(finite((*state.d.mergedspec)[*,0]) eq 1)
mspec = (*state.d.mergedspec)[z,*]
zb    = where(mspec[*,0] gt state.p.plot1xrange[0] and $
              mspec[*,0] lt state.p.plot1xrange[1])
specb = mspec[zb,1]

zy    = where((*state.d.greenspec)[*,0] gt state.p.plot1xrange[0] and $
              (*state.d.greenspec)[*,0] lt state.p.plot1xrange[1])
specy = (*state.d.greenspec)[zy,1]

fbmax = max(specb,/NAN,min=fbmin)
fymax = max(specy,/NAN,min=fymin)



state.p.plot1yrange = [(fbmin < fymin),(fbmax > fymax)]

end
;
;******************************************************************************
;
pro xmergeorders_getminmax2,state

;  2nd plot window.

z     = where(finite((*state.d.mergedspec)[*,0]) eq 1)
mspec = (*state.d.mergedspec)[z,*]
zb    = where(mspec[*,0] gt state.p.plot1xrange[0] and $
              mspec[*,0] lt state.p.plot1xrange[1],countb)
specb = (countb ne 0) ? (*state.d.mergedspec)[zb,2]:!values.f_nan


zy    = where((*state.d.greenspec)[*,0] gt state.p.plot1xrange[0] and $
              (*state.d.greenspec)[*,0] lt state.p.plot1xrange[1],county)
specy = (county ne 0) ? (*state.d.greenspec)[zy,2]:!values.f_nan


fbmax = max(specb,/NAN,min=fbmin)
fymax = max(specy,/NAN,min=fymin)

state.p.plot2yrange = [(fbmin < fymin),(fbmax > fymax)]

end
;
;******************************************************************************
;
pro xmergeorders_loadspec,state

;  Get files.

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

specfile = cfld(state.w.ispectrum_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
specfile = cfile(path+specfile,WIDGET_ID=state.w.xmergeorders_base,$
                 CANCEL=cancel)
if cancel then return

;  Read standard spectra.

readspec,specfile,spc,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,airmass

if state.r.errors eq 'No' then spc[*,2,*] = 1.

;  Save info

*state.d.origspec  = spc
*state.d.hdr    = hdr          
*state.r.orders = orders
state.r.norders = norders
*state.r.specscales = fltarr(norders)+1

state.p.xunits = xunits
state.p.yunits = yunits

;  Update droplists.

widget_control,state.w.anchororder_dl,$
  SET_VALUE=string((*state.r.orders),format='(i2.2)')

wset, state.p.plotwin1_wid
erase
xyouts,0.5,0.5,'!5Please Select an Anchor Order',/NORM,charsize=3,ALIGN=0.5
wset, state.p.pixmap1_wid
erase
xyouts,0.5,0.5,'!5Please Select an Anchor Order',/NORM,charsize=3,ALIGN=0.5

wset, state.p.plotwin2_wid
erase
wset, state.p.pixmap2_wid
erase

end
;
;******************************************************************************
;
pro xmergeorders_mergespec,state

xmergeorders_combinespec,state

*state.r.mergedorders = [*state.r.mergedorders,state.r.addorder]
xmergeorders_changeaddorder,state

*state.d.bluespec   = reform(*state.d.tempmergedspec)
*state.d.mergedspec = reform(*state.d.tempmergedspec)

;  Store the scale value and reset to 1.

z = where(*state.r.orders eq state.r.addorder)
(*state.r.specscales)[z] = state.r.scale

state.r.scale = 1.0
widget_control, state.w.scale_fld[1],SET_VALUE=strtrim(state.r.scale,2)

;  Default plot settings

state.p.plottype = 'Overlap'
widget_control, state.w.plottype_bg, SET_VALUE=0

state.r.trimspec = 'None'

state.p.reg[*] = !values.f_nan
state.r.trimblue = !values.f_nan
state.r.trimgreen = !values.f_nan

end
;
;******************************************************************************
;
pro xmergeorders_ploterror,state

xtitle ='!5Wavelength ('+strtrim(state.p.xunits,2)+')'
ytitle ='!5Error ('+strtrim(state.p.yunits,2)+')'

if state.p.plottype eq 'Overlap' then begin
    
    plot, findgen(10),/xsty,/ysty,/nodata,$
      xrange=state.p.plot1xrange,yrange=state.p.plot2yrange,$
      xtitle=xtitle,ytitle=ytitle,title='!5Error Spectra'
    
    if state.p.plotbluespec then oplot, (*state.d.bluespec)[*,0],$
      (*state.d.bluespec)[*,2],color=4,psym=10
    
    if state.p.plotgreenspec then oplot, (*state.d.greenspec)[*,0],$
      (*state.d.greenspec)[*,2],color=3,psym=10
    
endif else begin
    
    plot, (*state.d.tempmergedspec)[*,0],(*state.d.tempmergedspec)[*,2],$
      /xsty,/ysty,/nodata,xrange=state.p.plot1xrange,$
      yrange=state.p.plot2yrange,title='!5Error Spectrum'
    oplot,(*state.d.tempmergedspec)[*,0],(*state.d.tempmergedspec)[*,2],$
      color=7,psym=10
    
endelse

if state.r.cursormode eq 'Select' then begin

    if finite(state.p.reg[0,0]) eq 1 then $
      plots,[state.p.reg[0,0],state.p.reg[0,0]],!y.crange,color=2,$
      linestyle=1,thick=2
    if finite(state.p.reg[0,1]) eq 1 then $
      plots,[state.p.reg[0,1],state.p.reg[0,1]],!y.crange,color=2,$
      linestyle=1,thick=2
    
endif

state.p.pscale2 = !p
state.p.xscale2 = !x
state.p.yscale2 = !y

end
;
;******************************************************************************
;
pro xmergeorders_plotflux,state

xtitle ='!5Wavelength ('+strtrim(state.p.xunits,2)+')'
ytitle ='!5Flux ('+strtrim(state.p.yunits,2)+')'

if state.p.plottype eq 'Overlap' then begin
    
    plot, findgen(10),/xsty,/ysty,/nodata,$
      xrange=state.p.plot1xrange,yrange=state.p.plot1yrange,$
      xtitle=xtitle,ytitle=ytitle,title=state.p.title
    
    if state.p.plotbluespec then oplot, (*state.d.bluespec)[*,0],$
      (*state.d.bluespec)[*,1],color=4,psym=10
    
    if state.p.plotgreenspec then oplot, (*state.d.greenspec)[*,0],$
      (*state.d.greenspec)[*,1],color=3,psym=10
    
endif else begin
    
    plot, (*state.d.tempmergedspec)[*,0],(*state.d.tempmergedspec)[*,1],$
      /xsty,/ysty,/nodata,xrange=state.p.plot1xrange,$
      yrange=state.p.plot1yrange,xtitle=xtitle,ytitle=ytitle,$
      title='!5Merged Spectrum'
    
    oplot,(*state.d.tempmergedspec)[*,0],(*state.d.tempmergedspec)[*,1],$
      color=7,psym=10
    
endelse

if state.r.cursormode eq 'Select' then begin
    
    if finite(state.p.reg[0,0]) eq 1 then $
      plots,[state.p.reg[0,0],state.p.reg[0,0]],!y.crange,color=2,$
      linestyle=1,thick=2
    if finite(state.p.reg[0,1]) eq 1 then $
      plots,[state.p.reg[0,1],state.p.reg[0,1]],!y.crange,color=2,$
      linestyle=1,thick=2

endif

state.p.pscale1 = !p
state.p.xscale1 = !x
state.p.yscale1 = !y

end
;
;******************************************************************************
;
pro xmergeorders_plotupdate,state

wset, state.p.pixmap1_wid
erase
xmergeorders_plotflux,state

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]


wset, state.p.pixmap2_wid
erase 
xmergeorders_ploterror,state

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]


end
;
;******************************************************************************
;
pro xmergeorders_setminmax,state

widget_control, state.w.xmin_fld[1],SET_VAL=strtrim(state.p.plot1xrange[0],2)
widget_control, state.w.xmax_fld[1],SET_VAL=strtrim(state.p.plot1xrange[1],2)
widget_control, state.w.ymin1_fld[1],SET_VAL=strtrim(state.p.plot1yrange[0],2)
widget_control, state.w.ymax1_fld[1],SET_VAL=strtrim(state.p.plot1yrange[1],2)

widget_control, state.w.ymin2_fld[1],SET_VAL=strtrim(state.p.plot2yrange[0],2)
widget_control, state.w.ymax2_fld[1],SET_VAL=strtrim(state.p.plot2yrange[1],2)

end
;
;******************************************************************************
;
pro xmergeorders_writefile,state


path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

ifile = cfld(state.w.ispectrum_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

file = cfld(state.w.oname_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

if state.r.errors eq 'No' then (*state.d.mergedspec)[*,2] = 1.
start = 0
stop  = n_elements((*state.d.mergedspec)[*,1])-1

hdr = *state.d.hdr

fxaddpar,hdr,'IRAFNAME',file
fxaddpar,hdr,'ORDERS','0'
fxaddpar,hdr,'NORDERS',1
fxaddpar,hdr,'NAPS',1
fxaddpar,hdr,'START',start
fxaddpar,hdr,'STOP',stop

history = 'This merged spectrum was derived from '+strtrim(ifile,2)+'.'

morders = (*state.r.mergedorders)[sort(*state.r.mergedorders)]
match,morders,*state.r.orders,junk,idx

history = history+'  The scaled factors for orders '+$
  strjoin(strcompress(morders,/RE),', ')+' are '+$
  strjoin(strcompress((*state.r.specscales)[idx],/RE),', ')+' respectively.'

length = strlen(history)
loop = ceil(float(length)/65.)
for i = 0, loop-1 do begin
    
    hist = strmid(history,65*i,65)
    fxaddpar,hdr,'HISTORY',hist
    
endfor

if state.r.fitsoutput then writefits,path+file+'.fits',*state.d.mergedspec,hdr

xvspec,*state.d.mergedspec,1,1,ORDERS=1,START=start,STOP=stop,$
  XUNITS=xunits,YUNITS=yunits,HDR=hdr

if state.r.textoutput then begin

    npix = fxpar(hdr,'NAXIS1')
    openw,lun,path+file+'.txt', /get_lun
    
    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]
    
    for i = 0, npix-1 do begin
        
        printf, lun,  strjoin( reform((*state.d.mergedspec)[i,*],3),'  ' )
        
    endfor
    close, lun
    free_lun, lun
    
endif

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xmergeorders_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of
    
    'Add Order': begin
        
        state.r.addorder = fix(event.value)
        xmergeorders_addorder,state
        
    end
    'Anchor Order': xmergeorders_changeanchor,state,event.index
    
    'Auto Scale': xmergeorders_autoscale,state

    'Error Propagation': state.r.errors = strtrim(event.value,2)
    
    'Input Spectrum': begin
        
        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        if path ne '' then path = cpath(path,$
                                        WIDGET_ID=state.w.xmergeorders_base,$
                                        CANCEL=cancel)
        if cancel then return
        obj = dialog_pickfile(DIALOG_PARENT=state.w.xmergeorders_base,$
                              PATH=path,/MUST_EXIST,FILTER='*.fits')
        if obj eq '' then goto, cont
        widget_control,state.w.ispectrum_fld[1], $
          SET_VALUE = strmid(obj[0],strpos(obj,'/',/REVERSE_S)+1)
        setfocus,state.w.ispectrum_fld
        
    end
    
    'Keyboard': begin
        
        case strtrim(event.ch,2) of 
            
            'c': begin
                
                state.r.cursormode = 'None'
                state.p.reg[*] = !values.f_nan
                xmergeorders_plotupdate,state
                
            end
            
            'b': begin

                state.r.cursormode = 'None'
                state.r.trimspec = 'Blue'
            
            end
            'm': xmergeorders_mergeorder,state
            
            's': begin
                
                state.r.cursormode = 'Select'
                state.p.reg[*] = !values.f_nan
                
            end
            
            'g': begin

                state.r.cursormode = 'None'
                state.r.trimspec = 'Green'

            end
            
            'w': begin
                
                xmergeorders_getminmax1,state
                xmergeorders_getminmax2,state
                xmergeorders_plotupdate,state
                xmergeorders_setminmax,state
                
            end
            
            'x': begin 

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'XZoom'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'y': begin 

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'YZoom'
                    state.p.reg[*] = !values.f_nan
                    
                endif

            end

            'z': begin
                
                state.r.cursormode = 'Zoom'
                state.p.reg[*] = !values.f_nan
                
            end
            
            else:
            
        endcase
        
    end

    'Load Spectrum': xmergeorders_loadspec,state

    'Merge Orders': xmergeorders_mergespec,state

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select

    end

    'Path Button': begin
        
        path= dialog_pickfile(/DIRECTOR,$
                              DIALOG_PARENT=state.w.xmergeorders_base,$
                              TITLE='Select Path',/MUST_EXIST)

        if path ne '' then begin
            
            path = cpath(path,WIDGET_ID=state.w.xmergeorders_base,$
                         CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld
            
        endif
    end
    
    'Plot Spectrum': begin

        if event.value eq 'Blue' then state.p.plotbluespec = event.select
        if event.value eq 'Green' then state.p.plotgreenspec = event.select
        xmergeorders_plotupdate,state

    end

    'Plot Type': begin

        state.p.plottype = event.value
        if state.p.plottype eq 'Combine' then xmergeorders_combinespec,state
        xmergeorders_plotupdate,state

    end

    'Scale Spectrum': begin

        val = cfld(state.w.scale_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        state.r.scale = val
        xmergeorders_cutscalespec,state
        xmergeorders_plotupdate,state

    end

    'Quit': begin
        
        widget_control, event.top, /DESTROY
        goto, getout
        
    end
    
    'Write File': xmergeorders_writefile,state

    else:
    
endcase

cont: 

widget_control, state.w.xmergeorders_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
pro xmergeorders_plotwinevent1, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    wset, state.p.plotwin1_wid
    device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
                  state.p.pixmap1_wid]

    wset, state.p.plotwin2_wid
    device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                  state.p.pixmap2_wid]

    goto, cont

endif

 ;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1
x  = event.x/float(state.p.plot1size[0])
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin
    
    case state.r.cursormode of 

        'Select': begin
            
            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
                
                state.p.reg[*,1] = xy[0:1]
                
            endelse
            xmergeorders_plotupdate,state
            
        end

        'XZoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1]
                wset, state.p.pixmap1_wid
                plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,$
                  /DEVICE,linestyle=1,thick=2
                wset, state.p.plotwin1_wid
                device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,$
                              0,state.p.pixmap1_wid]
                wset, state.p.pixmap2_wid
                plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,$
                  /DEVICE,linestyle=1,thick=2
                wset, state.p.plotwin2_wid
                device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,$
                              0,state.p.pixmap2_wid]
               
            endif else begin

                state.p.reg[*,1] = xy[0:1]
                state.p.plot1xrange = [min(state.p.reg[0,*],max=m),m]
                state.r.cursormode = 'None'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xmergeorders_plotupdate,state
                xmergeorders_setminmax,state
                
            endelse


        end

        'YZoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then begin
                
                state.p.reg[*,0] = xy[0:1]
                wset, state.p.pixmap1_wid
                plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,$
                  /DEVICE,linestyle=1,thick=2

                wset, state.p.plotwin1_wid
                device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,$
                              0,state.p.pixmap1_wid]
               
            endif else begin

                state.p.reg[*,1] = xy[0:1]
                state.p.plot1yrange = [min(state.p.reg[1,*],max=m),m]
                state.r.cursormode = 'None'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xmergeorders_plotupdate,state
                xmergeorders_setminmax,state
                
            endelse

        end
        
        'Zoom': begin
            
            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
                
                state.p.reg[*,1] = xy[0:1]
                state.p.plot1xrange   = [min(state.p.reg[0,*],max=max),max]
                state.p.plot1yrange   = [min(state.p.reg[1,*],max=max),max]
                xmergeorders_getminmax2,state
                xmergeorders_plotupdate,state
                xmergeorders_setminmax,state
                state.r.cursormode   = 'None'
                state.p.reg      = [[!values.f_nan,!values.f_nan],$
                                    [!values.f_nan,!values.f_nan]]
                
            endelse
            
        end
        
        else: begin
            
            if state.r.trimspec ne 'None' then begin

                if event.release le 1 then begin
                    
                    if state.r.trimspec eq 'Blue' then state.r.trimblue = xy[0]
                    if state.r.trimspec eq 'Green' then $
                      state.r.trimgreen = xy[0]
                
                endif 
                if event.release gt 1 then begin
                    
                    if state.r.trimspec eq 'Blue' then $
                      state.r.trimblue = !values.f_nan
                    if state.r.trimspec eq 'Green' then $
                      state.r.trimgreen = !values.f_nan
                    
                endif
                xmergeorders_cutscalespec,state
                xmergeorders_plotupdate,state
            
            endif
        end
        
    endcase
    
endif

;  Copy the pixmaps and draw the lines.


wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

wset, state.p.plotwin1_wid

case state.r.cursormode of 

    'XZoom': begin

        wset, state.p.plotwin1_wid
        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        wset, state.p.plotwin2_wid
        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

    end

    'YZoom': plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE

    'Zoom': begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
        xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
        plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
          linestyle=2,color=2
        plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
          linestyle=2,color=2
        
    end

    else: begin

        plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
        plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE
        wset, state.p.plotwin2_wid
        plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

    end

endcase







;wset, state.p.plotwin1_wid
;device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
;              state.p.pixmap1_wid]
;plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
;if state.r.cursormode ne 'Select' then plots, [0,state.p.plot1size[0]],$
;  [event.y,event.y],color=2,/DEVICE

;wset, state.p.plotwin2_wid
;device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
;              state.p.pixmap2_wid]
;plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

;  Update the cursor tracking

label = 'Cursor X: '+strtrim(xy[0],2)
widget_control,state.w.message,set_value=label

if state.r.cursormode eq 'Zoom' then begin

    wset, state.p.plotwin1_wid
    xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
    plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
      linestyle=2,color=2
    plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
      linestyle=2,color=2

endif


cont:

widget_control, state.w.xmergeorders_base, SET_UVALUE=state, /NO_COPY
    
end
;
;******************************************************************************
;
pro xmergeorders_plotwinevent2, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale2
!x = state.p.xscale2
!y = state.p.yscale2
x  = event.x/float(state.p.plot2size[0])
y  = event.y/float(state.p.plot2size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 and state.r.cursormode eq 'YZoom' then begin

    z = where(finite(state.p.reg) eq 1,count)
    if count eq 0 then begin
        
        state.p.reg[*,0] = xy[0:1]
        wset, state.p.pixmap2_wid
        plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,$
          /DEVICE,linestyle=1,thick=2
        
        wset, state.p.plotwin2_wid
        device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
                      state.p.pixmap2_wid]
        
    endif else begin
        
        state.p.reg[*,1] = xy[0:1]
        state.p.plot2yrange = [min(state.p.reg[1,*],max=m),m]
        state.r.cursormode = 'None'
        state.p.reg = [[!values.f_nan,!values.f_nan],$
                       [!values.f_nan,!values.f_nan]]
        xmergeorders_plotupdate,state
        xmergeorders_setminmax,state
        
    endelse


endif

;  Copy the pixmaps and draw the lines.


wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]

if state.r.cursormode eq 'YZoom' then begin

    plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE

endif else begin

    plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE
    plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

    wset, state.p.plotwin1_wid
    plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

endelse


;  Update the cursor tracking

label = 'Cursor X: '+strtrim(xy[0],2)
widget_control,state.w.message,set_value=label

;  Plot zoom line

if state.r.cursormode eq 'Zoom' then begin

    wset, state.p.plotwin2_wid
    plots,!x.crange,[state.p.reg[1,0],state.p.reg[1,0]],linestyle=2,color=2

endif
    
cont:

widget_control, state.w.xmergeorders_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xmergeorders_minmax,event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of 

    'X Min': begin

        xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmin2 = crange(xmin,state.p.plot1xrange[1],'X Min',/KLT,$
                       WIDGET_ID=state.w.xmergeorders_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmin_fld[0],set_value=state.p.plotxrange[0]
            return
            
        endif else state.p.plot1xrange[0] = xmin2

    end
    'X Max': begin

        xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmax2 = crange(xmax,state.p.plot1xrange[0],'X Max',/KGT,$
                       WIDGET_ID=state.w.xmergeorders_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmax_fld[0],set_value=state.p.plotxrange[1]
            return
            
        endif else state.p.plot1xrange[1] = xmax2

    end
    'Y1 Min': begin

        ymin = cfld(state.w.ymin1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot1yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xmergeorders_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin1_fld[0],set_val=state.p.plot1yrange[0]
            return
            
        endif else state.p.plot1yrange[0] = ymin2
        
    end
    'Y1 Max': begin

        ymax = cfld(state.w.ymax1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot1yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xmergeorders_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax1_fld[0],set_val=state.p.plot1yrange[1]
            return
            
        endif else state.p.plot1yrange[1] = ymax2
        
    end
    'Y2 Min': begin

        ymin = cfld(state.w.ymin2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot2yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xmergeorders_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin2_fld[0],set_val=state.p.plot2yrange[0]
            return
            
        endif else state.p.plot2yrange[0] = ymin2
        
    end
    'Y2 Max': begin

        ymax = cfld(state.w.ymax2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot2yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xmergeorders_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax2_fld[0],set_val=state.p.plot2yrange[1]
            return
            
        endif else state.p.plot2yrange[1] = ymax2
        
    end
    
endcase

xmergeorders_plotupdate,state

widget_control, state.w.xmergeorders_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xmergeorders_resize, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

widget_control, state.w.xmergeorders_base, tlb_get_size = size

;  Window 1

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=(size[1]-state.p.buffer[1])*1.4/2.4

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

;  Window 2

state.p.plot2size[0]=size[0]-state.p.buffer[0]
state.p.plot2size[1]=(size[1]-state.p.buffer[1])/2.4

widget_control, state.w.plotwin2, draw_xsize=state.p.plot2size[0]
widget_control, state.w.plotwin2, draw_ysize=state.p.plot2size[1]

wdelete,state.p.pixmap2_wid
window, /free, /pixmap,xsize=state.p.plot2size[0],ysize=state.p.plot2size[1]
state.p.pixmap2_wid = !d.window

xmergeorders_plotupdate,state

widget_control, state.w.xmergeorders_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xmergeorders

if xregistered('xmergeorders') then goto, cont

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {addorder_base:0L,$
     addorder_bg:0L,$
     anchororder_dl:0L,$
     buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     keyboard:0L,$
     ispectrum_fld:[0L,0L],$
     message:0L,$
     oname_fld:[0L,0L],$
     path_fld:[0L,0L],$
     plotspec_bg:0L,$
     plottype_bg:0L,$
     plotwin1:0,$
     plotwin2:0,$
     scale_fld:[0L,0L],$
     xmergeorders_base:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin1_fld:[0L,0L],$
     ymax1_fld:[0L,0L],$
     ymin2_fld:[0L,0L],$
     ymax2_fld:[0L,0L]}

r = {addorder:0,$
     trimblue:!values.f_nan,$
     trimgreen:!values.f_nan,$
     cursormode:'None',$
     errors:'Yes',$
     fitsoutput:1,$
     mergedorders:ptr_new(fltarr(2)),$
     addorders:ptr_new(fltarr(2)),$
     norders:0,$
     orders:ptr_new(fltarr(2)),$
     scale:1.,$
     specscales:ptr_new(fltarr(2)),$
     textoutput:0,$
     trimspec:'Green'}

d = {hdr:ptr_new(strarr(2)),$
     bluespec:ptr_new(fltarr(2)),$
     tempmergedspec:ptr_new(fltarr(2)),$
     greenspec:ptr_new(fltarr(2)),$
     mergedspec:ptr_new(fltarr(2)),$
     origspec:ptr_new(fltarr(2))}

p = {activespec:1,$
     buffer:[0.,0.],$
     pixmap1_wid:0L,$
     pixmap2_wid:0L,$
     plotwin1_wid:0L,$
     plotwin2_wid:0L,$
     plot1xrange:[0.,0.],$
     plot1yrange:[0.,0.],$
     plot1size:[670,320],$
     plot2yrange:[0.,0.],$
     plot2size:[670,320/1.4],$
     plotbluespec:1,$
     plottype:'Overlap',$
     plotgreenspec:1,$
     title:'',$
     pscale1:!p,$
     xscale1:!x,$
     pscale2:!p,$
     xscale2:!x,$
     xunits:'',$
     greenpos:'',$
     yscale1:!y,$
     yscale2:!y,$
     yunits:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}


;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

;  Build the widget.

buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
    
state.w.xmergeorders_base = widget_base(title='Xmergeorders', $
                                      /COLUMN,$
                                      /TLB_SIZE_EVENTS)

   quit_button = widget_button(state.w.xmergeorders_base,$
                               FONT=buttonfont,$
                               EVENT_PRO='xmergeorders_event',$
                               VALUE='Quit',$
                               UVALUE='Quit')
   
   state.w.keyboard = widget_text(state.w.xmergeorders_base, $
                                  /ALL_EVENTS, $
                                  SCR_XSIZE=1, $
                                  SCR_YSIZE=1, $
                                  UVALUE='Keyboard', $
                                  EVENT_PRO='xmergeorders_event',$
                                  VALUE= '')
   
   row_base = widget_base(state.w.xmergeorders_base,$
                          /ROW)

      col1_base = widget_base(row_base,$
                              EVENT_PRO='xmergeorders_event',$
                              /COLUMN)
      
         box1_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
         
            label = widget_label(box1_base,$
                                 VALUE='1.  Load Spectra',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)
            
            
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               button = widget_button(row,$
                                      FONT=buttonfont,$
                                      VALUE='Path',$
                                      UVALUE='Path Button',$
                                      EVENT_PRO='xmergeorders_event')
               
               field = coyote_field2(row,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE=':',$
                                     UVALUE='Path Field',$
                                     XSIZE=25,$
                                     EVENT_PRO='xmergeorders_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
               state.w.path_fld = [field,textid]    
               
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               input = widget_button(row,$
                                     FONT=buttonfont,$
                                     VALUE='Input Spectrum',$
                                     UVALUE='Input Spectrum',$
                                     EVENT_PRO='xmergeorders_event')
               
               
               input_fld = coyote_field2(row,$
                                         LABELFONT=buttonfont,$
                                         FIELDFONT=textfont,$
                                         TITLE=':',$
                                         UVALUE='Input Spectrum Field',$
                                         XSIZE=15,$
                                         EVENT_PRO='xmergeorders_event',$
                                         /CR_ONLY,$
                                         TEXTID=textid)
               state.w.ispectrum_fld = [input_fld,textid]

               errors_bg = cw_bgroup(box1_base,$
                                     font=buttonfont,$
                                     ['Yes','No'],$
                                     /ROW,$
                                     /RETURN_NAME,$
                                     /EXCLUSIVE,$
                                     /NO_RELEASE,$
                                     LABEL_LEFT='Error Propagation:',$
                                     UVALUE='Error Propagation',$
                                     SET_VALUE=0)
               
            load = widget_button(box1_base,$
                                 VALUE='Load Spectrum',$
                                 UVALUE='Load Spectrum',$
                                 FONT=buttonfont)

         box2_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
         
            state.w.anchororder_dl = widget_droplist(box2_base,$
                                                     font=buttonfont,$
                                            title='2.  Choose Anchor Order: ',$
                                                     value='00',$
                                                     uvalue='Anchor Order')

         box3_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 FRAME=1)

            label = widget_label(box3_base,$
                                 value='3.  Cut/Scale/Merge Spectra',$
                                 font=buttonfont,$
                                 /align_left)

            state.w.addorder_base = widget_base(box3_base,$
                                                /COLUMN)

               state.w.addorder_bg = cw_bgroup(state.w.addorder_base,$
                                               ['00','00'],$
                                               /row,$
                                               /NO_RELEASE,$
                                               /return_name,$
                                               /exclusive,$
                                               label_left='Add Order:',$
                                               UVALUE='Add Order',$
                                               font=buttonfont)
            
            row = widget_base(box3_base,$
                              /BASE_ALIGN_CENTER,$
                              /ROW)
               
               scale = coyote_field2(row,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Scale:',$
                                     UVALUE='Scale Spectrum',$
                                     XSIZE=12,$
                                     VALUE=state.r.scale,$
                                     EVENT_PRO='xmergeorders_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
               state.w.scale_fld = [scale,textid]
               
               widget_button = widget_button(row,$
                                             value='Auto Scale',$
                                             uvalue='Auto Scale',$
                                             font=buttonfont)
            
            state.w.plotspec_bg = cw_bgroup(box3_base,$
                                            font=buttonfont,$
                                            ['Blue','Green'],$
                                            /row,$
                                            /return_name,$
                                            /nonexclusive,$
                                            label_left='Plot Spectrum:',$
                                            UVALUE='Plot Spectrum',$
                                            set_VALUE=[1,1])
            
            state.w.plottype_bg = cw_bgroup(box3_base,$
                                            font=buttonfont,$
                                            ['Overlap','Combine'],$
                                            /row,$
                                            /return_name,$
                                            /no_release,$
                                            /exclusive,$
                                            label_left='Spec Type:',$
                                            UVALUE='Plot Type',$
                                            set_VALUE=0)
               
            widget_button = widget_button(box3_base,$
                                          EVENT_PRO='xmergeorders_event',$
                                          value='Merge Orders',$
                                          uvalue='Merge Orders',$
                                          font=buttonfont)

         box4_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
         
            label = widget_label(box4_base,$
                                 VALUE='4.  Write Spectra to File',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)
            
            outformat_bg = cw_bgroup(box4_base,$
                                     font=buttonfont,$
                                     ['FITS','Text'],$
                                     /row,$
                                     /return_name,$
                                     /no_release,$
                                     /nonexclusive,$
                                     label_left='Output Format:',$
                                     UVALUE='Output Format',$
                                     set_VALUE=[1,0])
            
            oname = coyote_field2(box4_base,$
                                  labelfont=buttonfont,$
                                  fieldfont=textfont,$
                                  title='File Name:',$
                                  event_pro = 'xmergeorders_event',$
                                  uvalue='Object File Oname',$
                                  /cr_only,$
                                  xsize=18,$
                                  textID=textid)
            state.w.oname_fld = [oname,textid]
            
            write = widget_button(box4_base,$
                                  value='Write File',$
                                  uvalue='Write File',$
                                  font=buttonfont)

          col2_base = widget_base(row_base,$
                                  /COLUMN)

             label = 'X: 0.000000'
             state.w.message = widget_text(col2_base, $
                                           VALUE=label,$
                                           YSIZE=1)

             state.w.plotwin1 = widget_draw(col2_base,$
                                            XSIZE=state.p.plot1size[0],$
                                            YSIZE=state.p.plot1size[1],$
                                            /TRACKING_EVENTS,$
                                            /BUTTON_EVENTS,$
                                            /MOTION_EVENTS,$
                                         EVENT_PRO='xmergeorders_plotwinevent1',$
                                            UVALUE='Plot Window 1')
       
             row_base = widget_base(col2_base,$
                                    /ROW)
   
                xmin = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='X Min:',$
                                     UVALUE='X Min',$
                                     XSIZE=12,$
                                     EVENT_PRO='xmergeorders_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.xmin_fld = [xmin,textid]
                
                xmax = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='X Max:',$
                                     UVALUE='X Max',$
                                     XSIZE=12,$
                                     EVENT_PRO='xmergeorders_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.xmax_fld = [xmax,textid]
                
                ymin = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Min:',$
                                     UVALUE='Y1 Min',$
                                     XSIZE=12,$
                                     EVENT_PRO='xmergeorders_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymin1_fld = [ymin,textid]
                
                ymax = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Max:',$
                                     UVALUE='Y1 Max',$
                                     XSIZE=12,$
                                     EVENT_PRO='xmergeorders_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymax1_fld = [ymax,textid]
                
             state.w.plotwin2 = widget_draw(col2_base,$
                                            XSIZE=state.p.plot2size[0],$
                                            YSIZE=state.p.plot2size[1],$
                                            /TRACKING_EVENTS,$
                                            /MOTION_EVENTS,$
                                            /BUTTON_EVENTS,$
                                         EVENT_PRO='xmergeorders_plotwinevent2',$
                                            UVALUE='Plot Window 2')
          
             row_base = widget_base(col2_base,$
                                    /ROW,$
                                    EVENT_PRO='xmergeorders_event')
          
                ymin = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Min:',$
                                     UVALUE='Y2 Min',$
                                     XSIZE=12,$
                                     EVENT_PRO='xmergeorders_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymin2_fld = [ymin,textid]
                
                ymax = coyote_field2(row_base,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE='Y Max:',$
                                     UVALUE='Y2 Max',$
                                     XSIZE=12,$
                                     EVENT_PRO='xmergeorders_minmax',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
                state.w.ymax2_fld = [ymax,textid]

; Get things running.  Center the widget using the Fanning routine.
      
    centertlb,state.w.xmergeorders_base
      
    widget_control, state.w.xmergeorders_base, /REALIZE
    
    mkct
    
;  Get plotwin ids
    
    widget_control, state.w.plotwin1, GET_VALUE=x
    state.p.plotwin1_wid = x
    widget_control, state.w.plotwin2, GET_VALUE=x
    state.p.plotwin2_wid = x
    
    window, /free, /pixmap,xsize=state.p.plot1size[0],$
      ysize=state.p.plot1size[1]
    state.p.pixmap1_wid = !d.window

    window, /free, /pixmap,xsize=state.p.plot2size[0],$
      ysize=state.p.plot2size[1]
    state.p.pixmap2_wid = !d.window
    
;  Get sizes for things.
    
    widget_geom = widget_info(state.w.xmergeorders_base, /geometry)
    state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
    state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]-$
      state.p.plot2size[1]

; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xmergeorders', $
      state.w.xmergeorders_base, $
      cleanup = 'xmergeorders_cleanup',$
      event_handler='xmergeorders_resize',$
      /NO_BLOCK
    
; Put state variable into the user value of the top level base.

widget_control, state.w.xmergeorders_base, set_uvalue=state, /no_copy

cont:

end
