;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xnormspec_initcommon

common xnormspec_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {keyboard:0L,$
     message:0L,$
     plotwin1:0,$
     plotwin2:0,$
     shift_fld:[0L,0L],$
     xnormspec_base:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin1_fld:[0L,0L],$
     ymax1_fld:[0L,0L],$
     ymin2_fld:[0L,0L],$
     ymax2_fld:[0L,0L]}

r = {cancel:0,$
     cursormode:'None',$
     norm:0,$
     normreg:1,$
     residplot:0L,$
     normdegree:1}

d = {coeffs:ptr_new(fltarr(2)),$
     flux:ptr_new(fltarr(2)),$
     nflux:ptr_new(fltarr(2)),$
     normgoodbad:ptr_new(2),$
     wave:ptr_new(fltarr(2))}


p = {activespec:1,$
     buffer:[0.,0.],$
     pixmap1_wid:0L,$
     pixmap2_wid:0L,$
     plotwin1_wid:0L,$
     plotwin2_wid:0L,$
     plot1xrange:[0.,0.],$
     plot1yrange:[0.,0.],$
     plot1size:[700,300],$
     plot2yrange:[0.,0.],$
     plot2size:[700,214],$
     pscale1:!p,$
     xscale1:!x,$
     pscale2:!p,$
     xscale2:!x,$
     xtitle:'',$
     yscale1:!y,$
     yscale2:!y,$
     ytitle:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}


;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

end
;
;******************************************************************************
;
pro xnormspec_cleanup,xnormspec_base

common xnormspec_state

widget_control, xnormspec_base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin
    
        
endif

end
;
;******************************************************************************
;
pro xnormspec_plotdivspec

common xnormspec_state

plot,*state.d.wave,*state.d.nflux,/xsty,ysty=1,psym=10,$
  yrange=state.p.plot2yrange,xrange=state.p.plot1xrange,$
  xtitle=state.p.xtitle,ytitle='!5Normalized Flux'
plots,!x.crange,[1,1],color=3

state.p.pscale2 = !p
state.p.xscale2 = !x
state.p.yscale2 = !y

end
;
;******************************************************************************
;
pro xnormspec_normalize

common xnormspec_state

z = where(*state.d.normgoodbad eq 1,count)
if count eq 0 then goto,cont

*state.d.coeffs = robustpoly1d(*state.d.wave,*state.d.flux,$
;                               state.r.normorder,3,0.01,$
;                               YERR=*state.d.std_err/*state.d.std_flux,$
                               state.r.normdegree,3,0.01,/GAUSSJ,$
                               IGOODBAD=*state.d.normgoodbad,/SILENT,VAR=var)

ffit  = poly(*state.d.wave,*state.d.coeffs)

*state.d.nflux = *state.d.flux/ffit
tabinv,*state.d.wave,state.p.plot1xrange,idx
state.p.plot2yrange = [min((*state.d.nflux)[idx[0]:idx[1]],max=ymax,/NAN),ymax]

xnormspec_setminmax

cont:

end
;
;******************************************************************************
;
pro xnormspec_plotnormreg

common xnormspec_state

goodbad = *state.d.normgoodbad

z = where(goodbad eq 1,count_good)
if count_good eq 0 then goto, cont

n = n_elements(goodbad)
x = [goodbad-goodbad[1:*],goodbad[n-1]]
y = [goodbad[0],goodbad[1:*]-goodbad]
x[where( x lt 0)] = 0
y[where( y lt 0)] = 0

sum = x+y

z = where(sum eq 1,count)

for i = 0, count/2-1 do begin

    idx1 = z[i*2]
    idx2 = z[i*2+1]
        
    wmin = (*state.d.wave)[idx1]
    wmax = (*state.d.wave)[idx2]
   
    fmin = state.p.plot1yrange[0]+0.1*(state.p.plot1yrange[1]-$
                                       state.p.plot1yrange[0])
    dmin = (*state.d.flux)[idx1]
    dmax = (*state.d.flux)[idx2]
    
    plots,[wmin,wmin],[fmin,dmin], linestyle=2, color=7
    plots,[wmax,wmax],[fmin,dmax], linestyle=2, color=7
    oplot,[wmin,wmax],[fmin,fmin], color=7
    
endfor

cont:

end
;
;******************************************************************************
;
pro xnormspec_plotfit

common xnormspec_state

ffit  = poly(*state.d.wave,*state.d.coeffs)
oplot,*state.d.wave,poly(*state.d.wave,*state.d.coeffs),color=3

end
;
;******************************************************************************
;
pro xnormspec_plotspec

common xnormspec_state

plot,*state.d.wave,*state.d.flux,/xsty,ysty=1,$
  yrange=state.p.plot1yrange,xrange=state.p.plot1xrange,psym=10,$
  xtitle=state.p.xtitle,ytitle=state.p.ytitle

state.p.xscale1 = !x
state.p.yscale1 = !y
state.p.pscale1 = !p

end
;
;******************************************************************************
;
pro xnormspec_plotupdate

common xnormspec_state

wset, state.p.plotwin1_wid
erase
xnormspec_plotspec 
if state.r.normreg then xnormspec_plotnormreg
if state.r.norm then xnormspec_plotfit

wset, state.p.pixmap1_wid
erase
xnormspec_plotspec
if state.r.normreg then xnormspec_plotnormreg
if state.r.norm then xnormspec_plotfit

wset, state.p.plotwin2_wid
erase
if state.r.norm then xnormspec_plotdivspec

wset, state.p.pixmap2_wid
erase
if state.r.norm then xnormspec_plotdivspec




end
;
;******************************************************************************
;
pro xnormspec_setminmax

common xnormspec_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.plot1xrange[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.plot1xrange[1],2)
widget_control, state.w.ymin1_fld[1],$
  set_value=strtrim(state.p.plot1yrange[0],2)
widget_control, state.w.ymax1_fld[1],$
  set_value=strtrim(state.p.plot1yrange[1],2)

widget_control, state.w.ymin2_fld[1],$
  set_value=strtrim(state.p.plot2yrange[0],2)
widget_control, state.w.ymax2_fld[1],$
  set_value=strtrim(state.p.plot2yrange[1],2)

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xnormspec_event, event

common xnormspec_state

widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of

    'Accept': widget_control, event.top, /DESTROY

    'Cancel': begin

        state.r.cancel = 1
        widget_control, event.top, /DESTROY

    end
    'Clear Normalization': begin

        *state.d.normgoodbad=intarr(n_elements(*state.d.wave))
        state.r.norm = 0
        state.r.normreg = 0
        *state.d.nflux = *state.d.flux
        xnormspec_plotupdate
        state.r.cursormode = 'None'
        
    end

    'Keyboard': begin

        case strtrim(event.ch,2) of 
            
;            'c': begin
                
;                state.r.cursormode = 'None'
;                state.p.reg      = [[!values.f_nan,!values.f_nan],$
;                                    [!values.f_nan,!values.f_nan]]
;                xnormspec_plotupdate
            
;            end

            'n': begin

                state.r.cursormode = 'Norm'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                state.r.normreg = 1
                
            end

            'w': begin

                state.p.plot1xrange = [min(*state.d.wave,max=max,/NAN),max]
                state.p.plot1yrange = [0,max(*state.d.flux,/NAN)]
                xnormspec_plotupdate
                xnormspec_setminmax
                
            end

            'z': begin

                state.r.cursormode = 'Zoom'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                
            end
        
            else:

        endcase

    end

    'Normalization Fit Degree': begin

        x = ['1','2','3','4','5','6','7','8','9','10','11','12','13','14','15']
        state.r.normdegree = x[event.index]

    end

    'Normalize Spectrum': begin
        
        xnormspec_normalize
        state.r.norm = 1
        state.r.normreg = 0
        xnormspec_plotupdate

    end
    
    else:

endcase

cont: 

end
;
;******************************************************************************
;
pro xnormspec_plotwinevent1, event

common xnormspec_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale1
!x = state.p.xscale1
!y = state.p.yscale1
x  = event.x/float(state.p.plot1size[0])
y  = event.y/float(state.p.plot1size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin

    case state.r.cursormode of

        'Zoom': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
                
                state.p.reg[*,1] = xy[0:1]
                state.p.plot1xrange   = [min(state.p.reg[0,*],max=max),max]
                state.p.plot1yrange   = [min(state.p.reg[1,*],max=max),max]
                xnormspec_plotupdate
                state.r.cursormode   = 'None'
                state.p.reg      = [[!values.f_nan,!values.f_nan],$
                                    [!values.f_nan,!values.f_nan]]
                xnormspec_setminmax

            endelse

        end
        'Norm': begin

            z = where(finite(state.p.reg) eq 1,count)
            if count eq 0 then state.p.reg[*,0] = xy[0:1]
            if count eq 2 then begin
                
                state.p.reg[*,1] = xy[0:1]
                tabinv,*state.d.wave,state.p.reg[0,*],range
                range = range[sort(range)]
                (*state.d.normgoodbad)[range[0]:range[1]] = 1
                xnormspec_plotupdate
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                
            endif

        end
        else:
        
    endcase

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]
plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE
plots, [0,state.p.plot1size[0]],[event.y,event.y],color=2,/DEVICE


wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]
plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE

if state.r.cursormode eq 'Zoom' then begin

    wset, state.p.plotwin1_wid
    xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
    plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
      linestyle=2,color=2
    plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
      linestyle=2,color=2

endif

;  Update cursor.

;  Update cursor position.

tabinv, *state.d.wave,xy[0],idx
idx = round(idx)
label = 'Cursor X: '+strtrim(xy[0],2)+', Y:'+strtrim(xy[1],2)
label = label+'   Spectrum X: '+strtrim( (*state.d.wave)[idx],2)+$
  ', Y:'+strtrim( (*state.d.flux)[idx],2)

widget_control,state.w.message,set_value=label

cont:
    
end
;
;******************************************************************************
;
pro xnormspec_plotwinevent2, event

common xnormspec_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale2
!x = state.p.xscale2
!y = state.p.yscale2
x  = event.x/float(state.p.plot2size[0])
y  = event.y/float(state.p.plot2size[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 and state.r.cursormode eq 'Zoom' then begin
            
    z = where(finite(state.p.reg) eq 1,count)
    if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
        
        state.p.reg[*,1] = xy[0:1]
        state.p.plot2yrange = [min(state.p.reg[1,*],max=max,/NAN),max]
        xnormspec_plotupdate
        state.r.cursormode   = 'None'
        state.p.reg      = [[!values.f_nan,!values.f_nan],$
                            [!values.f_nan,!values.f_nan]]
        xnormspec_setminmax
        
    endelse 

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]
plots, [event.x,event.x],[0,state.p.plot1size[1]],color=2,/DEVICE

wset, state.p.plotwin2_wid
device, copy=[0,0,state.p.plot2size[0],state.p.plot2size[1],0,0,$
              state.p.pixmap2_wid]
plots, [event.x,event.x],[0,state.p.plot2size[1]],color=2,/DEVICE
plots, [0,state.p.plot2size[0]],[event.y,event.y],color=2,/DEVICE

;  Plot zoom line

if state.r.cursormode eq 'Zoom' then begin

    wset, state.p.plotwin2_wid
    plots,!x.crange,[state.p.reg[1,0],state.p.reg[1,0]],linestyle=2,color=2

endif

cont:

end
;
;******************************************************************************
;
pro xnormspec_minmax,event

common xnormspec_state
widget_control, event.id,  GET_UVALUE = uvalue


case uvalue of 

    'X Min': begin

        xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmin2 = crange(xmin,state.p.plot1xrange[1],'X Min',/KLT,$
                       WIDGET_ID=state.w.xnormspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmin_fld[0],set_value=state.p.plotxrange[0]
            return
            
        endif else state.p.plot1xrange[0] = xmin2

    end
    'X Max': begin

        xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        xmax2 = crange(xmax,state.p.plot1xrange[0],'X Max',/KGT,$
                       WIDGET_ID=state.w.xnormspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control, state.w.xmax_fld[0],set_value=state.p.plotxrange[1]
            return
            
        endif else state.p.plot1xrange[1] = xmax2

    end
    'Y1 Min': begin

        ymin = cfld(state.w.ymin1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot1yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xnormspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin1_fld[0],set_val=state.p.plot1yrange[0]
            return
            
        endif else state.p.plot1yrange[0] = ymin2
        
    end
    'Y1 Max': begin

        ymax = cfld(state.w.ymax1_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot1yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xnormspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax1_fld[0],set_val=state.p.plot1yrange[1]
            return
            
        endif else state.p.plot1yrange[1] = ymax2
        
    end
    'Y2 Min': begin

        ymin = cfld(state.w.ymin2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymin2 = crange(ymin,state.p.plot2yrange[1],'Y Min',/KLT,$
                       WIDGET_ID=state.w.xnormspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymin2_fld[0],set_val=state.p.plot2yrange[0]
            return
            
        endif else state.p.plot2yrange[0] = ymin2
        
    end
    'Y2 Max': begin

        ymax = cfld(state.w.ymax2_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return
        ymax2 = crange(ymax,state.p.plot2yrange[0],'Y Max',/KGT,$
                       WIDGET_ID=state.w.xnormspec_base,CANCEL=cancel)
        if cancel then begin
            
            widget_control,state.w.ymax2_fld[0],set_val=state.p.plot2yrange[1]
            return
            
        endif else state.p.plot2yrange[1] = ymax2
        
    end
    
endcase

xnormspec_plotupdate
xnormspec_setminmax

end
;
;******************************************************************************
;
pro xnormspec_resize, event

common xnormspec_state

widget_control, state.w.xnormspec_base, tlb_get_size = size

;  Window 1

state.p.plot1size[0]=size[0]-state.p.buffer[0]
state.p.plot1size[1]=(size[1]-state.p.buffer[1])*1.4/2.4

widget_control, state.w.plotwin1, draw_xsize=state.p.plot1size[0]
widget_control, state.w.plotwin1, draw_ysize=state.p.plot1size[1]

wdelete,state.p.pixmap1_wid
window, /free, /pixmap,xsize=state.p.plot1size[0],ysize=state.p.plot1size[1]
state.p.pixmap1_wid = !d.window

wset, state.p.plotwin1_wid
device, copy=[0,0,state.p.plot1size[0],state.p.plot1size[1],0,0,$
              state.p.pixmap1_wid]

;  Window 2

state.p.plot2size[0]=size[0]-state.p.buffer[0]
state.p.plot2size[1]=(size[1]-state.p.buffer[1])/2.4

widget_control, state.w.plotwin2, draw_xsize=state.p.plot2size[0]
widget_control, state.w.plotwin2, draw_ysize=state.p.plot2size[1]

wdelete,state.p.pixmap2_wid
window, /free, /pixmap,xsize=state.p.plot2size[0],ysize=state.p.plot2size[1]
state.p.pixmap2_wid = !d.window


xnormspec_plotupdate

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;

;
;******************************************************************************
;
pro xnormspec,wave,flux,nflux,MASK=MASK,YTITLE=ytitle,$
              XTITLE=xtitle,CANCEL=cancel

common xnormspec_state

zparcheck, 'xnormspec', wave, 1, [2,3,4,5], 1, 'Wave' 
zparcheck, 'xnormspec', flux, 2, [2,3,4,5], 1, 'Flux' 

if n_params() lt 2 then begin

    print, 'Syntax - xnormspec,wave,flux,nflux,$'
    print, '                   MASK=mask,YTITLE=ytitle,XTITLE=xtitle'
    return

endif

if not xregistered('xnormspec') then begin
        
    xnormspec_initcommon

;  Build the widget.

    buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
    textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
    
    state.w.xnormspec_base = widget_base(title='Xnormspec', $
                                         /COLUMN,$
                                         /TLB_SIZE_EVENTS)
    
       quit_button = widget_button(state.w.xnormspec_base,$
                                   FONT=buttonfont,$
                                   EVENT_PRO='xnormspec_event',$
                                   VALUE='Cancel',$
                                   UVALUE='Cancel')
       
       state.w.keyboard = widget_text(state.w.xnormspec_base, $
                                      /ALL_EVENTS, $
                                      SCR_XSIZE=1, $
                                      SCR_YSIZE=1, $
                                      UVALUE='Keyboard', $
                                      EVENT_PRO='xnormspec_event',$
                                      VALUE= '')

       val='Cursor X: 0.00000, Y: 0.0000000000   '+$
         'Spectrum X: 0.00000, Y: 0.0000000000'
       state.w.message = widget_text(state.w.xnormspec_base, $
                                     VALUE=val,$
                                     YSIZE=1)

       row_base = widget_base(state.w.xnormspec_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)

          value = ['1','2','3','4','5','6','7','8','9','10','11','12',$
                  '13','14','15']
          fitorder_dl = widget_droplist(row_base,$
                                        FONT=buttonfont,$
                                        TITLE='Fit Degree:',$
                                        VALUE=value,$
                                        EVENT_PRO='xnormspec_event',$
                                        UVALUE='Normalization Fit Degree')
       
          button = widget_button(row_base,$
                                 VALUE='Normalize',$
                                 FONT=buttonfont,$
                                 EVENT_PRO='xnormspec_event',$
                                 UVALUE='Normalize Spectrum')

          button = widget_button(row_base,$
                                 VALUE='Clear Normalization',$
                                 FONT=buttonfont,$
                                 EVENT_PRO='xnormspec_event',$
                                 UVALUE='Clear Normalization')


          button = widget_button(row_base,$
                                 FONT=buttonfont,$
                                 EVENT_PRO='xnormspec_event',$
                                 VALUE='Accept',$
                                 UVALUE='Accept')

       state.w.plotwin1 = widget_draw(state.w.xnormspec_base,$
                                      XSIZE=state.p.plot1size[0],$
                                      YSIZE=state.p.plot1size[1],$
                                      /TRACKING_EVENTS,$
                                      /BUTTON_EVENTS,$
                                      /MOTION_EVENTS,$
                                      EVENT_PRO='xnormspec_plotwinevent1',$
                                      UVALUE='Plot Window 1')
       
       row_base = widget_base(state.w.xnormspec_base,$
                              /ROW)
   
          xmin = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='X Min:',$
                               UVALUE='X Min',$
                               XSIZE=12,$
                               EVENT_PRO='xnormspec_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.xmin_fld = [xmin,textid]
          
          xmax = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='X Max:',$
                               UVALUE='X Max',$
                               XSIZE=12,$
                               EVENT_PRO='xnormspec_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.xmax_fld = [xmax,textid]
          
          ymin = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Min:',$
                               UVALUE='Y1 Min',$
                               XSIZE=12,$
                               EVENT_PRO='xnormspec_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymin1_fld = [ymin,textid]
          
          ymax = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Max:',$
                               UVALUE='Y1 Max',$
                               XSIZE=12,$
                               EVENT_PRO='xnormspec_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymax1_fld = [ymax,textid]
          
          state.w.plotwin2 = widget_draw(state.w.xnormspec_base,$
                                         XSIZE=state.p.plot2size[0],$
                                         YSIZE=state.p.plot2size[1],$
                                         /TRACKING_EVENTS,$
                                         /MOTION_EVENTS,$
                                         /BUTTON_EVENTS,$
                                         EVENT_PRO='xnormspec_plotwinevent2',$
                                         UVALUE='Plot Window 2')
          
       row_base = widget_base(state.w.xnormspec_base,$
                              /ROW)
          
          ymin = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Min:',$
                               UVALUE='Y2 Min',$
                               XSIZE=12,$
                               EVENT_PRO='xnormspec_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymin2_fld = [ymin,textid]
          
          ymax = coyote_field2(row_base,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Y Max:',$
                               UVALUE='Y2 Max',$
                               XSIZE=12,$
                               EVENT_PRO='xnormspec_minmax',$
                               /CR_ONLY,$
                               TEXTID=textid)
          state.w.ymax2_fld = [ymax,textid]

; Get things running.  Center the widget using the Fanning routine.
      
    centertlb,state.w.xnormspec_base
      
    widget_control, state.w.xnormspec_base, /REALIZE
    
    mkct
    
;  Get plotwin ids
    
    widget_control, state.w.plotwin1, GET_VALUE=x
    state.p.plotwin1_wid = x
    widget_control, state.w.plotwin2, GET_VALUE=x
    state.p.plotwin2_wid = x
    
    window, /free, /pixmap,xsize=state.p.plot1size[0],$
      ysize=state.p.plot1size[1]
    state.p.pixmap1_wid = !d.window

    window, /free, /pixmap,xsize=state.p.plot2size[0],$
      ysize=state.p.plot2size[1]
    state.p.pixmap2_wid = !d.window
    
;  Get sizes for things.
    
    widget_geom = widget_info(state.w.xnormspec_base, /geometry)
    state.p.buffer[0]=widget_geom.xsize-state.p.plot1size[0]
    state.p.buffer[1]=widget_geom.ysize-state.p.plot1size[1]-$
      state.p.plot2size[1]

    state.p.ytitle = (n_elements(YTITLE) ne 0) ? ytitle:''
    state.p.xtitle = (n_elements(XTITLE) ne 0) ? xtitle:''
    *state.d.wave  = wave
    *state.d.flux  = flux
    *state.d.nflux = flux
    *state.d.normgoodbad = (n_elements(MASK) eq 0) ? $
      intarr(n_elements(wave)):mask

    state.p.plot1xrange = [min(wave,max=max),max]
    state.p.plot1yrange = [0,max(flux,/NAN)]

    xnormspec_plotupdate
    xnormspec_setminmax
    
; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xnormspec', $
      state.w.xnormspec_base, $
      event_handler='xnormspec_resize'

    nflux = *state.d.nflux
    mask = *state.d.normgoodbad

    cancel = state.r.cancel
    
endif

end






