; NAME:
;     xproinfo
;
; PURPOSE:
;     To display the header of a program in the IDL path
;       
; CALLING SEQUENCE:
;     xproinfo,proname
;
; INPUTS:
;     proname
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     Duh
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xproinfo_event, event

widget_control, event.top, /DESTROY

end
;
;******************************************************************************
;
;-------------------------------Main Program----------------------------------
;
;******************************************************************************
;
pro xproinfo,proc_name

 OS = !VERSION.OS                     ;VMS or Unix operating system

 if (N_params() EQ 0) then begin      ;Prompt for procedure name?
   Proc_Name = ' ' 
   read,'Enter name of procedure for which you want the path: ',Proc_Name
 endif else zparcheck, 'getpro', Proc_Name, 1, 7, 0, 'Procedure name'

 NoPrint = keyword_set(NoPrint)
 DirList = strarr(1)
 ProList = strarr(1)

 fdecomp, Proc_Name, Disk, Dir, Name      ;Don't want file extensions
 Name = strtrim( Name, 2 )  

; Set up separate file and directory seperators for VMS and Unix

 case !VERSION.OS_FAMILY of
 
 'vms':  begin   
           DirSep  = ''
           Name    = strupcase(Name)
           lname = name
           remchar,lname,'*'
        end
 'Windows': DirSep = '\'
 'MacOS': DirSep = ''                 ;Fixed 21-Sep-1995
 else: DirSep = '/'
 
 endcase

 pathdir = expand_path(!PATH,/ARRAY, Count = N_dir)
 cd,current = dir
; Remove duplicate directories  in !PATH but keep original order
 path_dir = [dir]
 for i = 0,N_dir -1 do begin
      test = where(path_dir EQ pathdir[i], Ndup)
      if Ndup EQ 0 then path_dir = [path_dir,pathdir[i]]
 endfor
 N_dir = N_elements(path_dir)

; Loop over each directory in !PATH until procedure name found

   for idir = 0, N_dir-1 do begin

   dir = path_dir[idir]
   if (strmid(Dir,0,1) eq '@') then begin          ;Text Library?

         LibName = strmid( Dir, 1, strlen(Dir)-1 )      ;Remove the "@" symbol
         spawn, 'library /list ' + LibName, List
         if strpos(name,'*') GE 0 then begin
                lfound = where( strpos(list, lname) GE 0, Nfound)
         endif else lfound = where( list EQ name, Nfound)

         if (Nfound GT 0) then begin
           for j=0,Nfound-1 do begin
            DirList = [DirList, Dir]
            ProList = [ProList, name]
            Mess = list[lfound[j]] + ' found in the library  ' + LibName
            message, Mess, /CONT, NOPRINT=NoPrint, /NOPREFIX, /NONAME
          endfor
         endif

   endif else begin                              ;Directory
     found = 0b
      ProsFound = findfile(Dir + DirSep + Name + '.*', COUNT=Nfile)

      if (Nfile ge 1) then begin                     ;Found by FINDFILE?
       for j = 0,nfile-1 do begin
         fdecomp, ProsFound[j], ddisk,ddir,fname,ext
         if ((ext EQ 'pro') or (ext EQ 'sav')) then begin
         found = 1b
         profound = prosfound[j]
         if ext EQ 'pro' then $  
           message,/Con,NoPrint = NoPrint,/NoPrefix, /Noname, $
          'Procedure ' + fname + ' found in directory  ' + disk + Dir $
         else if ext EQ 'sav' then $ 
     message,/Con,NoPrint = NoPrint,/NoPrefix, /Noname, $
          'Save set ' + fname + '.sav found in directory  ' + disk + Dir
         endif
        endfor
         if found then begin
         DirList = [DirList, Dir]
         ProList = [ProList, ProsFound[0]]
         endif
       endif
   endelse

endfor

if (N_elements(DirList) GT 1) then begin
    DirList = DirList[1:*]
    ProList = ProList[1:*]
endif

; At this point !PATH has been searched.  If the procedure has not been found
; (nothing is in DirList) check if it is an intrinsic IDL procedure or function

if (DirList[0] eq '') then begin
    
    funcnames = routine_info(/system,/func)
    test = where ( funcnames EQ strupcase(name), fcount)
    
    funcnames = routine_info(/system)
    test = where ( funcnames EQ strupcase(name), pcount)
    
    if (fcount EQ 0) and (pcount EQ 0) then begin
        if not(NoPrint) then begin
          message, 'Procedure '+Name+' not found in a !PATH directory.', /CONT
         message, 'Check your spelling or search individual directories.', /INF
        endif
    endif else begin 
      DirList = ['INTRINSIC']
      ProList = ['INTRINSIC']
      if not(NoPrint) then begin
         message, 'Procedure ' + Name + ' is an intrinsic IDL procedure.', /CONT
         message, 'No path information available.', /INF
      endif
   endelse

endif else begin

    file = dirlist+'/'+proc_name
    openr,lun,file,/GET_LUN
    
    line = ''
    i = 0
    while not (eof(lun)) do begin
    
        readf,lun,line
        if strmid(line,0,1) eq ';' then begin

            array = (i eq 0) ? line:[[array],[line]]
            i = i + 1

        endif else goto, cont

    endwhile
    cont:
    free_lun, lun
    
    xproinfo_base = widget_base(/COLUMN, $
                                TITLE='Xproinfo')
    
       help_text = widget_text(xproinfo_base, $
                               /SCROLL, $
                               VALUE=array, $
                               XSIZE=80, $
                               YSIZE=30)
       
       quit = widget_button(xproinfo_base,$
                            value='Done',$
                            font=buttonfont)
       
    centertlb,xproinfo_base
           
    widget_control, xproinfo_base, /Realize
        
; Start the Event Loop. This will be a non-blocking program.
        
    XManager, 'xproinfo', $
      xproinfo_base, $
      /No_Block,$
      event_handler='xproinfo_event'

endelse
  
end
