; NAME:
;     xspexdriver
;
; PURPOSE:
;     A widget driver to call xeqwidth, xcompspec, etc.
;       
; CATEGORY:
;     Data analysis
;
; CALLING SEQUENCE:
;     xspexdriver
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Libray
;
; PROCEDURE:
;     Easy
;
; MODIFICATION HISTORY:
;     2001-11-09 - Written by M. Cushing, Institute for Astronomy, UH
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
pro xspexdriver_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Aps': state.r.ap = event.value

    'File Button': begin

        if state.r.path eq '' then begin

            fullpath = dialog_pickfile(dialog_parent=state.w.xspexdriver_base,$
                                       filter='*.fits',get_path=newpath,$
                                       /must_exist)
            if fullpath ne '' then state.r.path = newpath
            
        endif else begin

            fullpath = dialog_pickfile(dialog_parent=state.w.xspexdriver_base,$
                                       filter='*.fits',path=state.r.path,$
                                       get_path=newpath,/must_exist)          
            if fullpath ne '' then state.r.path = newpath

        endelse
        if fullpath ne '' then begin

            widget_control, state.w.file_fld[1], SET_VALUE=strtrim(fullpath)
            setfocus, state.w.file_fld

        endif

    end

    'Launch Program': xspexdriver_launchprogram,state

    'Load Buffer': begin

        state.r.buffer = fix(event.value)
        
        if state.d.(state.r.buffer-1).name ne 'None' then $
          widget_control, state.w.specname_fld[1],$
          SET_VALUE=state.d.(state.r.buffer-1).name else $
                    widget_control, state.w.specname_fld[1],$
          SET_VALUE=''

    end

    'Load Spectrum': xspexdriver_loadspec,state
    
    'Quit': begin

        widget_control, event.top, /destroy
        goto, getout
        
    end

    'Select Buffer': begin

        if state.r.program eq 'Xeqwidth' then begin

            (*state.r.dobuffer)[*] = 0
            (*state.r.dobuffer)[event.value] = 1

        endif else begin
            
            val = (event.select eq 1) ? 1:0
            (*state.r.dobuffer)[event.value] = val

        endelse
        
    end

    'Select Program': begin

        state.r.program = event.value
        xspexdriver_updatebufferlist,state

    end
    else:

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xspexdriver_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xspexdriver_cleanup,xspexdriver_base

widget_control, xspexdriver_base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then state = 0B

end
;
;******************************************************************************
;
pro xspexdriver_launchprogram,state

print, state.r.program
case state.r.program of 

    'Xcentroid': begin

        bidx = where(state.r.loadedbuffer eq 1,count)
        z = where(*state.r.dobuffer eq 1,count)
        xcentroid,*state.d.(total(bidx[z])).wave,$
          *state.d.(total(bidx[z])).flux

    end

    'Xlat': begin

        bidx = where(state.r.loadedbuffer eq 1,count)
        z = where(*state.r.dobuffer eq 1,count)
        yunits = fxpar(*state.d.(total(bidx[z])).hdr,'YUNITS')


        case yunits of 

            0: begin

                val = 0
                print, 'Assuming flux units are W m-2 um-1.'
                
            end
            
            'W m-2 um-1': val = 0
            
            'ergs s-1 cm-2 A-1': val = 1

            else: 

        endcase

        xlat,*state.d.(total(bidx[z])).wave,$
          *state.d.(total(bidx[z])).flux,*state.d.(total(bidx[z])).error,$
          0,val

    end
    'Xcompspec': begin

        bidx = where(state.r.loadedbuffer eq 1,count)
        if count eq 0 then goto, cont

        z = where(*state.r.dobuffer eq 1,count)
        for i = 0, count-1 do begin

            buffers = state.r.loadedbuffer
            
            spec   = [[*state.d.(bidx[z[i]]).wave],$
                      [*state.d.(bidx[z[i]]).flux]]
            key = 'spec'+strtrim(i,2)
            struct = (i eq 0) ? create_struct(key,spec):$
              create_struct(struct,key,spec)

            labels = (i eq 0) ? strtrim(state.d.(bidx[z[i]]).name,2):$
              [labels,strtrim(state.d.(bidx[z[i]]).name,2)]

        endfor
        xcompspec,struct,labels

        cont:
    end

    else:

endcase




end
;
;******************************************************************************
;
pro xspexdriver_loadspec,state

name = cfld(state.w.specname_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

file = cfld(state.w.file_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
file = cfile(file,WIDGET_ID=state.w.xspexdriver_base,CANCEL=cancel)
if cancel then return

;  Read spectra

readspec,file,spec,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
  airmass,CANCEL=cancel

if norders eq 1 then begin
    
    z = where(finite(spec[*,0,0]) eq 1)
    
    state.d.(state.r.buffer-1).name = name
    *state.d.(state.r.buffer-1).wave  = spec[z,0,0]
    *state.d.(state.r.buffer-1).flux  = spec[z,1,0]
    *state.d.(state.r.buffer-1).error = spec[z,2,0]
    *state.d.(state.r.buffer-1).hdr   = hdr
    
    widget_control, state.w.file_fld[1], SET_VALUE=''
    
endif else begin
    
    xpickorder,orders,naps,state.w.xspexdriver_base,porder,pap
    z = where(orders eq porder)
    
    state.d.(state.r.buffer-1).name = name
    *state.d.(state.r.buffer-1).wave = spec[*,0,z*naps+(pap-1)]
    *state.d.(state.r.buffer-1).flux = spec[*,1,z*naps+(pap-1)]
    *state.d.(state.r.buffer-1).hdr  = hdr
    widget_control, state.w.file_fld[1], SET_VALUE=''
    
endelse

state.r.loadedbuffer[state.r.buffer-1] = 1
xspexdriver_updatebufferlist,state

end
;
;******************************************************************************
;
pro xspexdriver_updatebufferlist,state

widget_control, state.w.sbuffer_base,UPDATE=0
widget_control, state.w.sbuffer_bg, /DESTROY

z = where(state.r.loadedbuffer eq 1,count)
buffers = string(z+1,format='(i1.1)')

n = 0
e = 0

if state.r.program eq 'Xlat' then begin
    
    e = 1
    no = 1
    
endif 
if state.r.program eq 'Xcentroid' then begin
    
    e = 1
    no = 1
    
endif 
if state.r.program eq 'Xcompspec' then begin
    
    n = 1
    no = 0
    
endif
state.w.sbuffer_bg = cw_bgroup(state.w.sbuffer_base,$
                               FONT=state.w.buttonfont,$
                               buffers,$
                               NONEXCLUSIVE=n,$
                               EXCLUSIVE=e,$
                               /RETURN_INDEX,$
                               NO_RELEASE=no,$
                               /ROW,$
                               LABEL_LEFT='Select Buffer:',$
                               UVALUE='Select Buffer')

widget_control, state.w.sbuffer_base,UPDATE=1
*state.r.dobuffer = intarr(count)


end
;
;******************************************************************************
;
pro xspexdriver,MODEL=model

;  Set the fonts

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {aps_base:0L,$
     buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     aps_bg:0L,$
     file_fld:[0L,0L],$
     filename_fld:[0L,0L],$
     model_base:0L,$
     orders_bg:0L,$
     orders_base:0L,$
     sbuffer_base:0L,$
     sbuffer_bg:0L,$
     specname_fld:[0L,0L],$
     spex_base:0L,$
     xspexdriver_base:0L}

r = {ap:0,$
     buffer:1,$
     datatype:'Data',$
     dobuffer:ptr_new(fltarr(2)),$
     hdr:ptr_new(fltarr(2)),$
     loadedbuffer:intarr(14),$
     norders:0,$
     naps:0,$
     order:0,$
     orders:ptr_new(fltarr(2)),$
     path:'',$
     program:'Xcompspec',$
     spec:ptr_new(fltarr(2))}

d = {buffer1:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer2:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer3:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer4:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer5:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer6:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer7:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer8:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer9:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer10:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer11:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer12:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer13:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))},$
     buffer14:{name:'None',wave:ptr_new(fltarr(2)),flux:ptr_new(fltarr(2)),$
              error:ptr_new(fltarr(2)),hdr:ptr_new(strarr(2))}}

state = {w:w,r:r,d:d}

;  Build the widget.

state.w.xspexdriver_base = widget_base(title='xspexdriver', $
                                       /COLUMN,$
                                       FRAME=5)

   quit_button = widget_button(state.w.xspexdriver_base,$
                               FONT=buttonfont,$
                               VALUE='Quit',$
                               EVENT_PRO='xspexdriver_event',$
                               UVALUE='Quit')

   box1_base = widget_base(state.w.xspexdriver_base,$
                           /COLUMN,$
                           FRAME=1)

      buffer_bg = cw_bgroup(box1_base,$
                            FONT=buttonfont,$
                            ['1','2','3','4','5','6','7','8','9','10','11',$
                            '12','13','14'],$
                            /EXCLUSIVE,$
                            /RETURN_NAME,$
                            /NO_RELEASE,$
                            /ROW,$
                            LABEL_LEFT='Load Buffer:',$
                            SET_VALUE=0,$
                            UVALUE='Load Buffer')

   if keyword_set(MODEL) then begin

       datatype_bg = cw_bgroup(box1_base,$
                               FONT=buttonfont,$
                               ['Data','Model'],$
                               /EXCLUSIVE,$
                               /RETURN_NAME,$
                               /NO_RELEASE,$
                               /ROW,$
                               LABEL_LEFT='Data Type:',$
                               SET_VALUE=0,$
                               UVALUE='Data Type')

       field = coyote_field2(box1_base,$
                             LABELFONT=buttonfont,$
                             FIELDFONT=textfont,$
                             TITLE='Spectrum Name:',$
                             UVALUE='Spectrum Name',$
                             XSIZE=25,$
                             EVENT_PRO='xspexdriver_event',$
                             /CR_ONLY,$
                             TEXTID=textid)
       state.w.specname_fld = [field,textid]       

       inputs_base = widget_base(box1_base)


          state.w.spex_base = widget_base(inputs_base,$
                                          /COLUMN,$
                                          MAP=1)

             row = widget_base(state.w.spex_base,$
                               /ROW,$
                               /BASE_ALIGN_CENTER)
       
                button = widget_button(row,$
                                       FONT=buttonfont,$
                                       VALUE='FITS File',$
                                       UVALUE='File Button',$
                                       EVENT_PRO='xspexdriver_event')
                
                field = coyote_field2(row,$
                                      LABELFONT=buttonfont,$
                                      FIELDFONT=textfont,$
                                      TITLE=':',$
                                      UVALUE='File Field',$
                                      XSIZE=30,$
                                      EVENT_PRO='xspexdriver_event',$
                                      /CR_ONLY,$
                                      TEXTID=textid)
                state.w.file_fld = [field,textid]               

          state.w.model_base = widget_base(inputs_base,$
                                           /COLUMN,$
                                           MAP=1)

             widget_control, state.w.model_base, MAP=0
      


          
      endif else begin

          field = coyote_field2(box1_base,$
                                LABELFONT=buttonfont,$
                                FIELDFONT=textfont,$
                                TITLE='Spectrum Label:',$
                                UVALUE='Spectrum Name',$
                                XSIZE=25,$
                                EVENT_PRO='xspexdriver_event',$
                                /CR_ONLY,$
                                TEXTID=textid)
          state.w.specname_fld = [field,textid]              
          
          row = widget_base(box1_base,$
                            /ROW,$
                            /BASE_ALIGN_CENTER)
          
          button = widget_button(row,$
                                 FONT=buttonfont,$
                                 VALUE='FITS File',$
                                 UVALUE='File Button',$
                                 EVENT_PRO='xspexdriver_event')
          
          field = coyote_field2(row,$
                                LABELFONT=buttonfont,$
                                FIELDFONT=textfont,$
                                TITLE=':',$
                                UVALUE='File Field',$
                                XSIZE=30,$
                                EVENT_PRO='xspexdriver_event',$
                                /CR_ONLY,$
                                TEXTID=textid)
          state.w.file_fld = [field,textid]               


      endelse

      load_button = widget_button(box1_base,$
                                  FONT=buttonfont,$
                                  VALUE='Load Spectrum',$
                                  UVALUE='Load Spectrum')

   box2_base = widget_base(state.w.xspexdriver_base,$
                           /COLUMN,$
                           FRAME=1)
   
      program_bg = cw_bgroup(box2_base,$
                             FONT=buttonfont,$
                             ['Xcompspec','Xlat','Xcentroid'],$
                             /EXCLUSIVE,$
                             /RETURN_NAME,$
                             /NO_RELEASE,$
                             /ROW,$
                             SET_VALUE=0,$
                             LABEL_LEFT='Select Program:',$
                             UVALUE='Select Program')


      state.w.sbuffer_base = widget_base(box2_base)

         state.w.sbuffer_bg = cw_bgroup(state.w.sbuffer_base,$
                                        FONT=buttonfont,$
                                        ['1','2','3','4','5','6'],$
                                        /NONEXCLUSIVE,$
                                        /RETURN_INDEX,$
                                        /NO_RELEASE,$
                                        /ROW,$
                                        LABEL_LEFT='Select Buffer:',$
                                        UVALUE='Select Buffer')

     launch_button = widget_button(box2_base,$
                                   FONT=buttonfont,$
                                   VALUE='Launch Program',$
                                   UVALUE='Launch Program')
         
; Get things running.  Center the widget using the Fanning routine.

centertlb,state.w.xspexdriver_base

widget_control, state.w.xspexdriver_base, /realize

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xspexdriver', $
  state.w.xspexdriver_base, $
  /No_Block,$
  cleanup = 'xspexdriver_cleanup'

;  Get sizes of things now for resizing

widget_control, state.w.xspexdriver_base, tlb_get_size = result

; Put state variable into the user value of the top level base.

widget_control, state.w.xspexdriver_base, set_uvalue=state, /no_copy

end
