; NAME:
;     xsyntphot
;
; PURPOSE:
;     To flux calibate a SpeX spectrum
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xsyntphot
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     Later
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xsyntphot_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Compute Magnitudes': xsyntphot_computemags,state

    'Load Spectrum': xsyntphot_loadspec,state

    'Filter': begin

    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xsyntphot_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xsyntphot_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld
            setfocus,state.w.spectrum_fld

        endif

    end

    'Path Field': setfocus,state.w.spectrum_fld

    'Plot Filter': begin

        state.p.plotfilter = total(event.value)
        xsyntphot_plotfilter,state


    end
    'Quit': begin

        widget_control, event.top, /DESTROY
        goto, getout
        
    end

    'Spectrum Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        spec = dialog_pickfile(DIALOG_PARENT=state.w.xsyntphot_base,$
                              PATH=path,/MUST_EXIST,FILTER='*.fits')
        if spec eq '' then goto, cont
        widget_control,state.w.spectrum_fld[1], $
          SET_VALUE = strmid(spec[0],strpos(spec,'/',/REVERSE_S)+1)

    end

    'System': begin

        state.r.currentsystem = total(event.index)
        xsyntphot_changesystem,state

    end

    else:

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xsyntphot_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xsyntphot_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin


    ptr_free, state.r.objmags
    ptr_free, state.r.scales
    ptr_free, state.r.usescale

    ptr_free, state.d.spec
    ptr_free, state.d.hdr
    ptr_free, state.d.wvega
    ptr_free, state.d.fvega
    ptr_free, state.d.filtertrans
    ptr_free, state.d.files


endif
state = 0B

end
;
;******************************************************************************
;
pro xsyntphot_changesystem,state

state.r.nfilters = n_elements((state.d.filters).(state.r.currentsystem))

;  Update the plotting buttons

widget_control, state.w.magfiltrow_base,UPDATE=0
widget_control, state.w.magfilt_bg,DESTROY=1

state.w.magfilt_bg = cw_bgroup(state.w.magfiltrow_base,$
                               (state.d.filters).(state.r.currentsystem),$
                               LABEL_LEFT='Plot Filter:',$
                               FONT=state.w.buttonfont,$
                               /ROW,$
                               /RETURN_INDEX,$
                               /NO_RELEASE,$
                               /EXCLUSIVE,$
                               UVALUE='Plot Filter')

widget_control, state.w.magfiltrow_base,UPDATE=1

;  Update the table 

labels = strarr(6)
labels[*] = ' '
labels[0:state.r.nfilters] = $
  ['Units',strtrim(state.d.filters.(state.r.currentsystem),2)]
widget_control, state.w.table,column_labels=labels,use_table_select=-1

widget_control, state.w.table,set_value=strarr(6,6),use_table_select=[0,0,5,5]

;  Load the filter profiles

for i = 0, state.r.nfilters-1 do begin

    rdfloat,state.r.packagepath+'data/'+((*state.d.files).(state.r.currentsystem))[i],w,t
    key = 'filter'+string(i,format='(i2.2)')
    struc = (i eq 0) ? create_struct(key,[[w],[t]]):$
      create_struct(struc,key,[[w],[t]])
    
endfor
*state.d.filtertrans = struc

;  Erase plot window

wset, state.p.plotwin_wid
erase

end
;
;******************************************************************************
;
pro xsyntphot_computemags,state

spec = *state.d.spec

string = strarr(6,6)
string[0,*] = ['microns','percent','microns','microns',state.p.yunits,' ']

for i = 0, state.r.nfilters-1 do begin

    trans = (*state.d.filtertrans).(i)
    result = filtcoverage(spec[*,0],spec[*,1],trans[*,0],trans[*,1])
    if result eq 0 or result eq 1 then begin
        
        syntphot,spec[*,0],spec[*,1],*state.d.wvega,*state.d.fvega,$
          state.r.vegazero[state.r.currentsystem],trans[*,0],trans[*,1],$
          bandwidth,coverage,l_eff,l_ave,flux,mag,CANCEL=cancel
        
        string[i+1,*] = [strtrim(bandwidth,2),$
                         string(coverage,format='(f6.2)'),$
                         strtrim(l_ave,2),$
                         strtrim(l_eff,2),$
                         strtrim(flux,2),$
                         strtrim(mag,2)]

    endif else string[i+1,*] = ['***','***','***','***','***','***']

endfor

widget_control, state.w.table,set_value=string,use_table_select=[0,0,5,5]



end
;
;******************************************************************************
;
pro xsyntphot_loadspec,state

;  Clear table.

widget_control, state.w.table,set_value=strarr(6,6),use_table_select=[0,0,5,5]

;  Get files.

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

spec = cfld(state.w.spectrum_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
spec = cfile(path+spec,WIDGET_ID=state.w.xsyntphot_base,CANCEL=cancel)
if cancel then return

;  Read spectra and load data.

readspec,spec,spec,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,stdairmass

z = where(finite(spec[*,0]) eq 1)


*state.d.spec = spec[z,*]
*state.d.hdr  = hdr

state.p.xunits = xunits
state.p.yunits = yunits

;  Load Vega

restore, state.r.packagepath+'data/vega5.sav'

*state.d.wvega = wvin/10000.
c = 2.99792458e+8

case state.p.yunits of 

    'ergss-1cm-2A-1': *state.d.fvega = fvin

    'ergss-1cm-2Hz-1': *state.d.fvega = fvin*(*state.d.wvega)^2*(1.0e-2/c)

    'Wm-2um-1': *state.d.fvega = fvin*10.

    'Wm-2Hz-1': *state.d.fvega = fvin*(*state.d.wvega)^2*(1.0e-5/c)
    
    'Jy': *state.d.fvega = fvin*(*state.d.wvega)^2*(1.0e21/c) 

endcase

end
;******************************************************************************
;
pro xsyntphot_plotfilter,state

wset, state.p.plotwin_wid

trans = *state.d.filtertrans
trange = [min((trans.(state.p.plotfilter))[*,0],max=max),max]

result = filtcoverage((*state.d.spec)[*,0],(*state.d.spec)[*,1],$
                      reform((trans.(state.p.plotfilter))[*,0]),$
                      reform((trans.(state.p.plotfilter))[*,1]))

if result eq 0 or result eq 1 then begin

    interpspec,(trans.(state.p.plotfilter))[*,0],$
      (trans.(state.p.plotfilter))[*,1],(*state.d.spec)[*,0],rfilter

    plot,(*state.d.spec)[*,0],rfilter,$
      xsty=5,ysty=5,color=5,xrange=trange,psym=10
    ticks = strtrim(findgen(11)*.1,2)
    axis,yaxis=1,yticks=10,ytickname=ticks,yminor=1,color=5

    plot,(*state.d.spec)[*,0],(*state.d.spec)[*,1],/xsty,ysty=9,$
      xrange=trange,/NOERASE,psym=10,xtitle='!5Wavelength (microns)'

endif else begin

    plot, (trans.(state.p.plotfilter))[*,0],(trans.(state.p.plotfilter))[*,1],$
      xsty=1,ysty=1,color=1,xrange=trange,psym=10,/NODATA,$
      xtitle='!5Wavelength (microns)',ytitle='!5Transmission'
    oplot,(trans.(state.p.plotfilter))[*,0],(trans.(state.p.plotfilter))[*,1],$
      color=5

endelse


end
;
;******************************************************************************
;
; ------------------------------Main Program---------------------------------- 
;
;******************************************************************************
;
;
pro xsyntphot

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.
;  p - contains info pertaining to plotting.


last   = strpos(!path,'Spantool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

packagepath = cpath(result,CANCEL=cancel)
if cancel then return

rdphotfile,packagepath+'/data/photsystems.dat',nsystems,systems,filters,$
  files,vegazero
if cancel then return


w = {buttonfont:'-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1',$
     col2_base:0L,$
     magfilt_bg:0L,$
     magfiltrow_base:0L,$
     path_fld:[0L,0L],$
     plotwin:0L,$
     spectrum_fld:[0L,0L],$
     table:0L,$
     textfont:'-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1',$
     xsyntphot_base:0L}

r = {currentsystem:0,$
     nfilters:n_elements(filters.(0)),$
     packagepath:packagepath,$
     vegazero:vegazero}

d = {files:ptr_new(fltarr(2)),$
     filters:filters,$
     filtertrans:ptr_new(fltarr(2)),$
     fvega:ptr_new(fltarr(2)),$
     hdr:ptr_new(fltarr(2)),$
     spec:ptr_new(fltarr(2)),$
     systems:systems,$
     wvega:ptr_new(fltarr(2))}

*d.files = files

p = {plotfilter:0,$
     plotwinsize:[860,400],$
     plotwin_wid:0L,$
     xunits:'',$
     yunits:''}

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

state.w.xsyntphot_base = widget_base(title='Xsyntphot', $
                                    /COLUMN,$
                                    /TLB_SIZE_EVENTS)
    
   quit_button = widget_button(state.w.xsyntphot_base,$
                               FONT=state.w.buttonfont,$
                               EVENT_PRO='xsyntphot_event',$
                               VALUE='Done',$
                               UVALUE='Quit')
       
   row1_base = widget_base(state.w.xsyntphot_base,$
                           /ROW)

      col1_base = widget_base(row1_base,$
                              FRAME=1,$
                              EVENT_PRO='xsyntphot_event',$
                              /COLUMN)

         row = widget_base(col1_base,$
                           /ROW,$
                           /BASE_ALIGN_CENTER)
         
            button = widget_button(row,$
                                   FONT=state.w.buttonfont,$
                                   VALUE='Path',$
                                   UVALUE='Path Button',$
                                   EVENT_PRO='xsyntphot_event')
            
            field = coyote_field2(row,$
                                  LABELFONT=state.w.buttonfont,$
                                  FIELDFONT=state.w.textfont,$
                                  TITLE=':',$
                                  UVALUE='Path Field',$
                                  XSIZE=25,$
                                  EVENT_PRO='xsyntphot_event',$
                                  /CR_ONLY,$
                                  TEXTID=textid)
            state.w.path_fld = [field,textid]    
            
         row = widget_base(col1_base,$
                           /ROW,$
                           /BASE_ALIGN_CENTER)
         
            spec_but = widget_button(row,$
                                     FONT=state.w.buttonfont,$
                                     VALUE='Spectrum',$
                                     UVALUE='Spectrum Button',$
                                     EVENT_PRO='xsyntphot_event')
            
            spec_fld = coyote_field2(row,$
                                     LABELFONT=state.w.buttonfont,$
                                     FIELDFONT=state.w.textfont,$
                                     TITLE=':',$
                                     UVALUE='Spectrum Field',$
                                     XSIZE=18,$
                                     EVENT_PRO='xsyntphot_event',$
                                     /CR_ONLY,$
                                     TEXTID=textid)
            state.w.spectrum_fld = [spec_fld,textid]
            
         load = widget_button(col1_base,$
                              VALUE='Load Spectrum',$
                              FONT=state.w.buttonfont,$
                              EVENT_PRO='xsyntphot_event',$
                              UVALUE='Load Spectrum')

      col2_base = widget_base(row1_base,$
                              FRAME=1,$
                              EVENT_PRO='xsyntphot_event',$
                              /COLUMN)

         label = ['Band Width','Coverage','l_ave','l_eff','Flux','Mag']
         state.w.table = widget_table(col2_base,$
                                      row_labels=label,$
                                      xsize=6,$
                                      column_width=80,$
                                      alignment=1,$
                                      uvalue='Table')

         load = widget_button(col2_base,$
                              VALUE='Compute Magnitudes',$
                              FONT=state.w.buttonfont,$
                              EVENT_PRO='xsyntphot_event',$
                              UVALUE='Compute Magnitudes')


   state.w.magfiltrow_base = widget_base(state.w.xsyntphot_base,$
                                         /ROW,$
                                         FRAME=1,$
                                         EVENT_PRO='xsyntphot_event',$
                                         /BASE_ALIGN_CENTER)

      systems_dl = widget_droplist(state.w.magfiltrow_base,$
                                   FONT=state.w.buttonfont,$
                                   TITLE='Photometric System:',$
                                   VALUE=state.d.systems,$
                                   UVALUE='System')   
         

   
      state.w.magfilt_bg = cw_bgroup(state.w.magfiltrow_base,$
                                     LABEL_LEFT='Plot Filter:',$
                                     FONT=state.w.buttonfont,$
                                     (state.d.filters).(0),$
                                     /ROW,$
                                     /RETURN_INDEX,$
                                     /NO_RELEASE,$
                                     /EXCLUSIVE,$
                                     UVALUE='Plot Filter')
      
   row3_base = widget_base(state.w.xsyntphot_base,$
                           /ROW)

      state.w.plotwin = widget_draw(row3_base,$
                                    xsize=state.p.plotwinsize[0],$
                                    ysize=state.p.plotwinsize[1],$
                                    uvalue='Plot Window')

; Get things running.  Center the widget using the Fanning routine.

mkct
            
centertlb,state.w.xsyntphot_base

widget_control, state.w.xsyntphot_base, /REALIZE
widget_control, state.w.plotwin, get_value = x
state.p.plotwin_wid = x
xsyntphot_changesystem,state


; Start the Event Loop. This will be a non-blocking program.

XManager, 'xsyntphot', $
  state.w.xsyntphot_base, $
  cleanup='xsyntphot_cleanup',$
  event_handler='xsyntphot_resizeevent',$
  /NO_BLOCK

; Put state variable into the user value of the top level base.

widget_control, state.w.xsyntphot_base, set_uvalue=state, /no_copy

end
