; NAME:
;     xtellcor
;
; PURPOSE:
;     To telluric correct SpeX spectra with an A0 V standard star
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xtellcor
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     Telluric corrects spectra using Vacca et al. method.
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xtellcor_helpevent,event

widget_control, event.top, /DESTROY

end
;
;******************************************************************************
;
pro xtellcor_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Additional Output': begin

        if event.value eq 'Telluric' then state.r.telluricoutput=event.select
        if event.value eq 'Vega' then state.r.vegaoutput=event.select

    end

    'B Magnitude Field': setfocus, state.w.vmag_fld

    'Construct Kernel': xtellcor_conkernel,state
    
    'Construct Telluric Spectra': xtellcor_contellspec,state

    'Get Scales': xtellcor_getscales,state

    'Get Shift': xtellcor_getshift,state

    'Help': xtellcor_help,state

    'Load Spectra': xtellcor_loadspec,state

    'Method': begin

        state.r.method = event.value
        
        if event.value eq 'Deconvolution' then widget_control, $
          state.w.box2a_base,MAP=1 else $
          widget_control, state.w.box2a_base, MAP=0
        
    end

    'Object Spectra Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        obj = dialog_pickfile(DIALOG_PARENT=state.w.xtellcor_base,$
                              PATH=path,/MUST_EXIST,FILTER='*.fits')
        if obj eq '' then goto, cont
        widget_control,state.w.objspectra_fld[1], $
          SET_VALUE = strmid(obj[0],strpos(obj,'/',/REVERSE_S)+1)
        setfocus,state.w.objspectra_fld

    end

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select
        
    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xtellcor_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xtellcor_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld
            setfocus,state.w.stdspectra_fld

        endif

    end

    'Plot Object Order': state.r.plotobjorder = $
      (*state.d.objorders)[event.index]

    'Standard Order': state.r.stdorder = (*state.d.stdorders)[event.index]

    'Quit': begin

        widget_control, event.top, /DESTROY
        goto, getout
        
    end

    'Spectrum Units': begin

        case event.index of 

            0: state.r.units = 'ergs s-1 cm-2 A-1'
            1: state.r.units = 'ergs s-1 cm-2 Hz-1'
            2: state.r.units = 'W m-2 um-1'
            3: state.r.units = 'W m-2 Hz-1'
            4: state.r.units = 'Jy'
        
        endcase 

    end

    'Standard Spectra Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        std = dialog_pickfile(DIALOG_PARENT=state.w.xtellcor_base,$
                             PATH=path,/MUST_EXIST,FILTER='*.fits')
        
        if std eq '' then goto, cont
        widget_control,state.w.stdspectra_fld[1],$
          SET_VALUE = strmid(std[0],strpos(std,'/',/REVERSE_S)+1)
        setfocus,state.w.stdspectra_fld

    end

    'Standard Spectra Field': setfocus,state.w.bmag_fld

    'V Magnitude Field': setfocus, state.w.objspectra_fld

    'Write File': xtellcor_writefile,state

    else:

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xtellcor_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xtellcor_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin

    ptr_free, state.r.normmask

    ptr_free, state.d.atmostrans
    ptr_free, state.d.corspec
    ptr_free, state.d.objhdr
    ptr_free, state.d.objorders
    ptr_free, state.d.objspec
    ptr_free, state.d.nstd
    ptr_free, state.d.stdhdr
    ptr_free, state.d.stdorders
    ptr_free, state.d.stdspec
    ptr_free, state.d.tellspec

    ptr_free, state.p.flux
    ptr_free, state.p.wave

endif
state = 0B

end
;
;******************************************************************************
;
pro xtellcor_changeunits,wave,tellspec,tellspec_error,state

c = 2.99792458e+8

case state.r.units of 

    'ergs s-1 cm-2 A-1': 

    'ergs s-1 cm-2 Hz-1': begin

        tellspec       = temporary(tellspec)* wave^2 * (1.0e-2 / c)
        tellspec_error = temporary(tellspec_error)* wave^2 * (1.0e-2 / c)

    end
    'W m-2 um-1': begin 

        tellspec       = temporary(tellspec)*10.
        tellspec_error = temporary(tellspec_error)*10.
            
    end
    'W m-2 Hz-1': begin

        tellspec       = temporary(tellspec)* wave^2 * (1.0e-5 / c)
        tellspec_error = temporary(tellspec_error)* wave^2 * (1.0e-5 / c)

    end
    
    'Jy': begin

        tellspec       = temporary(tellspec)* wave^2 * (1.0e21 / c) 
        tellspec_error = temporary(tellspec_error)* wave^2 * (1.0e21 / c) 

    end

endcase

end
;
;******************************************************************************
;
pro xtellcor_conkernel,state

if state.r.continue lt 1 then begin

    ok = dialog_message('Previous steps not complete.',/ERROR,$
                        DIALOG_PARENT=state.w.xtellcor_base)
    return
    
endif

if state.r.method eq 'Deconvolution' then begin

    idx = where(*state.d.stdorders eq state.r.stdorder)

    xconkern,(*state.d.stdspec)[*,0,idx],(*state.d.stdspec)[*,1,idx],$
      state.d.wvega,state.d.fvega,state.d.cfvega,state.d.cf2vega,$
      state.d.atmos_wave,state.d.atmos_trans,wline,kernel,scale,vshift,$
      YTITLE=ytitle,XTITLE='!5Wavelength (microns)',CANCEL=cancel
    if cancel then return

    state.r.vshift   = vshift
    state.r.scale    = scale

endif else begin

;  Construct the kernels for each order

    case string(state.d.slitw_arc,format='(f3.1)') of 
        
        '0.3': parms = [0.0,0.969822,0.768373]
        
        '0.5': parms = [0.0,1.60602,0.714163]
        
        '0.8': parms = [0.0,2.59172,0.751883]
        
        '1.6': parms = [0.0,5.16317,0.772884]

    endcase
    
    odisp = 1./total( fxpar(*state.d.objhdr,'RES') )
    vdisp = 1e-5

    oslitw_um = odisp*state.d.slitw_pix
    vslitw_pix = oslitw_um/vdisp

    nkernel = round(4.*vslitw_pix)
    if not nkernel mod 2 then nkernel = nkernel + 1
    
    kernel_x = findgen(nkernel)-nkernel/2
    kernel_x = kernel_x * (state.d.slitw_pix/vslitw_pix)
    kernel   = instrprof(parms,kernel_x)
    
    state.r.vshift   = 0.0
    state.r.scale    = 0.8

endelse

*state.d.kernel  = kernel
state.r.continue = 2       

end
;
;******************************************************************************
;
pro xtellcor_contellspec,state

if state.r.continue lt 3 then begin
    
    ok = dialog_message('Previous steps not complete.',/error,$
                        dialog_parent=state.w.xtellcor_base)
    return
    
endif

norders = fxpar(*state.d.objhdr,'NORDERS')
match,*state.d.stdorders,*state.d.objorders,stdidx

*state.d.tellspec = (*state.d.stdspec)[*,*,stdidx]
*state.d.vegaspec = (*state.d.stdspec)[*,*,stdidx]
(*state.d.vegaspec)[*,2,*] = 1.0

for i = 0, norders-1 do begin

    print, 'Constructing Telluric Correction Spectrum for Order '+$
      strtrim((*state.d.objorders)[i],2)+'...'
    z = where(*state.d.stdorders eq (*state.d.objorders)[i],count)

    telluric, (*state.d.stdspec)[*,0,z],(*state.d.stdspec)[*,1,z],$
      (*state.d.stdspec)[*,2,z],state.r.vmag,(state.r.bmag-state.r.vmag),$
      *state.d.kernel,(*state.r.scales)[*,z],state.d.swvega,$
      state.d.fvega,state.d.cfvega,state.d.cf2vega,tellcor,tellcor_error,scvega
    
    xtellcor_changeunits,(*state.d.stdspec)[*,0,z],tellcor,tellcor_error,state
    (*state.d.tellspec)[*,1,i] = tellcor
    (*state.d.tellspec)[*,2,i] = tellcor_error
    (*state.d.vegaspec)[*,1,i] = scvega

endfor

cont:

state.r.continue = 4


end
;
;******************************************************************************
;
pro xtellcor_getscales,state

if state.r.continue lt 2 then begin
    
    ok = dialog_message('Previous steps not complete.',/error,$
                        dialog_parent=state.w.xtellcor_base)
    return
    
endif

vshift = state.r.vshift
vrot = 0
print, state.r.method
if state.r.method ne 'Deconvolution' then begin

    delvarx,vshift
    delvarx,vrot

endif
xgetscales,*state.d.stdspec,*state.d.stdorders,state.r.vmag,$
  (state.r.bmag-state.r.vmag),state.d.wvega,state.d.fvega,state.d.cfvega,$
  state.d.cf2vega,*state.d.kernel,*state.d.objspec,*state.d.objorders,$
  state.d.objnaps,state.d.atmos_wave,state.d.atmos_trans,state.r.scale,$
  scales,CANCEL=cancel,Vshift=vshift,Vrot=vrot

if not cancel then begin
    
    *state.r.scales = scales
    state.r.continue = 3
    
    state.r.vshift = vshift
    z = state.r.vshift/2.99792458e+5 
    state.d.swvega  = state.d.wvega*(1+z)        

endif

end
;
;******************************************************************************
;
pro xtellcor_help,state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xtellcor_help') then begin

    help_base = widget_base(GROUP_LEADER=state.w.xtellcor_base, $
                            /COLUMN, $
                            TITLE='Xtellcor Help')
    
    openr, lun, filepath('xtellcor_helpfile.txt',$
                         ROOT_DIR=state.r.packagepath),/get_lun
    nlines = numlines(filepath('xtellcor_helpfile.txt',$
                               ROOT_DIR=state.r.packagepath))
    array = strarr(nlines)
    readf, lun, array
    free_lun, lun
    
    help_text = widget_text(help_base, $
                            /scroll, $
                            value = array, $
                            xsize = 70, $
                            ysize = 24)
    
    quit = widget_button(help_base,$
                         value='Done',$
                         font=buttonfont,$
                         uvalue='Done Help')
      
    centertlb,help_base

    widget_control, help_base, /Realize
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xtellcor_help', $
      help_base, $
      /No_Block,$
      event_handler='xtellcor_helpevent'
    
endif

end
;
;******************************************************************************
;
pro xtellcor_loadspec,state

;  Get files.

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

std = cfld(state.w.stdspectra_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
std = cfile(path+std,WIDGET_ID=state.w.xtellcor_base,CANCEL=cancel)
if cancel then return

bmag = cfld(state.w.bmag_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
vmag = cfld(state.w.vmag_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return

obj = cfld(state.w.objspectra_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
obj = cfile(path+obj,WIDGET_ID=state.w.xtellcor_base,CANCEL=cancel)
if cancel then return

;  Read spectra and load data.

readspec,std,std,stdhdr,stdobsmode,start,stop,stdnorders,stdnaps,stdorders,$
  stdxunits,stdyunits,slith_pix,slith_arc,slitw_pix,slitw_arc,stdairmass

readspec,obj,obj,objhdr,objobsmode,start,stop,objnorders,objnaps,objorders,$
  objxunits,objyunits,slith_pix,slith_arc,slitw_pix,slitw_arc,objairmass

*state.d.stdspec   = std
*state.d.stdorders = stdorders
*state.d.stdhdr    = stdhdr
state.d.slitw_pix  = slitw_pix
state.d.slitw_arc  = slitw_arc
state.d.res        = float(fxpar(stdhdr,'RES'))
*state.d.disp      = float(mcfxpar(stdhdr,'DPO*'))

*state.d.objspec   = obj
*state.d.objorders = objorders
*state.d.objhdr    = objhdr
state.d.objnaps    = objnaps

state.r.bmag = bmag
state.r.vmag = vmag
state.r.vshift = 0.0

state.r.xunits = stdxunits
state.r.yunits = stdyunits

state.r.stdorder = stdorders[0]
state.r.plotobjorder = objorders[0]

;  Compute airmass difference between obj and std.

widget_control, state.w.message,SET_VALUE=$
  'Std Airmass:'+string(stdairmass,format='(f7.4)')+', Obj Airmass:'+$
  string(objairmass,format='(f7.4)')+', (Std-Obj) Airmass: '+$
  string((stdairmass-objairmass),format='(f7.4)')

widget_control, state.w.objorder_dl,set_value=string(objorders,format='(i2.2)')
widget_control, state.w.stdorder_dl,set_value=string(stdorders,format='(i2.2)')

state.r.continue=1

end
;
;******************************************************************************
;
pro xtellcor_getshift,state

if state.r.continue lt 4 then begin
    
    ok = dialog_message('Previous steps not complete.',/error,$
                        dialog_parent=state.w.xtellcor_base)
    return
    
endif

naps = fxpar(*state.d.objhdr,'NAPS')
z    = where((*state.d.objorders) eq state.r.plotobjorder)

obj_wave = (*state.d.objspec)[*,0,z*naps]
obj_flux = (*state.d.objspec)[*,1,z*naps]

z        = where((*state.d.stdorders) eq state.r.plotobjorder)
tel_wave = (*state.d.tellspec)[*,0,z]
tel_flux = 1./(*state.d.tellspec)[*,1,z]

interpspec,tel_wave,tel_flux,obj_wave,new_flux
xfindshift,obj_wave,obj_flux,new_flux,shift,INITSHIFT=state.r.shift,$
  CANCEL=cancel

if not cancel then state.r.shift = shift

state.r.continue=5

end
;
;******************************************************************************
;
pro xtellcor_writefile,state

if state.r.continue lt 5 then begin
    
    ok = dialog_message('Previous steps not complete.',/error,$
                        dialog_parent=state.w.xtellcor_base)
    return
    
endif

obj = cfld(state.w.objoname_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return
    
stdfile = cfld(state.w.stdspectra_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

;  Construct Telluric correction spectra and corrected obj spectra

ndat    = n_elements((*state.d.objspec)[*,0,0])
norders = fxpar(*state.d.objhdr,'NORDERS')
naps    = fxpar(*state.d.objhdr,'NAPS')

*state.d.corspec = *state.d.objspec
tellspec         = *state.d.objspec

for i = 0, naps-1 do begin

    for j = 0, norders-1 do begin

        k = j*naps+i
        z = where((*state.d.stdorders) eq (*state.d.objorders)[j])

;  Interpolate telluric spectrum onto object wavelength sampling.

        interpspec,(*state.d.tellspec)[*,0,z],(*state.d.tellspec)[*,1,z],$
          (*state.d.objspec)[*,0,k],nflux,nerror,$
          YAERROR=(*state.d.tellspec)[*,2,z]

;  Now shift spectrum.

        x = findgen(n_elements((*state.d.tellspec)[*,0,z]))
        interpspec,x+state.r.shift,nflux,x,nnflux,nnerror,YAERROR=nerror

        tellspec[*,1,k] = nnflux
        tellspec[*,2,k] = nnerror

        (*state.d.corspec)[*,1,k] = nnflux*(*state.d.objspec)[*,1,k]
        (*state.d.corspec)[*,2,k] = $
          sqrt( nnflux^2 * (*state.d.objspec)[*,2,k]^2 + $
          (*state.d.objspec)[*,1,k]^2 * nnerror^2 )
        
    endfor

endfor

if state.r.telluricoutput then begin
    
    fxhmake,hdr,tellspec
    
;  Get info from obj hdr.
    
    xunits    = fxpar(*state.d.objhdr,'XUNITS')
    orders    = fxpar(*state.d.objhdr,'ORDERS')
    norders   = fxpar(*state.d.objhdr,'NORDERS')
    start     = fxpar(*state.d.objhdr,'START')
    stop      = fxpar(*state.d.objhdr,'STOP')
    ps        = fxpar(*state.d.objhdr,'PLATE_SC')
    obsmode   = fxpar(*state.d.objhdr, 'GRAT')
    naps      = 1
    
    std       = fxpar(*state.d.stdhdr,'OBJECT')

    match,*state.d.stdorders,*state.d.objorders,idx
    
    history = 'These telluric correction spectra were constructed from the '+$
      'spectra '+strtrim(stdfile,2)+'.  The B and V magnitudes of '+$
      strtrim(std,2)+' are '+strtrim(state.r.bmag,2)+', and '+$
      strtrim(state.r.vmag,2)+$
      ' respectively.  The velocity shift of Vega is '+$
      strtrim(state.r.vshift,2)+' km s-1.  The telluric '+$
      'correction spectra were then shifted by '+strtrim(state.r.shift,2)+$
      ' pixels.'

    fxaddpar,hdr,'IRAFNAME',strtrim(obj+'_tellspec.fits',2)
    fxaddpar,hdr,'YUNITS',state.r.units
    fxaddpar,hdr,'XUNITS',xunits
    fxaddpar,hdr,'ORDERS',orders
    fxaddpar,hdr,'NORDERS',norders
    fxaddpar,hdr,'START',start
    fxaddpar,hdr,'STOP',stop
    fxaddpar,hdr,'PLATE_SC',ps
    fxaddpar,hdr,'OBJECT','Tellluric Spectrum'
    fxaddpar,hdr,'GRAT',obsmode
    fxaddpar,hdr,'NAPS',1
    
    length = strlen(history)
    loop = ceil(float(length)/65.)
    for i = 0, loop-1 do begin
        
        hist = strmid(history,65*i,65)
        fxaddpar,hdr,'HISTORY',hist
        
    endfor
    
    if state.r.fitsoutput then writefits,path+obj+'_tellspec.fits',$
      *state.d.tellspec,hdr

    if state.r.textoutput then begin
        
        npix = fxpar(hdr,'NAXIS1')
        openw,lun,path+obj+'tellspec_.txt', /get_lun
        
        for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]
        
        for i = 0, npix-1 do begin
            
            printf, lun,  strjoin( reform((*state.d.tellspec)[i,*,*],$
                                          3*norders),'  ' )
            
        endfor
        close, lun
        free_lun, lun
        
    endif
    print,'Wrote the telluric correction spectrum to '+strtrim(path+obj,2)+$
      '_tellcor'
    
endif

if state.r.vegaoutput then begin

    fxhmake,hdr,*state.d.vegaspec
    
;  Get info from obj hdr.
    
    xunits    = fxpar(*state.d.objhdr,'XUNITS')
    orders    = fxpar(*state.d.objhdr,'ORDERS')
    norders   = fxpar(*state.d.objhdr,'NORDERS')
    start     = fxpar(*state.d.objhdr,'START')
    stop      = fxpar(*state.d.objhdr,'STOP')
    ps        = fxpar(*state.d.objhdr,'PLATE_SC')
    obsmode   = fxpar(*state.d.objhdr, 'GRAT')
    naps      = 1
    
    std       = fxpar(*state.d.stdhdr,'OBJECT')

    match,*state.d.stdorders,*state.d.objorders,idx

    fxaddpar,hdr,'IRAFNAME',strtrim(obj+'_vega.fits',2)
    fxaddpar,hdr,'YUNITS',state.r.units
    fxaddpar,hdr,'XUNITS',xunits
    fxaddpar,hdr,'ORDERS',orders
    fxaddpar,hdr,'NORDERS',norders
    fxaddpar,hdr,'START',start
    fxaddpar,hdr,'STOP',stop
    fxaddpar,hdr,'PLATE_SC',ps
    fxaddpar,hdr,'OBJECT','Tellluric Spectrum'
    fxaddpar,hdr,'GRAT',obsmode
    fxaddpar,hdr,'NAPS',1
    
    if state.r.fitsoutput then writefits,path+obj+'_vega.fits',$
      *state.d.vegaspec,hdr

    if state.r.textoutput then begin
        
        npix = fxpar(hdr,'NAXIS1')
        openw,lun,path+obj+'vega_.txt', /get_lun
        
        for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]
        
        for i = 0, npix-1 do begin
            
            printf, lun,  strjoin( reform((*state.d.vegaspec)[i,*,*],$
                                          3*norders),'  ' )
            
        endfor
        close, lun
        free_lun, lun
        
    endif
    print,'Wrote the Vega spectrum to '+strtrim(path+obj,2)+'_vega'
    


endif

;  Write the corrected spectrum to disk.

norders = fxpar(*state.d.objhdr,'NORDERS')
hdr = *state.d.objhdr
fxaddpar,hdr,'IRAFNAME',strtrim(obj+'.fits',2)
fxaddpar,hdr,'YUNITS',state.r.units

history = 'This spectrum was telluric corrected using the telluric '+$
  'correction spectra '+strtrim(path+obj,2)+'_tellcor.fits'

length = strlen(history)
loop = ceil(float(length)/65.)
for i = 0, loop-1 do begin
    
    hist = strmid(history,65*i,65)
    fxaddpar,hdr,'HISTORY',hist
    
endfor

if state.r.fitsoutput then writefits, path+obj+'.fits', *state.d.corspec,$
  hdr

if state.r.textoutput then begin
    
    npix = fxpar(hdr,'NAXIS1')
    openw,lun,path+obj+'.txt', /get_lun
    
    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]
    
    for i = 0, npix-1 do begin
        
        printf, lun,  strjoin( reform((*state.d.corspec)[i,*,*],$
                                      3*norders),'  ' )
        
    endfor
    close, lun
    free_lun, lun
    
endif    

print,'Wrote the corrected spectrum to '+strtrim(path+obj,2)


orders = long( strsplit( fxpar(*state.d.objhdr,'ORDERS'), ',', /extract) )

xvspec,*state.d.corspec,n_elements(orders),1,ORDERS=orders,$
  XUNITS=state.r.xunits,YUNITS=state.r.yunits,HDR=hdr



end
;
;******************************************************************************
;
; ------------------------------Main Program---------------------------------- 
;
;******************************************************************************
;
pro xtellcor

;  Startup

print, ' '
print, 'Loading Vega model and atmospheric transmission data...'
print, ' '

last  = strpos(!path,'Spantool')
first = strpos(!path,':',last,/REVERSE_SEARCH)
packagepath = strmid(!path,first+1,last-first+7)

;  Get atmosphere file

restore, packagepath+'/data/atrans.sav'

;  Get Vega file

restore, packagepath+'/data/vega5.sav'

;  Set the fonts

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.
;  p - contains info pertaining to plotting.

w = {bmag_fld:[0,0],$
     box2a_base:0L,$
     message:0L,$
     objoname_fld:[0L,0L],$
     objorder_dl:0L,$
     objspectra_fld:[0,0],$
     path_fld:[0L,0L],$
     stdspectra_fld:[0L,0L],$
     stdorder_dl:0L,$
     vmag_fld:[0,0],$
     xtellcor_base:0L}

r = {bmag:0.,$
     continue:0L,$
     fitsoutput:1,$
     method:'IP',$
     packagepath:packagepath,$
     plotobjorder:0,$
     stdorder:0,$
     scale:0.,$
     scales:ptr_new(fltarr(2)),$
     shift:0.,$
     spec:'',$
     textoutput:0,$
     vegaoutput:0,$
     telluricoutput:1,$
     units:'ergs s-1 cm-2 A-1',$
     vmag:0.,$
     vshift:0.,$
     wline:0.,$
     xunits:'',$
     yunits:''}

d = {airmass:'',$
     atmos_wave:awave,$
     atmos_trans:atrans,$
     disp:ptr_new(fltarr(2)),$
     kernel:ptr_new(fltarr(2)),$
     corspec:ptr_new(fltarr(2)),$
     objhdr:ptr_new(fltarr(2)),$
     objorders:ptr_new(fltarr(2)),$
     objnaps:0,$
     objspec:ptr_new(fltarr(2)),$
     nstd:ptr_new(fltarr(2)),$
     res:0.,$
     slitw_arc:0.,$
     slitw_pix:0.,$
     stdhdr:ptr_new(fltarr(2)),$
     stdorders:ptr_new(fltarr(2)),$
     stdspec:ptr_new(fltarr(2)),$
     tellspec:ptr_new(fltarr(2)),$
     fvega:fvin,$
     wvega:wvin/10000.,$
     swvega:wvin/10000.,$
     vegaspec:ptr_new(fltarr(2)),$
     cfvega:fcvin,$
     cf2vega:fc2vin}

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d}

state.w.xtellcor_base = widget_base(title='Xtellcor', $
                                    /COLUMN)
    
   quit_button = widget_button(state.w.xtellcor_base,$
                               FONT=buttonfont,$
                               EVENT_PRO='xtellcor_event',$
                               VALUE='Done',$
                               UVALUE='Quit')
       
   state.w.message = widget_text(state.w.xtellcor_base, $
                                 VALUE='',$
                                 YSIZE=1)


   row_base = widget_base(state.w.xtellcor_base,$
                          /ROW)

      col1_base = widget_base(row_base,$
                              EVENT_PRO='xtellcor_event',$
                              /COLUMN)
      
         box1_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
            
            label = widget_label(box1_base,$
                                 VALUE='1.  Load Spectra',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)
            
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
                  
               button = widget_button(row,$
                                      FONT=buttonfont,$
                                      VALUE='Path',$
                                      UVALUE='Path Button')
               
               field = coyote_field2(row,$
                                     LABELFONT=buttonfont,$
                                     FIELDFONT=textfont,$
                                     TITLE=':',$
                                     UVALUE='Path Field',$
                                     XSIZE=25,$
                                     /CR_ONLY,$
                                     TEXTID=textid)
               state.w.path_fld = [field,textid]    
               
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               std_but = widget_button(row,$
                                       FONT=buttonfont,$
                                       VALUE='Std Spectra',$
                                       UVALUE='Standard Spectra Button')
               
               std_fld = coyote_field2(row,$
                                       LABELFONT=buttonfont,$
                                       FIELDFONT=textfont,$
                                       TITLE=':',$
                                       UVALUE='Standard Spectra Field',$
                                       XSIZE=18,$
                                       VALUE='cspectra418-423.fits',$
                                       /CR_ONLY,$
                                       TEXTID=textid)
               state.w.stdspectra_fld = [std_fld,textid]
               
            row = widget_base(box1_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               mag = coyote_field2(row,$
                                   LABELFONT=buttonfont,$
                                   fieldfont=textfont,$
                                   TITLE='Std Mag (B,V):',$
                                   UVALUE='B Magnitude Field',$
                                   XSIZE=6,$
                                   VALUE='7',$
                                   /CR_ONLY,$
                                   TEXTID=textid)
               state.w.bmag_fld = [mag,textid]
               
               mag = coyote_field2(row,$
                                   LABELFONT=buttonfont,$
                                   fieldfont=textfont,$
                                   UVALUE='V Magnitude Field',$
                                   XSIZE=6,$
                                   VALUE='7',$
                                   TITLE='',$
                                   /CR_ONLY,$
                                   TEXTID=textid)
               state.w.vmag_fld = [mag,textid]
               
            row = widget_base(box1_base,$
                              /row,$
                              /base_align_center)
            
               obj_but = widget_button(row,$
                                       font=buttonfont,$
                                       VALUE='Obj Spectra',$
                                       UVALUE='Object Spectra Button',$
                                       EVENT_PRO='xtellcor_event')
               
               obj_fld = coyote_field2(row,$
                                       LABELFONT=buttonfont,$
                                       fieldfont=textfont,$
                                       TITLE=':',$
                                       UVALUE='Object Spectra Field',$
                                       VALUE='cspectra408-417.fits',$
                                       XSIZE=18,$
                                       /CR_ONLY,$
                                       TEXTID=textid)
               state.w.objspectra_fld = [obj_fld,textid]
                     
            load = widget_button(box1_base,$
                                 VALUE='Load Spectra',$
                                 FONT=buttonfont,$
                                 UVALUE='Load Spectra')

         box2_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /FRAME)
            
            label = widget_label(box2_base,$
                                 VALUE='2.  Construct Convolution Kernel',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)

            method_bg = cw_bgroup(box2_base,$
                                  font=buttonfont,$
                                  ['Deconvolution','IP'],$
                                  /ROW,$
                                  /RETURN_NAME,$
                                  /NO_RELEASE,$
                                  /EXCLUSIVE,$
                                  LABEL_LEFT='Method:',$
                                  UVALUE='Method',$
                                  SET_VALUE=1)

            row = widget_base(box2_base)
            
               state.w.box2a_base = widget_base(row,$
                                                /ROW,$
                                                /BASE_ALIGN_CENTER,$
                                                MAP=1)

                  state.w.stdorder_dl = widget_droplist(state.w.box2a_base,$
                                                        FONT=buttonfont,$
                                                        TITLE='Order:',$
                                                        VALUE='01',$
                                                       UVALUE='Standard Order')
                  widget_control, state.w.box2a_base,MAP=0

            button = widget_button(box2_base,$
                                   VALUE='Construct Kernel',$
                                   UVALUE='Construct Kernel',$
                                   FONT=buttonfont)
            
      col2_base = widget_base(row_base,$
                              EVENT_PRO='xtellcor_event',$
                              /COLUMN)


         box3_base = widget_base(col2_base,$
                                 /COLUMN,$
                                 /FRAME)
            
            label = widget_label(box3_base,$
                                 VALUE='3.  Construct Telluric Spectra',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)

            getshifts = widget_button(box3_base,$
                                      VALUE='Get Scales',$
                                      UVALUE='Get Scales',$
                                      FONT=buttonfont)

            value =['ergs s-1 cm-2 A-1','ergs s-1 cm-2 Hz-1',$
                    'W m-2 um-1','W m-2 Hz-1','Jy']
            units_dl = widget_droplist(box3_base,$
                                       font=buttonfont,$
                                       title='Units:',$
                                       value=value,$
                                       uvalue='Spectrum Units')

            constructspec = widget_button(box3_base,$
                                          VALUE='Construct Telluric Spectra',$
                                          UVALUE='Construct Telluric Spectra',$
                                          FONT=buttonfont)

         box4_base = widget_base(col2_base,$
                                 /COLUMN,$
                                 /FRAME)

            label = widget_label(box4_base,$
                                 VALUE='4.  Determine Shift',$
                                 FONT=buttonfont,$
                                 /ALIGN_LEFT)

            row = widget_base(box4_base,$
                              /ROW,$
                              /BASE_ALIGN_CENTER)
            
               state.w.objorder_dl = widget_droplist(row,$
                                                     FONT=buttonfont,$
                                                     TITLE='Order:',$
                                                     VALUE='01',$
                                                   UVALUE='Plot Object Order')
            
               shift = widget_button(row,$
                                     VALUE='Get Shift',$
                                     UVALUE='Get Shift',$
                                     FONT=buttonfont)

          box5_base = widget_base(col2_base,$
                                  /COLUMN,$
                                  /FRAME)

             label = widget_label(box5_base,$
                                  VALUE='5.  Write File',$
                                  FONT=buttonfont,$
                                  /ALIGN_LEFT)
            
            outformat_bg = cw_bgroup(box5_base,$
                                     font=buttonfont,$
                                     ['FITS','Text'],$
                                     /row,$
                                     /return_name,$
                                     /no_release,$
                                     /nonexclusive,$
                                     label_left='Output Format:',$
                                     UVALUE='Output Format',$
                                     set_VALUE=[1,0])

            oname = coyote_field2(box5_base,$
                                  labelfont=buttonfont,$
                                  fieldfont=textfont,$
                                  title='Object File:',$
                                  event_pro = 'xtellcor_event',$
                                  uvalue='Object File Oname',$
                                  /cr_only,$
                                  xsize=18,$
                                  textID=textid)
            state.w.objoname_fld = [oname,textid]
            

            addoutput_bg = cw_bgroup(box5_base,$
                                     font=buttonfont,$
                                     ['Telluric','Vega'],$
                                     /row,$
                                     /return_name,$
                                     /nonexclusive,$
                                     label_left='Additional:',$
                                     UVALUE='Additional Output',$
                                     set_VALUE=[1,0])

            write = widget_button(box5_base,$
                                  value='Write File',$
                                  uvalue='Write File',$
                                  font=buttonfont)

   help = widget_button(state.w.xtellcor_base,$
                        value='Help',$
                        uvalue='Help',$
                        font=buttonfont)
      
; Get things running.  Center the widget using the Fanning routine.
            
centertlb,state.w.xtellcor_base
widget_control, state.w.xtellcor_base, /REALIZE
mkct

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xtellcor', $
  state.w.xtellcor_base, $
  cleanup='xtellcor_cleanup',$
  /NO_BLOCK

widget_control, state.w.xtellcor_base, set_uvalue=state, /no_copy

end
