; NAME:
;     xtellcor_basic
;
; PURPOSE:
;     To telluric correct SpeX spectra
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xtellcor_basic
;
; INPUTS:
;     None
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     None
;
; PROCEDURE:
;     To telluric correct SpeX spectra by dividing by a standard star
;     and restoring the continuum with a blackbody
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xtellcor_basic_event,event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Construct Spectra': xtellcor_basic_tellcor,state

    'Object Spectra Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        obj = dialog_pickfile(DIALOG_PARENT=state.w.xtellcor_basic_base,$
                              PATH=path,/MUST_EXIST,FILTER='*.fits')
        if obj eq '' then goto, cont
        widget_control,state.w.objspectra_fld[1], $
          SET_VALUE = strmid(obj[0],strpos(obj,'/',/REVERSE_S)+1)
        setfocus,state.w.objspectra_fld

    end

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select
        
    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTORY,$
                              DIALOG_PARENT=state.w.xtellcor_basic_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xtellcor_basic_base,$
                         CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld

        endif

    end

    'Quit': begin

        widget_control, event.top, /DESTROY
        goto, getout
        
    end

    'Restore Continuum': begin

        state.r.restore = event.value
        if event.value eq 'Yes' then begin

            widget_control, state.w.temp_base, MAP=1
            widget_control, state.w.dl_base, MAP=1

        endif
        if event.value eq 'No'  then begin

            widget_control, state.w.temp_base, MAP=0
            widget_control, state.w.dl_base, MAP=0

        endif

    end

    'Spectrum Units': begin

        case event.index of 

            0: state.r.units = 'ergs s-1 cm-2 A-1'
            1: state.r.units = 'ergs s-1 cm-2 Hz-1'
            2: state.r.units = 'W m-2 um-1'
            3: state.r.units = 'W m-2 Hz-1'
            4: state.r.units = 'Jy'
        
        endcase 

    end

    'Standard Spectra Button': begin

        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        std = dialog_pickfile(DIALOG_PARENT=state.w.xtellcor_basic_base,$
                             PATH=path,/MUST_EXIST,FILTER='*.fits')
        
        if std eq '' then goto, cont
        widget_control,state.w.stdspectra_fld[1],$
          SET_VALUE = strmid(std[0],strpos(std,'/',/REVERSE_S)+1)
        setfocus,state.w.stdspectra_fld

    end


endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xtellcor_basic_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
;
;******************************************************************************
;
pro xtellcor_basic_changeunits,wave,spec,spec_error,state

c = 2.99792458e+8

case state.r.units of 

    'ergs s-1 cm-2 A-1': 

    'ergs s-1 cm-2 Hz-1': begin

        spec       = temporary(spec)* wave^2 * (1.0e-2 / c)
        spec_error = temporary(spec_error)* wave^2 * (1.0e-2 / c)

    end
    'W m-2 um-1': begin 

        spec       = temporary(spec)*10.
        spec_error = temporary(spec_error)*10.
            
    end
    'W m-2 Hz-1': begin

        spec       = temporary(spec)* wave^2 * (1.0e-5 / c)
        spec_error = temporary(spec_error)* wave^2 * (1.0e-5 / c)

    end
    
    'Jy': begin

        spec       = temporary(spec)* wave^2 * (1.0e21 / c) 
        spec_error = temporary(spec_error)* wave^2 * (1.0e21 / c) 

    end

endcase

end
;
;******************************************************************************
;
pro xtellcor_basic_cleanup,base

widget_control, base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin

endif
state = 0B

end
;
;******************************************************************************
;
pro xtellcor_basic_tellcor,state
;  Get files.

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

stdfile = cfld(state.w.stdspectra_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
stdfile = cfile(path+stdfile,WIDGET_ID=state.w.xtellcor_basic_base,$
                CANCEL=cancel)
if cancel then return

objfile = cfld(state.w.objspectra_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
objfile = cfile(path+objfile,WIDGET_ID=state.w.xtellcor_basic_base,$
                CANCEL=cancel)
if cancel then return

oname = cfld(state.w.objoname_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

readspec,stdfile,std,stdhdr,stdobsmode,start,stop,stdnorders,stdnaps,$
  stdorders,stdxunits,stdyunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
  stdairmass

readspec,objfile,obj,objhdr,objobsmode,start,stop,objnorders,objnaps,$
  objorders,objxunits,objyunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
  objairmass

if state.r.restore eq 'Yes' then begin
    
    temp = cfld(state.w.temp_fld,4,/EMPTY,CANCEL=cancel)
    if cancel then return

endif 

for i = 0,objnaps-1 do begin

    for j = 0,objnorders-1 do begin
        
        k = j*objnaps+i
        z = where(stdorders eq objorders[j])

;  Interpolate telluric spectrum onto object wavelength sampling.

        interpspec,std[*,0,z],std[*,1,z],obj[*,0,k],std_flux,std_error,$
          YAERROR=std[*,2,z]
        
        tell_flux  = 1.0/std_flux
        tell_error = (1.0/std_flux^2) * std_error

        corspec       = obj[*,1,k]*tell_flux
        corspec_error = sqrt(tell_flux^2 * obj[*,2,k]^2 + $
                             obj[*,1,k]^2 * tell_error^2)
        
        if state.r.restore eq 'Yes' then begin

            bbflux = PLANCK(obj[*,0,k]*10000.,temp)
            corspec       = temporary(corspec) * bbflux
            corspec_error = temporary(corspec_error) * bbflux
            
        endif

        xtellcor_basic_changeunits,obj[*,0,k],corspec,corspec_error,state
        
        obj[*,1,k] = corspec
        obj[*,2,k] = corspec_error

    endfor

endfor

;  Write the corrected spectrum to disk.

hdr = objhdr
fxaddpar,hdr,'IRAFNAME',strtrim(oname+'.fits',2)
fxaddpar,hdr,'YUNITS','F_lambda

history = 'This spectrum was telluric corrected using the standard star'+$
  'spectra '+strtrim(stdfile,2)+'.'

length = strlen(history)
loop = ceil(float(length)/65.)
for i = 0, loop-1 do begin
    
    hist = strmid(history,65*i,65)
    fxaddpar,hdr,'HISTORY',hist
    
endfor

if state.r.fitsoutput then writefits, path+oname+'.fits',$
  obj,hdr

if state.r.textoutput then begin
    
    npix = fxpar(hdr,'NAXIS1')
    openw,lun,path+oname+'.txt', /get_lun
    
    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]
    
    for i = 0, npix-1 do begin
        
        printf, lun,  strjoin( reform(obj[i,*,*],3*objnorders),'  ' )
        
    endfor
    close, lun
    free_lun, lun
    
endif    

xvspec,obj,objnorders,objnaps,ORDERS=objorders,$
  XUNITS='microns',YUNITS='F_lambda',HDR=hdr








end
;
;******************************************************************************
;
; ------------------------------Main Program---------------------------------- 
;
;******************************************************************************
;
;
pro xtellcor_basic

last   = strpos(!path,'Spantool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
packagepath = strmid(!path,first+1,last-first+7)

;  Set the fonts

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

w = {bmag_fld:[0L,0L],$
     dl_base:0L,$
     objoname_fld:[0L,0L],$
     objspectra_fld:[0L,0L],$
     path_fld:[0L,0L],$
     stdspectra_fld:[0L,0L],$
     temp_base:0L,$
     temp_fld:[0L,0L],$
     xtellcor_basic_base:0L,$
     vmag_fld:[0L,0L]}

r = {fitsoutput:1,$
     packagepath:packagepath,$
     restore:'Yes',$
     textoutput:0,$
     temp:0.,$
     units:'ergs s-1 cm-2 A-1'}


state = {w:w,r:r}

state.w.xtellcor_basic_base = widget_base(title='Xtellcor', $
                                          EVENT_PRO='xtellcor_basic_event',$
                                          /COLUMN)
    
   quit_button = widget_button(state.w.xtellcor_basic_base,$
                               FONT=buttonfont,$
                               VALUE='Done',$
                               UVALUE='Quit')
   
   box1_base = widget_base(state.w.xtellcor_basic_base,$
                           /COLUMN,$
                           /FRAME)
            
      label = widget_label(box1_base,$
                           VALUE='1.  Load Spectra',$
                           FONT=buttonfont,$
                           /ALIGN_LEFT)
      
      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
      
         button = widget_button(row,$
                                FONT=buttonfont,$
                                VALUE='Path',$
                                UVALUE='Path Button')
               
         field = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE=':',$
                               UVALUE='Path Field',$
                               XSIZE=25,$
                                     /CR_ONLY,$
                               TEXTID=textid)
         state.w.path_fld = [field,textid]    
         
      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
      
         std_but = widget_button(row,$
                                 FONT=buttonfont,$
                                 VALUE='Std Spectra',$
                                 UVALUE='Standard Spectra Button')
         
         std_fld = coyote_field2(row,$
                                 LABELFONT=buttonfont,$
                                 FIELDFONT=textfont,$
                                 TITLE=':',$
                                 UVALUE='Standard Spectra Field',$
                                 XSIZE=18,$
                                       /CR_ONLY,$
                                 TEXTID=textid)
         state.w.stdspectra_fld = [std_fld,textid]

      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
                  
      row = widget_base(box1_base,$
                        /row,$
                        /base_align_center)
      
      obj_but = widget_button(row,$
                              font=buttonfont,$
                              VALUE='Obj Spectra',$
                              UVALUE='Object Spectra Button')
      
      obj_fld = coyote_field2(row,$
                              LABELFONT=buttonfont,$
                              fieldfont=textfont,$
                              TITLE=':',$
                              UVALUE='Object Spectra Field',$
                              XSIZE=18,$
                              /CR_ONLY,$
                              TEXTID=textid)
      state.w.objspectra_fld = [obj_fld,textid]

   box2_base = widget_base(state.w.xtellcor_basic_base,$
                           /COLUMN,$
                           /FRAME)
   
      convolve_bg = cw_bgroup(box2_base,$
                              font=buttonfont,$
                              ['Yes','No'],$
                              /ROW,$
                              /RETURN_NAME,$
                              /NO_RELEASE,$
                              /EXCLUSIVE,$
                              LABEL_LEFT='Restore Continuum:',$
                              UVALUE='Restore Continuum',$
                              SET_VALUE=0)

      state.w.temp_base = widget_base(box2_base,$
                                      /ROW,$
                                      /BASE_ALIGN_CENTER)
      
         fld = coyote_field2(state.w.temp_base,$
                             LABELFONT=buttonfont,$
                             fieldfont=textfont,$
                             TITLE='BB Temp:',$
                             UVALUE='BB Temp Field',$
                             XSIZE=10,$
                             /CR_ONLY,$
                             TEXTID=textid)
         state.w.temp_fld = [fld,textid]         
         
         label = widget_label(state.w.temp_base,$
                              VALUE='K',$
                              FONT=buttonfont)

      state.w.dl_base = widget_base(box2_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)

         value =['ergs s-1 cm-2 A-1','ergs s-1 cm-2 Hz-1',$
                 'W m-2 um-1','W m-2 Hz-1','Jy']
         units_dl = widget_droplist(state.w.dl_base,$
                                    font=buttonfont,$
                                    title='Units:',$
                                    value=value,$
                                    uvalue='Spectrum Units')

   box3_base = widget_base(state.w.xtellcor_basic_base,$
                           /COLUMN,$
                           /FRAME)

      outformat_bg = cw_bgroup(box3_base,$
                               font=buttonfont,$
                               ['FITS','Text'],$
                               /row,$
                               /return_name,$
                               /no_release,$
                               /nonexclusive,$
                               label_left='Output Format:',$
                               UVALUE='Output Format',$
                               set_VALUE=[1,0])
      
      oname = coyote_field2(box3_base,$
                            labelfont=buttonfont,$
                            fieldfont=textfont,$
                            title='Object File:',$
                            event_pro = 'xtellcor_event',$
                            uvalue='Object File Oname',$
                            /cr_only,$
                            xsize=18,$
                            textID=textid)
      state.w.objoname_fld = [oname,textid]
      

   write = widget_button(state.w.xtellcor_basic_base,$
                            value='Construct Spectra',$
                            uvalue='Construct Spectra',$
                            font=buttonfont)
      


; Get things running.  Center the widget using the Fanning routine.

mkct            
centertlb,state.w.xtellcor_basic_base
widget_control, state.w.xtellcor_basic_base, /REALIZE


; Start the Event Loop. This will be a non-blocking program.

XManager, 'xtellcor_basic', $
  state.w.xtellcor_basic_base, $
  cleanup='xtellcor_basic_cleanup',$
  /NO_BLOCK

widget_control, state.w.xtellcor_basic_base, set_uvalue=state, /no_copy


end
